<?php
/**
* Interface pour les datastores, récupération et stokage des objects 
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Interface pour les datastores, récupération et stokage des objects
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage DataStore 
* @author     Pascal Martin 
*/ 
interface com_borealis_acl_datastore_DataStore extends com_borealis_foundation_util_Singleton
{
    /**
    * Delete in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    function deleteByID(com_borealis_acl_object_Base $pobjBase);

    /**
    * Update name in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object   
    * @param string                       $pstrName Nouveau nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    function update(com_borealis_acl_object_Base $pobjBase, $pstrName); 
    
    /**
    * Insert in datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException 
    * 
    * @return int ID
    */
    function insert($pstrType, $pstrName);
    
    /**
    * Return id from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @return int ID
    */
    function selectByName($pstrType, $pstrName);   
    
    /**
    * Return name from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Identificateur de l'action 
    * 
    * @return string Name
    */
    function selectByID($pstrType, $piID);
    
    /**
    * Return id and name from datastore of all object
    * 
    * @param string $pstrType Type de l'objet
    * 
    * @return ArrayObject
    */
    function selectAll($pstrType);
    
    /**
    * Insert relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function insertRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction);
    
    /**
    * Remove relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function deleteRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction);
    
    /**
    * Return id and name of all action linked to the ressource
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Target ressource
    * 
    * @return ArrayObject
    */
    function selectRessourceAction(com_borealis_acl_object_Ressource $pobjRessource);
    
    /**
    * Insert member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    function insertRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername);
    
    /**
    * Delete member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    function deleteRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername); 
    
    /**
    * Return list of member register in this role
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role  
    * 
    * @return ArrayObject of Username string
    */
    function selectRoleMember(com_borealis_acl_object_Role $pobjRole); 
    
    /**
    * Insert permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function insertRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction);
    
    /**
    * Delete permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    function deleteRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction);          
    
    /**
    * Return id and name of all action linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    function selectRolePermissionAction(com_borealis_acl_object_Role $pobjRole);
    
    /**
    * Return id and name of all ressource linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    function selectRolePermissionRessource(com_borealis_acl_object_Role $pobjRole);
    
    /**
    * Vérifie si l'utilisateur a le doit sur l'action de la ressource demander
    * 
    * @param string                            $pstrUsername  Utilisateur
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return bool true or false
    */
    function isAuthorized($pstrUsername,com_borealis_acl_object_Ressource $pobjRessource,com_borealis_acl_object_Action $pobjAction);
}
  
?>
