<?php
/**
* Classe qui représente une ressource du acl 
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Classe qui représente une ressource du acl
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage Object  
* @author     Pascal Martin 
*/ 
class com_borealis_acl_object_Ressource extends com_borealis_acl_object_Base
{
    /**
    * Create new ressource and return
    * 
    * @param string $pstrName Nom de la ressource
    * 
    * @return com_borealis_acl_object_Ressource
    */
    public static function create($pstrName)
    {    
        return parent::create('Ressource', $pstrName);
    }
    
    /**
    * Return ressource if exist else null
    * 
    * @param string $pstrName Nom de la ressource
    * 
    * @return com_borealis_acl_object_Ressource
    */
    public static function getByName($pstrName)
    {
        return parent::getByName('Ressource', $pstrName);               
    }   
    
    /**
    * Return ressource if exist else null
    * 
    * @param int $piID Indentificateur de la ressource
    * 
    * @return com_borealis_acl_object_Ressource
    */
    public static function getByID($piID)
    {
        return parent::getByID('Ressource', $piID);        
    } 
    
    /**
    * Return all ressource
    * 
    * @return ArrayObject of com_borealis_acl_object_Ressource 
    */
    public static function getAll()
    {
        return parent::getAll('Ressource');  
    }
    
    // ----------------------------
    // Custom to this object
    // ----------------------------
    
    /**
    * Insert relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Action $pobjAction Action
    * 
    * @return void
    */
    public function addAction(com_borealis_acl_object_Action $pobjAction)
    {
        com_borealis_acl_datastore_DataStoreFactory::getDataStore()->insertRessourceAction($this, $pobjAction);
    }
    
    /**
    * Delete relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Action $pobjAction Action
    * 
    * @return void
    */
    public function removeAction(com_borealis_acl_object_Action $pobjAction)
    {
        com_borealis_acl_datastore_DataStoreFactory::getDataStore()->deleteRessourceAction($this, $pobjAction);                                    
    }
    
    /**
    * Return list of ressource action
    * 
    * @return ArrayObject of com_borealis_acl_object_Action
    */
    public function getActions()
    {
        return com_borealis_acl_object_Action::getRessourceAction($this); 
    }
    
    /**
    * Return list of role permission ressource
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target Role
    * 
    * @return ArrayObject of com_borealis_acl_object_Ressource
    */
    public static function getRolePermissionRessource(com_borealis_acl_object_Role $pobjRole)
    {
        $objArrayInfos = com_borealis_acl_datastore_DataStoreFactory::getDataStore()->selectRolePermissionRessource($pobjRole);    
        return self::getArrayObjectFromArrayIDName('Ressource', $objArrayInfos);
    }
    
}
  
?>
