<?php  
/**
* Classe qui reprsente une session sso josso
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
  
/**
* Classe qui reprsente une session sso josso
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage Session 
* @author     Pascal Martin 
*/
class com_borealis_security_session_JossoSSOSession extends com_borealis_security_session_SSOSession
{
    private $m_objArrayToken = array(); 
    
    /**
    * Construit une nouvelle instance de com_borealis_security_session_JossoSSOSession  
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    function __construct($pstrUsername) 
    {
        parent::__construct($pstrUsername, 'JOSSO');
    } 
    
    /**
    * Return session token
    * 
    * @return string 
    */
    public function getToken()
    {
        $strToken = "";
        foreach ($this->m_objArrayToken as $strKey => $strValue)
        {
            if ($strToken !== '')
            {
                $strToken .= ',';     
            }
            $strToken .= $strKey.':'.$strValue;   
        } 
        return $strToken;      
    }
    
    /**
    * Add token into session
    * 
    * @param string $pstrSecurityDomain Security domain
    * @param string $pstrToken          Token
    * 
    * @return void
    */
    public function addToken($pstrSecurityDomain, $pstrToken)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrSecurityDomain);
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrToken);
        
        $this->m_objArrayToken[$pstrSecurityDomain] = $pstrToken; 
    } 
    
    /**
    * Return token for the desire security domain
    * 
    * @param string $pstrSecurityDomain Security domain
    * 
    * @return string
    */
    public function getTokenBySecurityDomain($pstrSecurityDomain)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrSecurityDomain); 
        
        if (array_key_exists($pstrSecurityDomain, $this->m_objArrayToken))
        {
            return  $this->m_objArrayToken[$pstrSecurityDomain];
        }
        else
        {
            return null;
        }        
    }   
    
    /**
    * Make JossoSSOSession from string
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrToken    Token
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    public static function fromToken($pstrUsername, $pstrToken)
    {
        $objSession = new com_borealis_security_session_JossoSSOSession($pstrUsername);
        $objArrayToken = explode(',', $pstrToken);
        foreach ($objArrayToken as $strValue)
        {
            $objArrayPart = explode(':', $strValue);  
            if (count($objArrayPart) !== 2)
            {
                throw new com_borealis_foundation_exception_AssertException("Bad token");    
            }
            $objSession->addToken($objArrayPart[0], $objArrayPart[1]);
        }
        return $objSession;
    }
}   
    
?>