<?php
/**
* Classe qui reprsente un utilisateur authentifier
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Classe qui reprsente un utilisateur authentifier
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage User  
* @author     Pascal Martin 
*/
class com_borealis_security_user_User
{
    private $m_objSession; 
    private $m_objArrayRoles;
    private $m_objArrayProperties;
    
    /**
    * Construit une nouvelle instance de com_borealis_security_user_User 
    * 
    * @param com_borealis_security_session_Session $pobjSession         Session
    * @param ArrayObject                           $pobjArrayRoles      Liste de role
    * @param ArrayObject                           $pobjArrayProperties Liste de proprit
    * 
    * @return com_borealis_security_user_User
    */
    function __construct(com_borealis_security_session_Session $pobjSession, ArrayObject $pobjArrayRoles, ArrayObject $pobjArrayProperties) 
    {    
        $this->m_objSession = $pobjSession;
        $this->m_objArrayRoles = $pobjArrayRoles; 
        $this->m_objArrayProperties = $pobjArrayProperties; 
    }

    /**
    * Return user session
    * 
    * @return com_borealis_security_session_Session
    */
    public function getSession()
    {
        return $this->m_objSession;
    }

    /**
    * Return roles list
    * 
    * @return ArrayObject of com_borealis_security_user_Role 
    */
    public function getRoles()
    {
        return $this->m_objArrayRoles;
    }
    
    /**
    * Return properties list
    * 
    * @return ArrayObject of com_borealis_security_user_Property 
    */
    public function getProperties()
    {
        return $this->m_objArrayProperties; 
    }
   
    /**
    * Return username
    * 
    * @return string
    */
    public function getUsername()
    {
        return $this->m_objSession->getUsername();
    }
    
    /**
    * Find role by name
    * 
    * @param string $pstrName Role name
    * 
    * @return com_borealis_security_user_Role
    */
    public function findRoleByName($pstrName)
    {
        return self::findByName($this->m_objArrayRoles->getIterator(), $pstrName);        
    }
    
    /**
    * Find property by name
    *
    * @param string $pstrName Property name
    * 
    * @return com_borealis_security_user_Property 
    */
    public function findPropertyByName($pstrName)
    {
        return self::findByName($this->m_objArrayProperties->getIterator(), $pstrName);        
    }
    
    /**
    * Find property by value return only first find
    * 
    * @param string $pstrValue Property value
    * 
    * @return com_borealis_security_user_Property
    */
    public function findPropertyByValue($pstrValue)
    {
        return self::findByValue($this->m_objArrayProperties->getIterator(), $pstrValue); 
    }
    
    /**
    * Find property by value return all property find
    * 
    * @param string $pstrValue Property value
    * 
    * @return ArrayObject of com_borealis_security_user_Property
    */
    public function findPropertiesByValue($pstrValue)
    {
        return self::findsByValue($this->m_objArrayProperties->getIterator(), $pstrValue); 
    }
    
    /**
    * Find in ArrayObject object where getName() return desired value
    *   
    * @param ArrayIterator $pobjIterator Iterateur
    * @param strring       $pstrName     Name  
    * 
    * @return object 
    */
    private static function findByName(ArrayIterator $pobjIterator, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        return self::findBy($pobjIterator, 'getName', $pstrName);    
    }
   
    /**
    * Find in ArrayObject object where getValue() return desired value
    *   
    * @param ArrayIterator $pobjIterator Iterateur
    * @param strring       $pstrValue    Value  
    * 
    * @return ArrayObject of object 
    */ 
    private static function findsByValue(ArrayIterator $pobjIterator, $pstrValue)
    {
        
        return self::findsBy($pobjIterator, 'getValue', $pstrValue);
    }
    
    /**
    * Find in ArrayObject object where getValue() return desired value, only first find
    *   
    * @param ArrayIterator $pobjIterator Iterateur
    * @param strring       $pstrValue    Value  
    * 
    * @return object 
    */ 
    private static function findByValue(ArrayIterator $pobjIterator, $pstrValue)
    {        
        return self::findBy($pobjIterator, 'getValue', $pstrValue);
    }
    
    /**
    * Find in ArrayObject object where $pstrMethod return desired $pstrValue, only first find 
    *   
    * @param ArrayIterator $pobjIterator Iterateur
    * @param strring       $pstrMethod   Method Name
    * @param strring       $pstrValue    Value  
    * 
    * @return object 
    */ 
    private static function findBy(ArrayIterator $pobjIterator, $pstrMethod, $pstrValue)
    {
        com_borealis_foundation_util_Assert::assertIsString($pstrValue);
         
        while ($pobjIterator->valid()) 
        {
            $strObjValue = call_user_func(array($pobjIterator->current(), $pstrMethod));
            if (strcmp($strObjValue, $pstrValue) === 0)
            {
                return $pobjIterator->current();
            }
            $pobjIterator->next();
        }  
   
        return null;    
    }
    
    /**
    * Find in ArrayObject object where $pstrMethod return desired $pstrValue 
    *   
    * @param ArrayIterator $pobjIterator Iterateur
    * @param strring       $pstrMethod   Method Name
    * @param strring       $pstrValue    Value  
    * 
    * @return ArrayObject of object 
    */ 
    private static function findsBy(ArrayIterator $pobjIterator, $pstrMethod, $pstrValue)
    {
        com_borealis_foundation_util_Assert::assertIsString($pstrValue);
         
        $objArrayResult = new ArrayObject();
        
        while ($pobjIterator->valid()) 
        {
            $strObjValue = call_user_func(array($pobjIterator->current(), $pstrMethod));
            if (strcmp($strObjValue, $pstrValue) === 0)
            {
                $objArrayResult->append($pobjIterator->current());
            }
            $pobjIterator->next();
        }  
   
        return $objArrayResult;      
    }
}   
    
?>