<?php
/**
* Récupère les informations de la base de donnée     
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations de la base de donnée
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage DataStore  
* @author     Pascal Martin 
*/ 
class com_borealis_acl_datastore_WebServicesDataStore implements com_borealis_acl_datastore_DataStore
{
    
    private static $s_strDataTypeInteger    = null;    
    //private static $s_strDataTypeDecimal    = null;   
    private static $s_strDataTypeString     = null;       
    //private static $s_strDataTypeDate       = null;      
    //private static $s_strDataTypeTime       = null;      
    //private static $s_strDataTypeTimestamp  = null;   
    
    /**
    * @var com_borealis_acl_datastore_WebServicesDataStore
    */
    private static $s_objInstance = null; 
    
    /**
    * Construit une nouvelle instance de com_borealis_acl_datastore_WebServicesDataStore 
    */
    private function __construct()
    {    
        if (self::$s_strDataTypeString == null)
        {
            self::$s_strDataTypeString     = self::getWebServiceMethodReturnValue('getDataTypeString');     
        }
        if (self::$s_strDataTypeInteger == null)
        {
            self::$s_strDataTypeInteger     = self::getWebServiceMethodReturnValue('getDataTypeInteger');     
        }
        //self::$s_strDataTypeDecimal    = self::getWebServiceMethodReturnValue('getDataTypeDecimal');   
        //self::$s_strDataTypeDate       = self::getWebServiceMethodReturnValue('getDataTypeDate');      
        //self::$s_strDataTypeTime       = self::getWebServiceMethodReturnValue('getDataTypeTime');      
        //self::$s_strDataTypeTimestamp  = self::getWebServiceMethodReturnValue('getDataTypeTimestamp');
    }
    
    // @codeCoverageIgnoreStart
    /**
    * Destructeur
    */
    public function __destruct()  
    {    
        
    }
    // @codeCoverageIgnoreEnd 
    
    /**
    * return instanceof com_borealis_acl_datastore_WebServicesDataStore 
    * 
    * @return com_borealis_acl_datastore_WebServicesDataStore 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null)
        {
            self::$s_objInstance = new com_borealis_acl_datastore_WebServicesDataStore();
        }
        return self::$s_objInstance;
    } 
    
    
    
    /**
    * Delete in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    public function deleteByID(com_borealis_acl_object_Base $pobjBase)
    {
        $strQuery = self::getQueryByObject($pobjBase, 'PREPARE_DELETE_BY_ID');
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjBase->getID());         
        self::executeUpdate($strQuery, $objParams);        
    }

    /**
    * Update name in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object   
    * @param string                       $pstrName Nouveau nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    public function update(com_borealis_acl_object_Base $pobjBase, $pstrName)
    {   
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName); 
        
        $strQuery = self::getQueryByObject($pobjBase, 'PREPARE_UPDATE');  
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrName);  
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjBase->getID());
        self::executeUpdate($strQuery, $objParams);
    } 
    
    /**
    * Insert in datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException 
    * 
    * @return int ID
    */
    public function insert($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_INSERT');     
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrName);
        self::executeUpdate($strQuery, $objParams);
        
        return self::selectByName($pstrType, $pstrName); 
    }
    
    /**
    * Return id from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @return int ID
    */
    public function selectByName($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);  
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_BY_NAME');     
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrName);
        $objResultXML = self::executeQuery($strQuery, $objParams);
        
        if ($objResultXML == null || $objResultXML->getName() != "results" || sizeof($objResultXML) <= 0)
        {
            return null;   
        }
        else
        {
            return (int)$objResultXML->row->id; 
        }
    }   
    
    /**
    * Return name from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Identificateur de l'action 
    * 
    * @return string Name
    */
    public function selectByID($pstrType, $piID)
    {
        com_borealis_foundation_util_Assert::assertIsInteger($piID);
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_BY_ID');     
        $objParams = array();                                       
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $piID);
        $objResultXML = self::executeQuery($strQuery, $objParams);
        
        if ($objResultXML == null || $objResultXML->getName() != "results" || sizeof($objResultXML) <= 0)
        {
            return null;   
        }
        else
        {
            return (String)$objResultXML->row->name; 
        }   
    }
    
    
    /**
    * Return id and name from datastore of all object
    * 
    * @param string $pstrType Type de l'objet
    * 
    * @return ArrayObject
    */
    public function selectAll($pstrType)
    {    
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_ALL'); 
           
        return $this->getArrayIDName($strQuery);   
    }
   
    /**
    * Insert relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function insertRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareInsertRessourceAction();      
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRessource->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjAction->getID());
        self::executeUpdate($strQuery, $objParams); 
    }
    
    /**
    * Remove relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function deleteRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareDeleteRessourceAction();         
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRessource->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjAction->getID());
        self::executeUpdate($strQuery, $objParams);  
    }
    
    /**
    * Return id and name of all action linked to the ressource
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Target ressource
    * 
    * @return ArrayObject
    */
    public function selectRessourceAction(com_borealis_acl_object_Ressource $pobjRessource)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareSelectRessourceAction();        
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRessource->getID());
        
        return $this->getArrayIDName($strQuery, $objParams);  
    }
    
    /**
    * Insert member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    public function insertRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareInsertRoleMember();          
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrUsername);
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID());
        self::executeUpdate($strQuery, $objParams); 
    }
    
    /**
    * Delete member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    public function deleteRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareDeleteRoleMember();            
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrUsername);
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID());
        self::executeUpdate($strQuery, $objParams);
    }
    
    /**
    * Return list of member register in this role
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role  
    * 
    * @return ArrayObject of Username string
    */
    public function selectRoleMember(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareSelectRoleMember();            
        $objParams = array();
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID());
        $objResultXML = self::executeQuery($strQuery, $objParams);
        
        $objArrayResult = new ArrayObject();
        if ($objResultXML != null)
        {
            if ($objResultXML->getName() == "results")
            {
                foreach ($objResultXML->children() as $objRow)
                {
                    if ($objRow->getName() == "row")
                    {
                        $objArrayResult->append((String)$objRow->username);
                    }
                }
            }
        }
             
        return $objArrayResult;   
    }
    
    /**
    * Insert permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function insertRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareInsertRolePermission();            
        $objParams = array();                                          
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRessource->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjAction->getID());
        self::executeUpdate($strQuery, $objParams); 
    }
    
    /**
    * Delete permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function deleteRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareDeleteRolePermission();              
        $objParams = array();                                          
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRessource->getID());
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjAction->getID());
        self::executeUpdate($strQuery, $objParams); 
    }
    
    /**
    * Return id and name of all action linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    public function selectRolePermissionAction(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareSelectRolePermissionAction();             
        $objParams = array();                                          
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID()); 
        self::executeQuery($strQuery, $objParams);
           
        return $this->getArrayIDName($strQuery, $objParams);    
    }
    
    /**
    * Return id and name of all ressource linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    public function selectRolePermissionRessource(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareSelectRolePermissionRessource();               
        $objParams = array();                                          
        self::addParameter(&$objParams, self::$s_strDataTypeInteger, $pobjRole->getID()); 
        self::executeQuery($strQuery, $objParams);
            
        return $this->getArrayIDName($strQuery, $objParams);
    }
   
    /**
    * Vérifie si l'utilisateur a le doit sur l'action de la ressource demander
    * 
    * @param string                            $pstrUsername  Utilisateur
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return bool true or false
    */
    public function isAuthorized($pstrUsername,com_borealis_acl_object_Ressource $pobjRessource,com_borealis_acl_object_Action $pobjAction)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getDataStorePrepareSelectAuthorized();                     
        $objParams = array();                                          
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pstrUsername);                                      
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pobjRessource->getID());                                      
        self::addParameter(&$objParams, self::$s_strDataTypeString, $pobjAction->getID()); 
        $objResultXML = self::executeQuery($strQuery, $objParams);
        
        return sizeof($objResultXML) > 0;        
    }
    
    // --------------------------------------------------------------------------
    // --- Private static
    // --------------------------------------------------------------------------
    
    /**
    * Return id and name of all row in the result query  
    * 
    * @param mixed $pstrQuery  Query to execute
    * @param mixed $pobjParams Array of parameters to pass to the query
    * 
    * @return ArrayObject
    */
    private function getArrayIDName($pstrQuery, $pobjParams = null)
    {    
        $objResultXML = self::executeQuery($pstrQuery, $pobjParams);
        
        $objArrayResult = new ArrayObject();
        if ($objResultXML != null)
        {
            if ($objResultXML->getName() == "results")
            { 
                foreach ($objResultXML->children() as $objRow)
                {
                    if ($objRow->getName() == "row")
                    {
                        $objArrayResult->append(array('id' => (String)$objRow->id, 'name' => (String)$objRow->name));
                    }
                }
            }
        }
        
        return $objArrayResult;  
    }
    
    
    /**
    * Return query by type   
    * 
    * @param com_borealis_acl_object_Base $pobjBase  Object
    * @param string                       $pstrQuery Query Name 
    *  
    * @return string
    */
    private static function getQueryByObject(com_borealis_acl_object_Base $pobjBase, $pstrQuery)
    {
        $strType = 'Role';
        if ($pobjBase instanceof com_borealis_acl_object_Action)
        {
            $strType = 'Action';     
        }
        else if ($pobjBase instanceof com_borealis_acl_object_Role)
        {
            $strType = 'Role';     
        } 
        else if ($pobjBase instanceof com_borealis_acl_object_Ressource)
        {
            $strType = 'Ressource';     
        } 
        return self::getQueryByType($strType, $pstrQuery);  
    }
    
    /**
    * Return query by type
    * 
    * @param string $pstrType  Object Type
    * @param string $pstrQuery Query Name
    * 
    * @return string
    */
    private static function getQueryByType($pstrType, $pstrQuery)
    {
        $objArrayQuery = array(
                              'Action'  => array('PREPARE_INSERT'          => com_borealis_acl_SettingImpl::getDataStorePrepareInsertAction(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareDeleteActionByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getDataStorePrepareUpdateAction(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getDataStorePrepareSelectActionByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareSelectActionByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getDataStorePrepareSelectActionAll()
                                                 ),
                                                 
                              'Role'    => array('PREPARE_INSERT'          => com_borealis_acl_SettingImpl::getDataStorePrepareInsertRole(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareDeleteRoleByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getDataStorePrepareUpdateRole(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRoleByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRoleByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRoleAll() 
                                                 ),
                                                 
                              'Ressource'=> array('PREPARE_INSERT'         => com_borealis_acl_SettingImpl::getDataStorePrepareInsertRessource(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareDeleteRessourceByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getDataStorePrepareUpdateRessource(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRessourceByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRessourceByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getDataStorePrepareSelectRessourceAll()
                                                 )
                              );  
          
        return $objArrayQuery[$pstrType][$pstrQuery];      
    }        
    
    /**
    * Execute query statement and throw exception on error
    * 
    * @param mixed $pstrQuery  Query to execute
    * @param mixed $pobjParams Array of parameters to pass to query
    * 
    * @return SimpleXML result from the webService
    */
    private static function executeUpdate($pstrQuery, $pobjParams)
    {
        $strWsdl = com_borealis_acl_SettingImpl::getWebServiceDataStoreWSDL();
        $strConnectionLookup = com_borealis_acl_SettingImpl::getWebServiceDataStoreConnectionLookup();
        return com_borealis_foundation_util_WebServices::executePreparedUpdate($strWsdl, $strConnectionLookup, $pstrQuery, $pobjParams);
    }                     
    
    /**
    * Execute query statement and throw exception on error
    * 
    * @param mixed $pstrQuery  Query to execute
    * @param mixed $pobjParams Array of parameters to pass to query
    * 
    * @return SimpleXML result from the webService
    */
    private static function executeQuery($pstrQuery, $pobjParams)
    {
        $strWsdl = com_borealis_acl_SettingImpl::getWebServiceDataStoreWSDL();
        $strConnectionLookup = com_borealis_acl_SettingImpl::getWebServiceDataStoreConnectionLookup();
        return com_borealis_foundation_util_WebServices::executePreparedQuery($strWsdl, $strConnectionLookup, $pstrQuery, $pobjParams);
    }           
    
    /**
    * Function to add a parameter (type & value pair) into the parameters array.
    * 
    * @param mixed &$pobjParams    Parameters array
    * @param mixed $pstrParamType  Type of the parameter (known by the WebService)
    * @param mixed $pstrParamValue Value of the parameter
    * 
    * @return void
    */
    private static function addParameter(&$pobjParams, $pstrParamType, $pstrParamValue)
    {
        com_borealis_foundation_util_WebServices::addPreparedStatementParameter($pobjParams, $pstrParamType, $pstrParamValue);
    }
    
    /**
    * Appelle une méthode du webService et retourne la valeur string du return 
    * 
    * @param mixed $pstrMethodName   Nom de la méthode a appeler
    * @param mixed $pobjMethodParams Array de paramètres à passer à la méthode
    * 
    * @return object result from the webService 
    */
    private static function getWebServiceMethodReturnValue($pstrMethodName, $pobjMethodParams = null)
    {
        $strWsdl = com_borealis_acl_SettingImpl::getWebServiceDataStoreWSDL();
        return com_borealis_foundation_util_WebServices::getWebServiceMethodReturnValue($strWsdl, $pstrMethodName, $pobjMethodParams);
    }

}
  
?>
