<?php   
/**
* Classe de base pour tous les checker, classe de vérification d'authorisation de connexion
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-authentication  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Classe de base pour tous les checker, classe de vérification d'authorisation de connexion
* 
* @category   PHP 
* @package    Borealis-authentication
* @subpackage Checker
* @author     Pascal Martin 
*/
abstract class com_borealis_authentication_checker_Checker
{
    /**
     * @var com_borealis_authentication_password_PasswordCreator
     */    
    private $m_objPasswordCreator;
    
    /**
    * Construit une nouvelle instance de com_borealis_authentication_checker_Checker 
    * 
    * @param com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator Password Creator
    * 
    * @return com_borealis_authentication_checker_Checker
    */
    function __construct(com_borealis_authentication_password_PasswordCreator $pobjPasswordCreator) 
    {
        $this->m_objPasswordCreator = $pobjPasswordCreator;
    }
    
    /**
     * Retourne le mot de passe dans le bonne encodage
     * 
     * @param string $pstrPassword Mot de passe
     * 
     * @return string
     */
    protected function getPasswordValue($pstrPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrPassword);        

        return $this->m_objPasswordCreator->getPassword($pstrPassword)->getValue();
    }    
    
    /**
    * Vérifie le doit d'authentification
    * 
    * @param string $pstrUsername Nom d'utilisateur
    * @param string $pstrPassword Mot de passe
    * 
    * @return boolean Authorized 
    */
    abstract public function isAuthorized($pstrUsername, $pstrPassword);   
}   
    
?>