<?php
/**
* Methode utilitaire pour appeller un webService  
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-foundation  
* @author    Charles Villemure
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/  

/**
* Methode utilitaire pour appeller un webService  
*
* PHP versions 5
* 
* @category   PHP
* @package    Borealis-foundation 
* @subpackage Util 
* @author     Charles Villemure
*/ 
class com_borealis_foundation_util_WebServices
{
    /**
    * Soap error from the last soap call
    * 
    * @var mixed
    */
    private static $s_objSoapClientError = null;                                       
             
    /**
    * Generic soap call.  Call the given method from the given WSDL with 
    * the given parameters with the given soap parameters.
    * Default soap argument used :
    *       compression         => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP | 5
    *       encoding            => UTF-8
    *       connection_timeout  => 0
    *       exceptions          => true
    *       uri                 => http://service.web.borealis.com
    *            
    * @param mixed $pstrWsdl            WSDL location
    * @param mixed $pstrMethodName      Method to invoke from the WSDL
    * @param mixed $pobjMethodArguments Arguments to pass to the method invocation (Array of SoapVar)(OPTIONAL)
    * @param mixed $pobjSoapArguments   Soap Arguments to use with the soapCall (OPTIONAL)
    * 
    * @return mixed
    */
    public static function call($pstrWsdl, $pstrMethodName, $pobjMethodArguments = array(), $pobjSoapArguments = null)
    {
        $objSoapArguments = array('compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP | 5, 
                                'encoding'=>'UTF-8',
                                'connection_timeout'=> 0,
                                'exceptions' => true,
                                'location' => $pstrWsdl, 
                                'uri'  => 'http://service.web.borealis.com');             
        self::$s_objSoapClientError = null;
        $objSoapClient = null;

        if ($pobjSoapArguments != null)
        {
            $objSoapClient = new SoapClient(null, $pobjSoapArguments);
        }
        else
        {
            $objSoapClient = new SoapClient(null, $objSoapArguments);        
        }

        if ($objSoapClient == null)
        {
            throw new Exception('Web service url incorrect.');      
        }
             
        if ($pobjMethodArguments == null || !is_array($pobjMethodArguments))
        {
            $pobjMethodArguments = array();
        }
        
        $objResult = $objSoapClient->__soapCall($pstrMethodName, $pobjMethodArguments); 

        if (is_soap_fault($objResult))
        {   
            self::$s_objSoapClientError = $objSoapClient->getError(); 
            trigger_error("SOAP Fault: (faultcode: {$objResult->faultcode}, faultstring: {$objResult->faultstring})", E_USER_ERROR); 
        }
        
        return $objResult;
        
    }
    
    /**
    * Return the soap error of the last "call" method call
    * If it is null, the "call" method was not called or
    * the last "call" method call was successful.
    * 
    * @return SoapFault Soap Error of the last soap call
    */
    public static function getSoapClientError()
    {
        return self::$s_objSoapClientError;   
    }
    
    /**
    * Return the value of web service call
    * 
    * @param mixed $psResult Result from the soap call
    * 
    * @return string Return value of the soap call
    */ 
    public static function getReturn($psResult)
    {
        $sResult = null;
       
        if (!empty($psResult))
        {           
            //Axis 2 return an Array object
            if (is_array($psResult) && isset($psResult['return']))
            {
                $sResult = $psResult['return'];                 
            }
            else if (isset($psResult->return))
            {
                $sResult = $psResult->return;
            } 
            else if (is_string($psResult))
            {
                $sResult = $psResult;
            }
        }
        
        return $sResult;
    } 
    
    /**
    * Execute update statement and throw exception on error
    * 
    * @param mixed $pstrWSDL             URL du WSDL pour le webService
    * @param mixed $pstrConnectionLookup String representing the connectionLookup in Tomcat
    * @param mixed $pstrQuery            String query
    * @param mixed $pobjParams           Array of parameters to pass to query
    * 
    * @return int Number of row affected by update
    */
    public static function executePreparedUpdate($pstrWSDL, $pstrConnectionLookup, $pstrQuery, $pobjParams)
    {
        $objMethodParams = array();
        $objMethodParams[] = self::getSoapVar($pstrConnectionLookup, XSD_STRING, 'connectionLookup');
        $objMethodParams[] = self::getSoapVar($pstrQuery, XSD_STRING, 'sql');
        if ($pobjParams != null && is_array($pobjParams))
        {
            $objMethodParams[] = self::getSoapVar(json_encode($pobjParams), XSD_STRING, 'parameters');  
        }
        
        $objResult = self::getWebServiceMethodReturnValue($pstrWSDL, 'executePreparedUpdate', $objMethodParams);
        return $objResult; 
    }
    
    /**
    * Execute query statement and throw exception on error
    * 
    * @param mixed $pstrWSDL             URL du WSDL pour le webService
    * @param mixed $pstrConnectionLookup String representing the connectionLookup in Tomcat
    * @param mixed $pstrQuery            String query
    * @param mixed $pobjParams           Array of parameters to pass to query
    * 
    * @return SimpleXML result from the webService
    */
    public static function executePreparedQuery($pstrWSDL, $pstrConnectionLookup, $pstrQuery, $pobjParams)
    {
        $objMethodParams = array();
        $objMethodParams[] = self::getSoapVar($pstrConnectionLookup, XSD_STRING, 'connectionLookup');
        $objMethodParams[] = self::getSoapVar($pstrQuery, XSD_STRING, 'sql');          
        if ($pobjParams != null && is_array($pobjParams))
        {
            $objMethodParams[] = self::getSoapVar(json_encode($pobjParams), XSD_STRING, 'parameters');  
        }
        
        $objResult = self::getWebServiceMethodReturnValue($pstrWSDL, 'executePreparedQuery', $objMethodParams);
        
        return simplexml_load_string($objResult); 
    }
    
    /**
    * Appelle une méthode du webService et retourne la valeur string du return 
    * 
    * @param mixed $pstrWSDL         URL du WSDL pour le webService
    * @param mixed $pstrMethodName   Nom de la méthode a appeler
    * @param mixed $pobjMethodParams Array de paramètres à passer à la méthode
    * 
    * @return object result from the webService 
    */
    public static function getWebServiceMethodReturnValue($pstrWSDL, $pstrMethodName, $pobjMethodParams = null)
    {
        try
        {
            $objResult = self::call($pstrWSDL, $pstrMethodName, $pobjMethodParams);
        }
        catch(SoapFault $objException)
        {
            if (strstr($objException->getMessage(), 'Le connectionLookup fournit n\'existe pas'))
            {
                throw new Exception($objException->getMessage());
            }
            else  if (strstr($objException->getMessage(), 'Votre requête sqlUpdate et/ou sqlQuery contient un erreur:'))
            {
                throw new SQLException($objException->getMessage());
            }
            else
            {
                throw $objException;
            }
        }
        
        return self::getReturn($objResult);
    }
    
    /**
    * Return a soap variable
    * 
    * @param mixed $pstrVarValue value of the variable
    * @param mixed $pstrVarType  type (SOAP) of the variable
    * @param mixed $pstrVarName  name of the variable
    * 
    * @return SoapVar
    */
    private static function getSoapVar($pstrVarValue, $pstrVarType, $pstrVarName = null)
    {
        return new SoapVar($pstrVarValue, $pstrVarType, null, null, $pstrVarName, 'http://service.web.borealis.com'); 
    } 
    
    /**
    * Function to add a parameter (type & value pair) into the parameters array.
    * 
    * @param mixed &$pobjParams    Parameters array
    * @param mixed $pstrParamType  Type of the parameter (known by the WebService)
    * @param mixed $pstrParamValue Value of the parameter
    * 
    * @return void
    */
    public static function addPreparedStatementParameter(&$pobjParams, $pstrParamType, $pstrParamValue)
    {
        if ($pobjParams == null)
        {
            $pobjParams = array();
        }
        $objParam = array();
        $objParam[0] = $pstrParamType;
        $objParam[1] = $pstrParamValue;

        $pobjParams[] = $objParam;
    }
}

?>
