<?php
/**
* Récupère les informations de la base de donnée     
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-users  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations de la base de donnée
* 
* @category   PHP 
* @package    Borealis-users
* @subpackage DataStore  
* @author     Pascal Martin 
*/ 
class com_borealis_users_datastore_PDODataStore implements com_borealis_users_datastore_DataStore
{
    private $m_strDSN = null;
    private $m_strUsersQuery = null;
    private $m_objArrayUsers = null;
    
    /**
    * Constructeur de la classe com_borealis_users_datastore_PDODataStore 
    * 
    * @param string $pstrDSN        Chaine de connexion à la basse de données
    * @param string $pstrUsersQuery Requête sql pour la récupération des user doit retouné des ligne avec la colonne 'name' et 'username'
    * 
    * @return com_borealis_users_datastore_PDODataStore
    */
    public function __construct($pstrDSN, $pstrUsersQuery) 
    {        
        $this->m_strDSN = $pstrDSN;
        $this->m_strUsersQuery = $pstrUsersQuery;
        $this->fill();
    }
    
    /**
    * Chargement de utilisateur de la source de donnée
    * 
    * @return void
    */
    public function fill()
    {
        $this->m_objArrayUsers = array();
        
        $objPDO = new PDO($this->m_strDSN);     
        $objStatement = $objPDO->prepare($this->m_strUsersQuery);         
        self::execute($objStatement);

        while ($objArrayRow = $objStatement->fetch(PDO::FETCH_ASSOC)) 
        {
            $this->m_objArrayUsers[] = new com_borealis_users_User((string)$objArrayRow['name'], (string)$objArrayRow['username']);
        }                    
    }
    
    /**
    * Retourne la liste de utilisateur de la soruce de donnée
    * 
    * @return array
    */
    public function getUsers()
    {
        return $this->m_objArrayUsers;         
    }  
    
    /**
    * Execute statement and throw exception on error
    * 
    * @param PDOStatement &$pobjStatement PDO Statement already prepare
    * 
    * @throws com_borealis_acl_exception_SQLException
    * 
    * @return void 
    */
    private static function execute(PDOStatement &$pobjStatement)
    {
        if (!$pobjStatement->execute())
        {
            $objArrayErrorInfo = $pobjStatement->errorInfo();
            throw new Exception("SQLSTATE: $objArrayErrorInfo[0] | Driver Error Code: $objArrayErrorInfo[1] | Description: $objArrayErrorInfo[2]");
        }      
    }  
}
  
?>
