<?php
/**
* Fichier php pour le l'affichage de la section de gestion des actions
*
* PHP versions 5
* 
* @category  PHP
* @package   ACL
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

// -----------------------------------------------
// --- Include
// -----------------------------------------------
require_once 'common.php';      

// -----------------------------------------------
// --- Set domain
// -----------------------------------------------
bindtextdomain("actions", $g_strLanguageDir);
textdomain("actions");
bind_textdomain_codeset("actions", "UTF-8");

// -----------------------------------------------
// --- Variable global
// -----------------------------------------------
$g_strErrorAddAction = '';   
$g_strErrorDeleteAction = '';
$g_strInfoMessage = '';

// -----------------------------------------------
// --- Action
// -----------------------------------------------

/**
* Ajoute un action
* 
* @return void
*/
function addAction()
{
    global $g_strErrorAddAction;
    global $g_strInfoMessage;
    
    if (isset($_REQUEST['name']) && trim($_REQUEST['name']) != '')
    {
        $strName = trim($_REQUEST['name']); 
        try
        {        
            com_borealis_acl_object_Action::create($strName);             
            $g_strInfoMessage = sprintf(gettext('info_ok_add'), $strName);          
        }
        catch(com_borealis_acl_exception_SQLException $objError)
        {            
            $g_strErrorAddAction .= sprintf(gettext('error_already_exist_action'), $strName);        
        }
    }
    else
    {
        $g_strErrorAddAction .= gettext('error_empty_action');
    }   
}

/**
* Supprime un action
* 
* @return void   
*/
function deleteAction()
{
    global $g_strInfoMessage;
    global $g_strErrorDeleteAction; 
    
    $objAction = com_borealis_acl_object_Action::getByName($_REQUEST['actionname']);
    try
    {
        $objAction->remove();        
        $g_strInfoMessage = sprintf(gettext('info_ok_delete'), $_REQUEST['actionname']);
    }
    catch(Exception $objError)
    {    
        $g_strErrorDeleteAction .= sprintf(gettext('error_action_delete'), $_REQUEST['actionname']);  
    }
}

/**
* Met à jour le nom de l'action
* 
* @return void   
*/
function updateAction()
{
    global $g_strInfoMessage;
    global $g_strErrorDeleteAction; 
    
    if (isset($_REQUEST['name']) && trim($_REQUEST['name']) != '')
    { 
        $strName = trim($_REQUEST['name']);
        $objAction = com_borealis_acl_object_Action::getByName($_REQUEST['actionname']);
        try
        {
            $objAction->setName($strName);            
            $g_strInfoMessage = sprintf(gettext('info_ok_update'), $_REQUEST['actionname'], $strName);
        }
        catch(Exception $objError)
        {        
            $g_strErrorDeleteAction .= sprintf(gettext('error_action_update'), $_REQUEST['actionname'], $strName);  
        }
    }
    else
    {
        $g_strErrorDeleteAction .= gettext('error_empty_action');
    }   
}
     
if (isset($_REQUEST['action']))
{   
    switch($_REQUEST['action'])
    {
        case 'add_action':
            addAction();
            break;
        case 'delete_action':
            deleteAction();
            break;
        case 'update_action':
            updateAction();
            break;
    }
}

// -----------------------------------------------
// --- Smarty display
// -----------------------------------------------
$g_objSmarty->assign('actions', com_borealis_acl_object_Action::getAll()); 
$g_objSmarty->assign('errorAddAction', $g_strErrorAddAction);
$g_objSmarty->assign('errorDeleteAction', $g_strErrorDeleteAction);     
$g_objSmarty->assign('infoMessage', $g_strInfoMessage); 
                                                                 
$g_objSmarty->display('actions.tpl');
  
?>