<?php

session_start();

// ----------------
// GLOBAL VARIABLES
// ----------------
$g_strPDODSN = 'pgsql:host=127.0.0.1;port=5432;dbname=geoprismademo;user=mapgears;password=mapgears';
$strCurrentDir = dirname(__FILE__);
require "$strCurrentDir/../../config.php";

$strGeoprismaRoot = $strCurrentDir."/../../../"; 
$g_strBorealisAuthentication = "$strGeoprismaRoot/lib/php-borealis-authentication-0.1.0.2-lite";
$g_strBorealisAcl = "$strGeoprismaRoot/lib/php-borealis-acl-0.1.0.9-lite"; 

// --------------------------------------
// (1) Set include path
// --------------------------------------
set_include_path(
    "."
    .PATH_SEPARATOR.$g_strGeoprismaCore
    .PATH_SEPARATOR.$g_strBorealisFoundation
    .PATH_SEPARATOR.$g_strBorealisAuthentication
    .PATH_SEPARATOR.$g_strBorealisAcl
);

// --------------------------------------
// (2) Require class
// --------------------------------------
require_once 'com/borealis/Autoload.php';   


// ----------
// (3) LOGOUT
// ----------

if(isset($_REQUEST['logout']))
{
    unset($_SESSION['login_username']);
}

// -----------------------------------------------------------------------------
// (4) USERNAME & PASSWORD
//     If alread logged in, no need to do anything.  Else, check if username and
//     password are set.
// -----------------------------------------------------------------------------

if (!isset($_SESSION['login_username'])) 
{
    $strUsername = null;
    $strPassword = null;
    $strMethod = null;
    $strError = '';

    if(isset($_REQUEST['username']))
    {
        $strUsername = $_REQUEST['username'];
    }
    if(isset($_REQUEST['password']))
    {
        $strPassword = $_REQUEST['password'];
    }

    if ($strUsername == null || $strPassword == null)
    {
        if(!isset($_REQUEST['logout']))
        {
            $strError = 'Username and password are mandatory';
	}
        include 'login.php';
        exit();
    }

    // ----------------------------------
    // (5) SET AUTHENTIFICATION METHOD
    // ----------------------------------
    // Set password creator
    $objRawPasswordCreator = new com_borealis_authentication_password_RawPasswordCreator();

    // Create one PDO checker (only)
    $g_strPDOQuery = 'select id_user from t_user where username = :username and password = :password';
    $objChecker = new com_borealis_authentication_checker_PDOChecker($objRawPasswordCreator, $g_strPDODSN, $g_strPDOQuery);

    // Set checkers to borealis_authentification
    com_borealis_authentication_SettingImpl::setCheckers(array($objChecker));

    // ----------------------------------------------
    // (6) IS AUTHORIZED
    //     Check if the user is authorized to log in.
    // ----------------------------------------------

    $bIsAuthorized = com_borealis_authentication_AuthenticationManager::isAuthorized($strUsername, $strPassword);

    if (!$bIsAuthorized)
    {
        $strError = 'The username or password is incorrect.  Please try again.';
        include 'login.php';
	exit();
    }

    $_SESSION['login_username'] = $strUsername;
}

if (isset($_SESSION['login_username'])) 
{
    // ----------------------------------------
    // (7) GEOPRISMA VARIABLES
    //     Set some variables used by GeoPrisma
    // ----------------------------------------  
    org_geoprisma_SettingImpl::setTemplateDir($g_strTemplateDir);
    org_geoprisma_SettingImpl::setWidgetDir($g_strWidgetDir);
    org_geoprisma_SettingImpl::setLocaleDir($g_strLocaleDir);   

    org_geoprisma_SettingImpl::setLibUrl($g_strLibUrl);
    org_geoprisma_SettingImpl::setTemplateUrl($g_strTemplateUrl);
    org_geoprisma_SettingImpl::setWidgetUrl($g_strWidgetUrl );

    // Force selection of local language
    //$_SESSION[org_geoprisma_SettingImpl::getLocaleLanguageRequest()] = 'fr_CA';

    // -------
    // (8) ACL
    // -------
    // set config
    org_geoprisma_SettingImpl::setConfig('org_geoprisma_config_PGSQLMapContextConfig');
    $szConnection = 'host=localhost dbname=mcconfig user=demo password=demo port=5432';

    org_geoprisma_config_PGSQLMapContextConfig::setConnectionString($szConnection);
    org_geoprisma_config_PGSQLMapContextConfig::setCurrentMapContextName("MC_Default");
    org_geoprisma_config_PGSQLMapContextConfig::setCurrentApplicationName("AP_Default");

    // set ACL
    org_geoprisma_SettingImpl::setACLClass('org_geoprisma_acl_BorealisACL');

    // ACL settings
    com_borealis_acl_SettingImpl::setDataStore('com_borealis_acl_datastore_PDODataStore');
    com_borealis_acl_SettingImpl::setPDODataStoreDSN($g_strPDODSN);

    // Set username in ACL
    org_geoprisma_acl_BorealisACL::setUsername($_SESSION['login_username']);
}

?>
