<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2011- Mapgears Inc., published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:editfeature_copy="http://geoprisma.org/editfeature_copy"
    extension-element-prefixes="dyn">
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <xsl:template name="editfeature_copy:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>editfeature_copy</xsl:text>
    </xsl:variable>

    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/editfeature/EditFeature.js</xsl:text>
      </xsl:attribute>
    </script>

    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/editfeature/copy/lib/OpenLayers/Control/EditFeature_Copy.js</xsl:text>
      </xsl:attribute>
    </script>

    <script type="text/javascript">
      if (!Ext.isDefined(goEditFeatureActions)) {
          var goEditFeatureActions = {};
      }
    </script>

  </xsl:template>

  <xsl:template name="editfeature_copy:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>editfeature_copy</xsl:text>
    </xsl:variable>

    <!-- This widget supports this servicetype only-->
    <xsl:variable name="pServiceType">
      <xsl:text>featureserver</xsl:text>
    </xsl:variable>

    <script type="text/javascript">
      <!-- Quick reference to the Map object -->
      oMap = objGPWidget<xsl:value-of select="$pMapName"/>;

      var options, defaultOptions, action;
      
      defaultOptions =
      <xsl:for-each select="./options">
        <xsl:call-template name="getObjectFromNode">
          <xsl:with-param name="pNode" select="." />
        </xsl:call-template>
      </xsl:for-each>;

      <xsl:for-each select="./resources/resource">

        <!-- ResourceName -->
        <xsl:variable name="pResourceName">
          <xsl:value-of select="." />
        </xsl:variable>

        <!-- Use the DataStoreName to validate that the resource has a valid
             service type
          -->
        <!-- DataStoreName -->
        <xsl:variable name="pDataStoreName">
          <xsl:call-template name="getDataStoreInfoFromWidget">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pServiceType" select="$pServiceType" />
            <xsl:with-param name="pNodeName"><xsl:text>name</xsl:text></xsl:with-param>
            <xsl:with-param name="pResourceName" select="$pResourceName"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:if test="$pDataStoreName!=''">
          <!-- get resource 'iconClsCopy' option, if any -->
          <xsl:variable name="iconClsCopy">
            <xsl:value-of select="/geoprisma/resources/resource[./name = $pResourceName]/options/iconClsCopy"/>
          </xsl:variable>

          <!-- get resource 'tooltipTextCopy' option, i18n supported -->
          <xsl:variable name="tooltipTextCopy">
            <xsl:for-each select="/geoprisma/resources/resource[./name = $pResourceName]/options/tooltipTextCopy"><xsl:call-template name="getText" /></xsl:for-each>
          </xsl:variable>

          var actionId = Ext.id();
          var options = Ext.applyIf({
              'resource': '<xsl:value-of select="$pResourceName" />',
              'id': actionId,
              <!-- resourceOptions -->
              'resourceOptions' :
                <xsl:for-each select="/geoprisma/resources/resource[./name = $pResourceName]/options">
                  <xsl:call-template name="getObjectFromNode">
                    <xsl:with-param name="pNode" select="." />
                  </xsl:call-template>
                </xsl:for-each>
          }, defaultOptions);

          <!-- 'selectorMethod' resource option set as Function  -->
          <xsl:if test="/geoprisma/resources/resource[./name = $pResourceName]/options/selectorMethod">
            options['resourceOptions'].selectorMethod = <xsl:value-of select="/geoprisma/resources/resource[./name = $pResourceName]/options/selectorMethod" />;
	  </xsl:if>

          <!-- attribute cloning management -->
          <xsl:choose>
            <xsl:when test="/geoprisma/resources/resource[./name = $pResourceName]/options/copyFieldList">
              options.attributes = '<xsl:value-of select="/geoprisma/resources/resource[./name = $pResourceName]/options/copyFieldList" />'.split(",");
            </xsl:when>
            <xsl:otherwise>
              <!-- add support for resource fields here -->
            </xsl:otherwise>
          </xsl:choose>

          <!-- Widget object creation  -->
          var objGPWidget<xsl:value-of select="$pWidgetName" /><xsl:value-of select="$pResourceName" /> = new OpenLayers.Control.EditFeature_Copy(options);

          <!-- GeoExt.Action object for the geoexttoolbar-->
          action = new GeoExt.Action({
            id: actionId,
          <xsl:choose>
            <xsl:when test="$iconClsCopy != ''">
            iconCls: '<xsl:value-of select="$iconClsCopy" />',
	    </xsl:when>
	    <xsl:otherwise>
            iconCls: 'editfeature_copy',
	    </xsl:otherwise>
	  </xsl:choose>
            control: objGPWidget<xsl:value-of select="$pWidgetName" /><xsl:value-of select="$pResourceName" />,
            map: oMap,
            // button options
            toggleGroup: oMap.id,
            allowDepress: false,
          <xsl:choose>
            <xsl:when test="$tooltipTextCopy != ''">
            tooltip: <xsl:value-of select="$tooltipTextCopy" />,
	    </xsl:when>
	    <xsl:otherwise>
              <xsl:choose>
                <xsl:when test="$g_locale = 'fr_CA'">
            tooltip: "Outil de copie de géométrie",
	        </xsl:when>
	        <xsl:otherwise>
            tooltip: "Geometry copying widget",
                </xsl:otherwise>
              </xsl:choose>
	    </xsl:otherwise>
	  </xsl:choose>
            // check item options
            group: oMap.id
          });

          <!-- Push action in goEditFeatureActions['resource'] array  -->
          if (!goEditFeatureActions['<xsl:value-of select="$pResourceName" />'])
          {
              goEditFeatureActions['<xsl:value-of select="$pResourceName" />'] = [];
          }
          goEditFeatureActions['<xsl:value-of select="$pResourceName" />'].push(action);

          <!-- toggleEdit option management, if set, listen to map event
               'printallwidgetexecutions' to set widget 'updateControl' -->
          <xsl:if test="../../options/toggleEdit">
            <xsl:for-each select="/geoprisma/widgets/widget[./type = 'editfeature_update']/resources[./resource = $pResourceName]">
              <xsl:variable name="editFeatureUpdateWidgetName">
                <xsl:value-of select="../name"/>
              </xsl:variable>
              oMap.events.on({"printallwidgetexecutions": function(event) {
                  objGPWidget<xsl:value-of select="$pWidgetName" /><xsl:value-of select="$pResourceName" />.updateControl = objGPWidget<xsl:value-of select="$editFeatureUpdateWidgetName" /><xsl:value-of select="$pResourceName" />;
              }});
            </xsl:for-each>
          </xsl:if>

        </xsl:if>
      </xsl:for-each>
    </script>

    <script type="text/javascript">
    </script>
  </xsl:template>

</xsl:stylesheet>
