<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:featurepanel_selector="http://geoprisma.org/featurepanel_selector"
    extension-element-prefixes="dyn">
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="featurepanel_selector:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_selector</xsl:text>
    </xsl:variable>

    <!-- Stylesheets and locale-specific stylesheets  -->
    <link rel="stylesheet" type="text/css" >
      <xsl:attribute name="href" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/featurepanel/selector/resources/css/featurepanel_selector.css</xsl:text>
      </xsl:attribute>
    </link>
    <xsl:choose>
      <xsl:when test="$g_locale = 'fr_CA'">
        <link rel="stylesheet" type="text/css" >
          <xsl:attribute name="href" >
            <xsl:value-of select="$g_widgets_url"/>
            <xsl:text>/featurepanel/selector/resources/css/featurepanel_selector_fr.css</xsl:text>
          </xsl:attribute>
        </link>
      </xsl:when>
      <xsl:otherwise />
    </xsl:choose>

    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/featurepanel/selector/Selector.js</xsl:text>
      </xsl:attribute>
    </script>

    <script language="javascript" type="text/javascript">
      var nbcolumns = 0;
      <xsl:for-each select="//widget[./type = 'featurepanel_selector']">
	nbcolumns = Math.max(nbcolumns, <xsl:value-of select="count(./options/fields/field)" />);
      </xsl:for-each>	
	
      objGPWidget<xsl:value-of select="$WidgetType" /> = new org.GeoPrisma.FeaturePanel.Selector(
        {
          i18n_button_delegate : <xsl:value-of select="php:functionString('featurepanel_selector::getText','i18n_button_delegate','JS')" />,
          i18n_featurepanel_selector_grid_title : <xsl:value-of select="php:functionString('featurepanel_selector::getText','i18n_featurepanel_selector_grid_title','JS')" />,
          i18n_featurepanel_selector_no_context_title : <xsl:value-of select="php:functionString('featurepanel_selector::getText','i18n_featurepanel_selector_no_context_title','JS')" />,
          i18n_featurepanel_selector_no_context_message : <xsl:value-of select="php:functionString('featurepanel_selector::getText','i18n_featurepanel_selector_no_context_message','JS')" />,
          forceFit: true,
          enableColumnHide: false,
          nbcolumns: nbcolumns
        }
      );

      Ext.onReady(function() {
      <xsl:for-each select="//options[.//featurepanel[. = //widget[./type = 'featurepanel_selector']/name]]">
	<xsl:for-each select="../name">
	  <xsl:variable name="ownerWidgetName">
	    <xsl:value-of select="." />
	  </xsl:variable>
	  if (objGPWidget<xsl:value-of select="$ownerWidgetName" />)
	  {
            objGPWidget<xsl:value-of select="$ownerWidgetName" />.addFeaturePanel(objGPWidget<xsl:value-of select="$WidgetType" />);
            objGPWidget<xsl:value-of select="$WidgetType" />.addOwner(objGPWidget<xsl:value-of select="$ownerWidgetName" />);
	  }
          else
          {
	    objGPWidget<xsl:value-of select="$ownerWidgetName" />.addFeaturePanel(objGPWidget<xsl:value-of select="$WidgetType" />);
	    objGPWidget<xsl:value-of select="$WidgetType" />.addOwner(objGPWidget<xsl:value-of select="$ownerWidgetName" />);
	  }
	</xsl:for-each>
      </xsl:for-each>
      });
    </script>
  </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="featurepanel_selector:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_selector</xsl:text>
    </xsl:variable>

    <script type="text/javascript">
      <xsl:for-each select="./resources/resource">
        <!-- ResourceName -->
        <xsl:variable name="resourceName">
          <xsl:value-of select="." />
        </xsl:variable>

	objGPWidget<xsl:value-of select="$WidgetType" />.featureConfigs["<xsl:value-of select="." />"] = 
	<xsl:for-each select="//widget[./name = $pWidgetName]">
	  <xsl:call-template name="getObjectFromNode">
	    <xsl:with-param name="pNode" select="./options" />
	  </xsl:call-template>
	</xsl:for-each>;

    <xsl:if test="../../options/method != ''">
        objGPWidget<xsl:value-of select="$WidgetType" />.featureConfigs["<xsl:value-of select="." />"].options.method = <xsl:value-of select="../../options/method" />;
    </xsl:if>

	objGPWidget<xsl:value-of select="$WidgetType" />.featureConfigs["<xsl:value-of select="$resourceName" />"]['fieldTitlesByName'] = {};

        <!-- look for resource field titles -->
	<xsl:for-each select="//widget[./name = $pWidgetName]">
          <xsl:for-each select="./options/fields/field">
            <xsl:variable name="fieldName">
              <xsl:value-of select="./name" />
            </xsl:variable>

            <xsl:variable name="fieldType">
              <xsl:value-of select="./type" />
            </xsl:variable>

            <xsl:if test="$fieldType = 'key'">

              <!-- Resource field title -->
              <xsl:variable name="resourceFieldTitle">
                <xsl:for-each select="/geoprisma/resources/resource[./name = $resourceName]/fields/field[./name = $fieldName]/title">
                  <xsl:call-template name="getText" />
                </xsl:for-each>
              </xsl:variable>

              <xsl:if test="$resourceFieldTitle != ''">
                objGPWidget<xsl:value-of select="$WidgetType" />.featureConfigs["<xsl:value-of select="$resourceName" />"]['fieldTitlesByName']['<xsl:value-of select="$fieldName" />'] = <xsl:value-of select="$resourceFieldTitle" />;
              </xsl:if>
            </xsl:if>

	  </xsl:for-each>
        </xsl:for-each>

        <!-- titleColumn -->
        <!-- resource 'selectorTitle' option OR its 'title' OR its 'name' -->
        <xsl:variable name="titleColumn">
          <xsl:choose>
            <xsl:when test="/geoprisma/resources/resource[./name = $resourceName]/options/selectorTitle != ''">
              <xsl:for-each select="/geoprisma/resources/resource[./name = $resourceName]/options/selectorTitle">
                <xsl:call-template name="getText" />
              </xsl:for-each>
            </xsl:when>
            <xsl:when test="/geoprisma/resources/resource[./name = $resourceName]/title != ''">
              <xsl:for-each select="/geoprisma/resources/resource[./name = $resourceName]/title">
                <xsl:call-template name="getText" />
              </xsl:for-each>
            </xsl:when>
	    <xsl:otherwise>
              '<xsl:value-of select="$resourceName" />'
	    </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        objGPWidget<xsl:value-of select="$WidgetType" />.featureConfigs["<xsl:value-of select="$resourceName" />"]['titleColumn'] = <xsl:value-of select="$titleColumn" />;

      </xsl:for-each>
    </script>
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="featurepanel_selector:drawWidget">
    <xsl:param name="pWidgetName" />
    
    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_selector</xsl:text>
    </xsl:variable>
    
      objGPWidget<xsl:value-of select="$WidgetType" />
  </xsl:template>

<!-- 
     Function : drawWidgets

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="featurepanel_selector:drawWidgets">
    <xsl:param name="pContainer" />

    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_selector</xsl:text>
    </xsl:variable>

    if (window.objGPWidget<xsl:value-of select="$WidgetType" /> != undefined)
    {
        if(<xsl:value-of select="$pContainer" />['ctype']){
            <xsl:value-of select="$pContainer" />.add(
                objGPWidget<xsl:value-of select="$WidgetType" />
            );
        } else {
            <xsl:value-of select="$pContainer" />.items.push(
                objGPWidget<xsl:value-of select="$WidgetType" />
            )
        }
    }
  </xsl:template>

<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="featurepanel_selector:getWidgetDepedencies">
<!-- TODO : this widget has a toolbar,map,Result.js and queryonclick dependency -->
  </xsl:template>

</xsl:stylesheet>
