<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Mapgears Inc., published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:geoextux_zoomto="http://geoprisma.org/geoextux_zoomto"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <!-- printWidgetSource START  -->
  <xsl:template name="geoextux_zoomto:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>geoextux_zoomto</xsl:text>
    </xsl:variable>

    <!-- GeoExt.ux.ZoomTo source -->
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_lib_url"/>
             <xsl:text>/geoext-ux-zoomto/lib/GeoExt.ux/widgets/ZoomTo.js</xsl:text>
        </xsl:attribute>
    </script>
    
    <!-- CSS -->
    <link rel="stylesheet" type="text/css" >
      <xsl:attribute name="href" >
        <xsl:value-of select="$g_lib_url"/>
        <xsl:text>/geoext-ux-zoomto/resources/css/ZoomTo.css</xsl:text>
      </xsl:attribute>
    </link>

    <!-- i18n : "fr_CA" -->
    <xsl:if test="$g_locale = 'fr_CA'">
        <script>
            <xsl:attribute name="src" >
                 <xsl:value-of select="$g_lib_url"/>
                 <xsl:text>/geoext-ux-zoomto/resources/lang/fr.js</xsl:text>
            </xsl:attribute>
        </script>
    </xsl:if>

    <script type="text/javascript">
      var objGPWidget<xsl:value-of select="$WidgetType" />GeoExtToolbarOptions;
    </script>

   </xsl:template>
  <!-- printWidgetSource END  -->

  <!-- printWidgetExecution START  -->
  <xsl:template name="geoextux_zoomto:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>geoextux_zoomto</xsl:text>
    </xsl:variable>

      <!-- Proj4JS, only included if there are 'projection' option(s) defined.
           Since this widget should be included only once, including some lib
           here doesn't matter -->
    <!-- Proj4JS include START -->
    <xsl:choose>
      <xsl:when test="./options//projection">
        <!-- Source files + 900913 START -->
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/proj4js-compressed.js</xsl:text>
          </xsl:attribute>
        </script>
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/projCode/tmerc.js</xsl:text>
          </xsl:attribute>
        </script>
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/proj4js-compressed.js</xsl:text>
          </xsl:attribute>
        </script>
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/projCode/merc.js</xsl:text>
          </xsl:attribute>
        </script>
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/defs/EPSG900913.js</xsl:text>
          </xsl:attribute>
        </script>
        <!-- Source files + 900913 END -->

        <!-- projection array -->
        <script type="text/javascript">
          var objArrayProjections = [];
        </script>
      </xsl:when>
      <xsl:otherwise>
        <!-- projection array -->
        <script type="text/javascript">
          var objArrayProjections = [['EPSG:4326'],['EPSG:900913']];
        </script>
      </xsl:otherwise>
    </xsl:choose>

    <!-- include each additional projection file defined in options and push
         it in the Ext.data.SimpleStore object as well.
      -->
    <xsl:variable name="replaceSemiColons">
      <xsl:text>:</xsl:text>
    </xsl:variable>
    <xsl:variable name="byBlank">
      <xsl:text></xsl:text>
    </xsl:variable>

    <xsl:for-each select="./options//projection">
      <!-- include projection file, except for EPSG:4326 (already included) -->
      <xsl:if test=". != 'EPSG:4326'">
        <script>
          <xsl:attribute name="src" >
            <xsl:value-of select="$g_lib_url"/>
            <xsl:text>/proj4js/lib/defs/</xsl:text>
            <xsl:call-template name="stringReplaceAll">
              <xsl:with-param name="text" select="."/>
              <xsl:with-param name="replace" select="$replaceSemiColons"/>
              <xsl:with-param name="by" select="$byBlank"/>
            </xsl:call-template>
            <xsl:text>.js</xsl:text>
          </xsl:attribute>
        </script>
      </xsl:if>
      <!-- push to array -->
      <script type="text/javascript">
        if (Proj4js.defs['<xsl:value-of select="." />']) {
            objArrayProjections.push(['<xsl:value-of select="." />']);
        } else {
            alert(
                "The projection code '" + '<xsl:value-of select="." />' + 
                "' is not defined\nin the Proj4js library currently loaded. " +
                "Please,\nadd it first."
            );
        }
      </script>
    </xsl:for-each>

    <!-- Proj4JS include END -->

    <script type="text/javascript">
      oMap = objGPWidget<xsl:value-of select="$pMapName"/>;
      var objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions;

      var objWidgetOptions = {};
      <xsl:for-each select="./options">
        objWidgetOptions =
        <xsl:call-template name="getObjectFromNode">
          <xsl:with-param name="pNode" select="." />
        </xsl:call-template>;
      </xsl:for-each>
      objWidgetOptions['map'] = oMap;

      <!-- if at least one projection, then create SimpleStore obj -->
      if (objArrayProjections.length) {
          objWidgetOptions['projectionStore'] = new Ext.data.SimpleStore({
              fields: ['projection'],
              data : objArrayProjections
          });
      }

      <!-- Widget object creation -->
      var objGPWidget<xsl:value-of select="$pWidgetName" /> = 
          new GeoExt.ux.ZoomTo(objWidgetOptions);

      <!-- draw in toolbar -->
      objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions =
          objGPWidget<xsl:value-of select="$pWidgetName" />

    </script>

  </xsl:template>
  <!-- printWidgetExecution END  -->

</xsl:stylesheet>
