<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009-2012 Mapgears Inc.
   Published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 
<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:initialview="http://geoprisma.org/initialview"
    extension-element-prefixes="dyn">

  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <xsl:template name="initialview:printWidgetSource">
    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/initialview/lib/org.GeoPrisma.InitialView.LocalFormat.js</xsl:text>
      </xsl:attribute>    
    </script>
    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/initialview/lib/org.GeoPrisma.InitialView.Manager.js</xsl:text>
      </xsl:attribute>    
    </script>
    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/initialview/lib/org.GeoPrisma.InitialView.View.js</xsl:text>
      </xsl:attribute>    
    </script>
    <script>
      <xsl:attribute name="src" >
        <xsl:value-of select="$g_widgets_url"/>
        <xsl:text>/initialview/lib/org.GeoPrisma.InitialView.AsyncView.js</xsl:text>
      </xsl:attribute>    
    </script>
  </xsl:template>

  <xsl:template name="initialview:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <!-- ServiceTypes supported by this widget -->
    <xsl:variable name="fsServiceType">
      <xsl:text>featureserver</xsl:text>
    </xsl:variable>
    <xsl:variable name="wfsServiceType">
      <xsl:text>wfs</xsl:text>
    </xsl:variable>

    <script type="text/javascript">

      <!-- Manager creation -->
      var widgetOptions = Ext.applyIf({
          mapPanel: objGPWidgetMapPanel<xsl:value-of select="$pMapName" />
      }, 
      <xsl:for-each select="./options">
        <xsl:call-template name="getObjectFromNode">
          <xsl:with-param name="pNode" select="." />
        </xsl:call-template>
      </xsl:for-each>);

      <!-- vectorLayer option management -->
      <xsl:if test="./options/vectorlayer">
        <xsl:variable name="vectorLayer">
          <xsl:value-of select="./options/vectorlayer" />
        </xsl:variable>
        <xsl:if test="/geoprisma/widgets/widget[./name = $vectorLayer]">
            if (window.objGPWidget<xsl:value-of select="$vectorLayer" />) {
                widgetOptions.highlightLayer =
                    window.objGPWidget<xsl:value-of select="$vectorLayer" />;
            }
            delete widgetOptions.vectorlayer;
        </xsl:if>
      </xsl:if>

      objGPWidget<xsl:value-of select="$pWidgetName" /> =
          new org.GeoPrisma.InitialView.Manager(widgetOptions);

      <!-- localView -->
      <xsl:variable name="localViewIndex">
        <xsl:value-of select="php:functionString('InitialView::getLocalViewIndex',$pWidgetName)" />
      </xsl:variable>

      <xsl:variable name="localViewString">
        <xsl:choose>
          <xsl:when test="$localViewIndex > 1">
            <xsl:value-of select="./options/localViews/localView[$localViewIndex]" />
          </xsl:when>
          <xsl:when test="$localViewIndex = 1">
            <xsl:choose>
              <xsl:when test="./options/localView">
                <xsl:value-of select="./options/localView" />
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="./options/localViews/localView[$localViewIndex]" />
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
        </xsl:choose>
      </xsl:variable>

      <xsl:choose>
      <!-- if a localView string was found, use it -->
        <xsl:when test="$localViewString != ''">
          objGPWidget<xsl:value-of select="$pWidgetName" />.createLocalView(<xsl:value-of select="$localViewIndex" />-1);          
        </xsl:when>
        <!-- no valid localView found, browse resources -->
        <xsl:otherwise>

          <xsl:for-each select="/geoprisma/resources/resource">
            <xsl:variable name="resourceName">
              <xsl:value-of select="./name" />
            </xsl:variable>

            <!-- Use the DataStoreName to validate that the resource has a valid
                 service type, either 'featureserver' or 'wfs'
              -->
            <xsl:variable name="fsLayerName">
              <xsl:call-template name="getDataStoreInfoFromResource">
                <xsl:with-param name="pServiceType" select="$fsServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="wfsLayerName">
              <xsl:call-template name="getDataStoreInfoFromResource">
                <xsl:with-param name="pServiceType" select="$wfsServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="layerName">
              <xsl:choose>
                <xsl:when test="$fsLayerName != ''">
                  <xsl:value-of select="$fsLayerName" />
	        </xsl:when>
	        <xsl:otherwise>
                  <xsl:value-of select="$wfsLayerName" />
	        </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>

            <xsl:if test="$layerName != '' and ./options/initViewUrlField != ''">
              <xsl:choose>

                <!-- DataStore is FeatureServer -->
                <xsl:when test="$fsLayerName != ''">
                  <!-- get featuresString using widget getFeature php method -->
                  <xsl:variable name="featuresString">
                    <xsl:value-of select="php:functionString('initialview::getFeature',$resourceName,$fsServiceType)"></xsl:value-of>
                  </xsl:variable>

                  <xsl:if test="$featuresString != ''">
                    objGPWidget<xsl:value-of select="$pWidgetName" />.createView({
                    featuresString: <xsl:value-of select="php:functionString('org_geoprisma_locale_Locale::getText', string($featuresString), 'nodomain', 'JS')" />,
                    resourceName: '<xsl:value-of select="$resourceName" />',
                    serviceType: '<xsl:value-of select="$fsServiceType" />'
                    });
                  </xsl:if>
	        </xsl:when>

                <!-- DataStore is WFS -->
	        <xsl:otherwise>

                  <!-- get property value from url using widget php method -->
                  <xsl:variable name="urlFieldValue">
                    <xsl:value-of select="php:functionString('initialview::getURLFieldValue',$resourceName,$wfsServiceType)"></xsl:value-of>
                  </xsl:variable>

                  <xsl:if test="$urlFieldValue != ''">
                    <!-- ServiceName -->
                    <xsl:variable name="wfsServiceName">
                      <xsl:call-template name="getDataStoreInfoFromResource">
                        <xsl:with-param name="pServiceType" select="$wfsServiceType" />
                        <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
                        <xsl:with-param name="pResourceName" select="$resourceName" />
                      </xsl:call-template>
                    </xsl:variable>

                    objGPWidget<xsl:value-of select="$pWidgetName" />.createAsyncView({
                      filterProperty: '<xsl:value-of select="./options/initViewUrlField" />',
                      filterValue: '<xsl:value-of select="$urlFieldValue" />',
                      layerName: '<xsl:value-of select="$wfsLayerName" />',
                      resourceName: '<xsl:value-of select="$resourceName" />',
                      serviceName:  '<xsl:value-of select="$wfsServiceName" />',
                      serviceType: '<xsl:value-of select="$wfsServiceType" />',
                      url: '<xsl:value-of select="$proxyurl" />'
                    });
                  </xsl:if>
	        </xsl:otherwise>
              </xsl:choose>
            </xsl:if>

          </xsl:for-each>

        </xsl:otherwise>
      </xsl:choose>

      <!-- set map center -->
      objGPWidget<xsl:value-of select="$pWidgetName" />.setCenter();

    </script>
  </xsl:template>
</xsl:stylesheet>
