/* 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license. 
*/ 
/**
 * APIMethod: getLayerByResource
 * Get a layer with resource matching the given resource in its resources
 * array.
 *
 * Parameter:
 * match - {String} A resource name.  If no layers are found, an empty array is
 *                  returned.
 * serviceType - {String} (Optional) A service type name.  If defined, the layer
 *                        must also be of this type.
 *
 * Returns:
 * {Array(<OpenLayers.Layer>)} A array of layers matching the given resource
 *     name.  An empty array is returned if no matches are found.
 */
OpenLayers.Map.prototype.getLayersByResource = function(match, serviceType){
    var layers = [];
    for(var i=0; i<this.layers.length; i++){
        layer = this.layers[i];
        
        if(!layer.resources){
            continue;
        }

        if(OpenLayers.Util.indexOf(layer.resources, match) == -1){
            continue;
        }

        if(serviceType && layer.serviceType != serviceType){
            continue;
        }

        layers.push(layer);
    }

    return layers;
};

/**
 * APIMethod: show
 * Intelligent way to show a layer, i.e. if it has a 'params' property and 
 * the layer was build from multiple resources, merge new 'params' using 
 * this.layersByResource property.
 *
 * Also force the visibility to true in all cases.
 *
 * Parameter:
 * resourceName - {String} A resource name used if the layer has 'params'
 *                         property.
 */
OpenLayers.Layer.prototype.show = function(resourceName){
    if(resourceName && (typeof(this.params) != 'undefined') &&
       (typeof(this.layersByResource) != 'undefined')){
        params = OpenLayers.Util.upperCaseObject(this.params);
        sublayers = params['LAYERS'];

        sublayerToShow = this.layersByResource[resourceName];

        var p = {};
        p["LAYERS"] = [sublayerToShow];
        this.mergeNewParams(p);
    }

    this.setVisibility(true);
};

/**
 * Constant: EVENT_TYPES
 * Add 'GeoPrisma' specific map event types.
 * - unselectallfeatures
 * - printallwidgetexecutions : called after printAllWidgetExecutions
 */
OpenLayers.Map.prototype.EVENT_TYPES =
    OpenLayers.Map.prototype.EVENT_TYPES.concat(['unselectallfeatures',
                                                 'printallwidgetexecutions']);
