<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:map="http://geoprisma.org/map"
    xmlns:exslt="http://exslt.org/common"
    extension-element-prefixes="dyn exslt map">
  
  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

  <!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <xsl:template name="map:printWidgetSource">
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/map/Map.js</xsl:text>
        </xsl:attribute>    
    </script>

    <!-- NB: see in this file for important details -->
    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/map/MapFishProtocol.js</xsl:text>
        </xsl:attribute>    
    </script>

    <!-- N.B. Since the default 'delete' style is display: 'none', selected
         features to be deleted become invisible.  That's a little confusing,
         so the 'delete' style is overwritten here.  Same style as 'select',
         except fillColor: "black" instead of "blue"
      -->
    <script type="text/javascript">
    OpenLayers.Feature.Vector.style['delete'] = {
        fillColor: "black",
        fillOpacity: 0.4, 
        hoverFillColor: "white",
        hoverFillOpacity: 0.8,
        strokeColor: "blue",
        strokeOpacity: 1,
        strokeWidth: 2,
        strokeLinecap: "round",
        strokeDashstyle: "solid",
        hoverStrokeColor: "red",
        hoverStrokeOpacity: 1,
        hoverStrokeWidth: 0.2,
        pointRadius: 6,
        hoverPointRadius: 1,
        hoverPointUnit: "%",
        pointerEvents: "visiblePainted",
        cursor: "pointer"
    };
    </script>

  </xsl:template>
  
  <!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <xsl:template name="map:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="sXpathMap">
      <xsl:text>/geoprisma/widgets/widget[./name='</xsl:text>
      <xsl:value-of select="$pMapName"/>
      <xsl:text>']</xsl:text>
    </xsl:variable>
    
    <xsl:variable name="token">
        <xsl:text>TODO Delete this</xsl:text>
    </xsl:variable>

    <xsl:for-each select="dyn:evaluate($sXpathMap)">

      <script type="text/javascript">
        <!-- LayersToAdd -->
        var objGPWidget<xsl:value-of select="$pWidgetName" />LayersToAdd = [];
        var objArrayLayers = objGPWidget<xsl:value-of select="$pWidgetName" />LayersToAdd;

        <![CDATA[
        //   Get all the map options from config.xml
        var oMapOptions = {};
        ]]>
        <xsl:if test="./options/units">
          oMapOptions['units'] = "<xsl:value-of select='./options/units'/>";
        </xsl:if>
        <xsl:if test="./options/projection">
          oMapOptions['projection'] = new OpenLayers.Projection("<xsl:value-of select='./options/projection'/>");
        </xsl:if>
        <xsl:if test="./options/maxresolution">
          oMapOptions['maxResolution'] = <xsl:value-of select='./options/maxresolution'/>;
        </xsl:if>
        <xsl:if test="./options/maxextent">
          oMapOptions['maxExtent'] = new OpenLayers.Bounds(<xsl:value-of select='./options/maxextent'/>);
	    </xsl:if>
        <xsl:if test="./options/restrictedextent">
          oMapOptions['restrictedExtent'] = new OpenLayers.Bounds(<xsl:value-of select='./options/restrictedextent'/>);
	</xsl:if> 
	<xsl:if test="./options/tilesize">
          oMapOptions['tileSize'] = new OpenLayers.Size(<xsl:value-of select='./options/tilesize'/>);
	    </xsl:if>
	    <xsl:if test="./options/displayprojection">
          oMapOptions['displayProjection'] = new OpenLayers.Projection("<xsl:value-of select='./options/displayprojection'/>");
        </xsl:if>
        <xsl:if test="./options/restrictedextent">
          oMapOptions['restrictedExtent'] = new OpenLayers.Bounds(<xsl:value-of select='./options/restrictedextent'/>);
	    </xsl:if>
	    <xsl:if test="./options/resolutions">
          oMapOptions['resolutions'] = Array(<xsl:value-of select='./options/resolutions'/>);
        </xsl:if>
        <xsl:if test="./options/numzoomlevels">
          oMapOptions['numZoomLevels'] = <xsl:value-of select='./options/numzoomlevels'/>;
        </xsl:if>
        <xsl:if test="./options/scales">
          oMapOptions['scales'] = Array(<xsl:value-of select='./options/scales'/>);
        </xsl:if>
        <xsl:if test="./options/alloverlays">
          oMapOptions['allOverlays'] = <xsl:value-of select='./options/alloverlays'/>;
        </xsl:if>
        <![CDATA[
				 
		/*Add OpenLayers controller to activate the css of the zoomslider*/
		oMapOptions['controls'] = [
		 	new OpenLayers.Control.PanPanel(),
         	new OpenLayers.Control.ZoomPanel()
		];
		oMapOptions['theme'] = null;
        /*var dv = document.createElement('div'); // create dynamically div tag
        dv.setAttribute('id',"map");       //give id to it
        document.appendChild(dv);   */
        oMap = new OpenLayers.Map( 'map', oMapOptions );

        /*
            Create all OL layers from the map resource array
            We need:
            URL = the service name, resource name, token id
            OL layer type = service type (in map/resources)
            layers from datastore
            resource options
            resource params
         */
        var objOptions = {}; 
        var objParams = {}; 
        var objOLLayer;
        //Get the proxy from somewhere
        ]]>

        <xsl:for-each select="./layers/layer">
          <xsl:variable name="pResourceName">
            <xsl:value-of select="./resourcenames/resourcename[1]"/>
          </xsl:variable>

          <xsl:variable name="pServiceType">
            <xsl:value-of select="./servicetype"/>
          </xsl:variable>

          <xsl:variable name="pServiceName">
            <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
              <xsl:with-param name="pResourceName" select="$pResourceName"/>
              </xsl:call-template>
          </xsl:variable>

          <xsl:variable name="pLayerName">
            <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
              <xsl:with-param name="pResourceName" select="$pResourceName"/>
              </xsl:call-template>
          </xsl:variable>

          <!-- Service version (option) -->
          <xsl:variable name="serviceVersion">
            <xsl:value-of select="/geoprisma/services/service[./name = $pServiceName]/options/version" />
          </xsl:variable>

          <!-- Service staticCache (option) -->
          <xsl:variable name="serviceStaticCache">
            <xsl:value-of select="/geoprisma/services/service[./name = $pServiceName]/options/staticCache" />
          </xsl:variable>

          <!--
              Build the Layer URL
            -->
          <![CDATA[
              objParams = {};
              objOptions = {};
              var strGPToken = "]]><xsl:value-of select="$token" /><![CDATA[";
              var oURLParams = {
                'osmservice': ']]><xsl:value-of select="$pServiceName" /><![CDATA[',
                'osmtoken': strGPToken
              };

              var strURL = ']]><xsl:value-of select="$proxyurl" /><![CDATA[';

              var strURLAliases = ']]><xsl:value-of select="$g_proxy_url_aliases" /><![CDATA[';
              var boolURLAliasesEnabled = (strURLAliases != '');
              var objArrayURLAliases, strSeparator;

              if(boolURLAliasesEnabled) {
                  strSeparator = ']]><xsl:value-of select="$g_url_separator" /><![CDATA[';
                  objArrayURLAliases = strURLAliases.split(strSeparator);
              }

              var strParams = OpenLayers.Util.getParameterString(oURLParams);

              ]]>
              <xsl:for-each select="./resourcenames/resourcename">
                <![CDATA[
                strParams += '&osmresource[]=]]><xsl:value-of select="." /><![CDATA[';
                ]]>
              </xsl:for-each>
              <![CDATA[

              if(strParams.length > 0) {
                  var strSeparator = (strURL.indexOf('?') > -1) ? '&' : '?';
                  strURL += strSeparator + strParams;
                  if(boolURLAliasesEnabled) {
                      for(var i=0; i<objArrayURLAliases.length; i++){
                          var strSeparator = (objArrayURLAliases[i].indexOf('?') > -1) ? '&' : '?';
                          objArrayURLAliases[i] += strSeparator + strParams;
                      }
                  }
              }

              ]]>
          <!--
              Build the layer option array
            -->
          <xsl:if test="./options/projection">
            objOptions['projection'] = new OpenLayers.Projection("<xsl:value-of select='./options/projection'/>");
          </xsl:if>
          <xsl:if test="./options/maxresolution">
            objOptions['maxResolution'] = <xsl:value-of select='./options/maxresolution'/>;
          </xsl:if>
          <xsl:if test="./options/minresolution">
            objOptions['minResolution'] = <xsl:value-of select='./options/minresolution'/>;
          </xsl:if>
          <xsl:if test="./options/minscale">
            objOptions['minScale'] = <xsl:value-of select='./options/minscale'/>;
          </xsl:if>
          <xsl:if test="./options/maxscale">
            objOptions['maxScale'] = <xsl:value-of select='./options/maxscale'/>;
          </xsl:if>
          <xsl:if test='./options/isbaselayer'>
            objOptions['isBaseLayer'] =<xsl:value-of select='./options/isbaselayer'/>;
          </xsl:if>
          <xsl:if test='./options/extractAttributes'>
            objOptions['extractAttributes'] =<xsl:value-of select='./options/extractAttributes'/>;
          </xsl:if>
          <xsl:if test='./options/singletile'>
            objOptions['singleTile'] =<xsl:value-of select='./options/singletile'/>;
          </xsl:if>
          <xsl:if test='./options/sphericalmercator'>
            objOptions['sphericalMercator'] =<xsl:value-of select='./options/sphericalmercator'/>;
          </xsl:if>
		  <xsl:if test="./options/minzoomlevel">
            objOptions['minZoomLevel'] = <xsl:value-of select='./options/minzoomlevel'/>;
          </xsl:if>
          <xsl:if test="./options/transitioneffect">
            objOptions['transitionEffect'] = '<xsl:value-of select='./options/transitioneffect'/>';
          </xsl:if>
          <xsl:if test="./options/visibility">
            objOptions['visibility'] = <xsl:value-of select='./options/visibility'/>;
          </xsl:if>
          <xsl:if test="./options/scales">
            objOptions['scales'] = Array(<xsl:value-of select='./options/scales'/>);
          <xsl:if test="./options/resolutions">
            objOptions['resolutions'] = Array(<xsl:value-of select='./options/resolutions'/>);
        </xsl:if>
          </xsl:if>
          <xsl:if test="./options/buffer">
            objOptions['buffer'] = <xsl:value-of select='./options/buffer'/>;
          </xsl:if>
          <xsl:if test="./options/ratio">
            objOptions['ratio'] = <xsl:value-of select='./options/ratio'/>;
          </xsl:if>
          <xsl:if test="./options/wrapdateline">
            objOptions['wrapDateLine'] = <xsl:value-of select='./options/wrapdateline'/>;
          </xsl:if>
          <xsl:if test="./options/group">
            objOptions['group'] = '<xsl:value-of select="./options/group"/>';
          </xsl:if>

	    objOptions['serviceType'] = '<xsl:value-of select="$pServiceType"/>';
          <!-- 
               The layersByResource property is used to have a reference to
               the DataStore layers to the created OpenLayers.Layer object, 
               because sometimes a layer "params.LAYERS" property changes...
            -->
          objOptions["layersByResource"] = {};
          <xsl:for-each select="./resourcenames/resourcename">
            
            <xsl:variable name="strParamsLayers">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="."/>
              </xsl:call-template>
            </xsl:variable>
            
            objOptions["layersByResource"]["<xsl:value-of select="."/>"] = "<xsl:value-of select='$strParamsLayers'/>";
          </xsl:for-each>

          <!--
              Check the layername property.  If inexistant, take the resourceName
              todo : if no layername and no resourcename == bug
            -->
          <xsl:if test='not(./options/layername)'>
            strLayerName = "<xsl:value-of select='$pResourceName'/>";
          </xsl:if>
          <xsl:if test='./options/layername'>
            strLayerName = "<xsl:value-of select='./options/layername'/>";
          </xsl:if>


          <!--
              Build the layer params array
            -->

          <!-- version & exceptions -->
          <xsl:if test="$serviceVersion != ''">
            objParams["version"] = '<xsl:value-of select="$serviceVersion" />';

            <!-- temp fix, related to http://trac.openlayers.org/ticket/2478 -->
            <xsl:if test="$serviceVersion = '1.3.0'">
              objParams["exceptions"] = "INIMAGE";
            </xsl:if>
          </xsl:if>

          <xsl:if test='./options/format'>
            objParams["format"] = "<xsl:value-of select='./options/format'/>";
          </xsl:if>
          <xsl:if test='./options/maxfeatures'>
            objParams["maxfeatures"] = "<xsl:value-of select='./options/maxfeatures'/>";
          </xsl:if>
          <xsl:if test='./options/transparent'>
            objParams["transparent"] = "<xsl:value-of select='./options/transparent'/>";
          </xsl:if>
          <xsl:if test='./options/typename'>
            objParams["typename"] = "<xsl:value-of select='./options/typename'/>";
          </xsl:if>



          objOLLayer = '';

          <xsl:choose>
            <xsl:when test="$pServiceType='wms'">
              objParams["layers"] = [];
              objResources = [];
              <xsl:for-each select="./resourcenames/resourcename">


                <xsl:variable name="strParamsLayers">
                  <xsl:call-template name="getDataStoreInfoFromWidget">
                    <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                    <xsl:with-param name="pServiceType" select="$pServiceType" />
                    <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                    <xsl:with-param name="pResourceName" select="."/>
                  </xsl:call-template>
                </xsl:variable>

                objParams["layers"].push("<xsl:value-of select='$strParamsLayers'/>");
                objResources.push("<xsl:value-of select='.'/>");

              </xsl:for-each>

              if(!boolURLAliasesEnabled) {
                  objArrayURLAliases = strURL;
              }

              <!-- 'vendor' service option -->
              objOptions.vendor = "<xsl:value-of select="/geoprisma/services/service[./name = $pServiceName]/options/vendor" />".toLowerCase();
              
              objOLLayer = new OpenLayers.Layer.WMS(strLayerName, objArrayURLAliases, objParams, objOptions );
              <xsl:if test='./options/opacity'>
                objOLLayer.setOpacity( <xsl:value-of select='./options/opacity'/> );
              </xsl:if>
              objArrayLayers.push(objOLLayer);
              objOLLayer.resources = objResources;
            </xsl:when>
            <xsl:when test="$pServiceType='tilecache'">
              objParams["layers"] = "<xsl:value-of select='$pLayerName'/>";

              if(!boolURLAliasesEnabled) {
                  objArrayURLAliases = strURL;
              }

              <!-- if service 'staticCache' option was set to 'true', then
                   the cache is accessed directly usint the 'TileCache' layer
                   object.
                -->
              <xsl:choose>
                <xsl:when test="$serviceStaticCache='true'">
                  <xsl:if test='./options/format'>
                   objOptions["format"] = "<xsl:value-of select='./options/format'/>";
                  </xsl:if>
                  if (objArrayURLAliases instanceof Array) {
                      for(var key in objArrayURLAliases) {
                          objArrayURLAliases[key] =  OpenLayers.Util.urlAppend(objArrayURLAliases[key], "staticCache=");
                      }
                  } else {
                      objArrayURLAliases = OpenLayers.Util.urlAppend(objArrayURLAliases, "staticCache=");
                  }
                objOLLayer = new OpenLayers.Layer.TileCache(strLayerName, objArrayURLAliases, "<xsl:value-of select='$pLayerName'/>", objOptions );
                </xsl:when>
                <xsl:otherwise>
                objOLLayer = new OpenLayers.Layer.WMS(strLayerName, objArrayURLAliases, objParams, objOptions );
                </xsl:otherwise>
              </xsl:choose>

              <xsl:if test='./options/opacity'>
                objOLLayer.setOpacity( <xsl:value-of select='./options/opacity'/> );
              </xsl:if>
              objArrayLayers.push(objOLLayer);
              objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
            </xsl:when>
            <xsl:when test="$pServiceType='featureserver'">
              <![CDATA[
              var aszURLArray = strURL.split("?");
              strURL = aszURLArray[0]+"/]]><xsl:value-of select='$pLayerName'/><![CDATA[?"+aszURLArray[1];
              objOptions['strategies'] = [ ];

              ]]>
              <xsl:choose>
	              <xsl:when test='./options/strategies'>
		              <xsl:if test='./options/strategies/bbox'> 
		                objOptions['strategies'].push( new OpenLayers.Strategy.BBOX() );
		              </xsl:if>
		              <xsl:if test='./options/strategies/fixed'> 
		                objOptions['strategies'].push( new OpenLayers.Strategy.Fixed() );
		              </xsl:if>
		              <xsl:if test='./options/strategies/cluster'>
		                objOptions['strategies'].push( new OpenLayers.Strategy.Cluster() );
		              </xsl:if>
		          </xsl:when>
	              <xsl:otherwise>
	              	<!-- defaults to BBOX strategy if no strategies element is specified -->
	              	objOptions['strategies'].push( new OpenLayers.Strategy.BBOX() );
	              </xsl:otherwise>
              </xsl:choose>
              <![CDATA[

        objOptions["protocol"] = new mapfish.Protocol.MapFish({
            url: mapfish.Util.formatURL(strURL),
            params: {format: "GeoJSON"},
            format: new OpenLayers.Format.GeoJSON()
        });

// old version of the protocol
/*
              objOptions["protocol"] = new OpenLayers.Protocol.HTTP({
                        url: strURL,
                        params: {
                            format: "GeoJSON",
                            //format: "WFS", // changed by adube
                            service: "WFS",
                            request: "GetFeatures",
                            srs: objOptions["projection"]?objOptions["protocol"]:"",
                            maxfeatures: objParams["maxfeatures"]
                        },
                        //format: new OpenLayers.Format.GML() //changed by adube
                        format: new OpenLayers.Format.GeoJSON()
                    });

              ]]>
*/

              <!-- STYLEMAP  -->
              <xsl:for-each select='./options/stylemap'>
                objOptions["styleMap"] = new OpenLayers.StyleMap({});

                <!-- STYLES  -->
                <xsl:for-each select='./renderers/renderer'>
                    objOptions["styleMap"].styles["<xsl:value-of select='./renderintent'/>"] = new OpenLayers.Style(OpenLayers.Util.applyDefaults(
                    <xsl:call-template name="map:getStyleObject">
                      <xsl:with-param name="pStyle" select="./style" />
                    </xsl:call-template>, 
                    OpenLayers.Feature.Vector.style["<xsl:value-of select='./renderintent'/>"]),
                    <xsl:call-template name="map:getStyleContext">
                      <xsl:with-param name="pStyle" select="./style" />
                    </xsl:call-template>);
                </xsl:for-each>

                <!-- UNIQUE VALUE RULES  -->
                <xsl:for-each select='./uniquevaluerules/uniquevaluerule'>
                  objSymbolizers = {};

                  <xsl:for-each select='./symbolizers/symbolizer'>
                    objSymbolizers['<xsl:value-of select='./value'/>'] =   
                    <xsl:call-template name="map:getStyleObject">
                      <xsl:with-param name="pStyle" select="./style" />
                    </xsl:call-template>;
                  </xsl:for-each>

                  <!-- TODO, should support the 4th 'context' parameter some day... -->
                  objOptions["styleMap"].addUniqueValueRules(
                    '<xsl:value-of select='./renderintent'/>',
                    '<xsl:value-of select='./property'/>',
                    objSymbolizers);

                  <!-- Add an other symbolizer for the features that don't
                       fall under one of the rules created above.  Useful to
                       keep the vertices when editing.  TODO: theses values
                       are currently HARDCODED.
                    -->
                  rules = [new OpenLayers.Rule({
                      symbolizer: {strokeColor:"red",strokeWidth: 2},
                      elseFilter: true
                  })];
                  objOptions["styleMap"].styles["<xsl:value-of select='./renderintent'/>"].addRules(rules);
                </xsl:for-each>

              </xsl:for-each>

              <![CDATA[
              objOLLayer = new OpenLayers.Layer.Vector(strLayerName, objOptions );
              objArrayLayers.push(objOLLayer);

              objOLLayer.resources = ["]]><xsl:value-of select='$pResourceName'/><![CDATA["];
              ]]>
            </xsl:when>
            <xsl:when test="$pServiceType='gymo'">
              
              <xsl:variable name="strGYMOProvider">
                <xsl:value-of select="//services/service[name=$pServiceName]/provider" />
              </xsl:variable>
              <xsl:variable name="strGYMOSource">
                <xsl:value-of select="//services/service[name=$pServiceName]/source" />
              </xsl:variable>

              <xsl:choose>
                <xsl:when test="$strGYMOProvider='google'">
                  <xsl:choose>
                    <xsl:when test="$pLayerName='satellite'">
                      objOptions['type'] = google.maps.MapTypeId.SATELLITE;
                    </xsl:when>
                    <xsl:when test="$pLayerName='hybrid'">
                      objOptions['type'] = google.maps.MapTypeId.HYBRID;
                    </xsl:when>
                    <xsl:when test="$pLayerName='terrain'">
                      objOptions['type'] = google.maps.MapTypeId.TERRAIN;
                    </xsl:when>
                    <!-- 'physical' was v2 value for 'terrain' -->
                    <xsl:when test="$pLayerName='physical'">
                      objOptions['type'] = google.maps.MapTypeId.TERRAIN;
                    </xsl:when>
                  </xsl:choose>
                  objOLLayer = new OpenLayers.Layer.Google(strLayerName, objOptions );
                  objArrayLayers.push(objOLLayer);
                  objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
                </xsl:when>

                 <xsl:when test="$strGYMOProvider='openstreetmap'">
                  strURL = "<xsl:value-of select='$strGYMOSource'/>";
                  objOLLayer = new OpenLayers.Layer.OSM(strLayerName, strURL, objOptions);
                  objArrayLayers.push(objOLLayer);

                  objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
                </xsl:when>

                <xsl:when test="$strGYMOProvider='bing'">
                  <xsl:choose>
                    <xsl:when test="$pLayerName='shaded'">
                      objOptions['type'] = VEMapStyle.Shaded;
                    </xsl:when>
                    <xsl:when test="$pLayerName='hybrid'">
                      objOptions['type'] = VEMapStyle.Hybrid;
                    </xsl:when>
                    <xsl:when test="$pLayerName='aerial'">
                      objOptions['type'] = VEMapStyle.Aerial;
                    </xsl:when>
                  </xsl:choose>
                  objOLLayer = new OpenLayers.Layer.VirtualEarth(strLayerName, objOptions );
                  objArrayLayers.push(objOLLayer);

                  objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
                </xsl:when>

                <xsl:when test="$strGYMOProvider='virtualearth'">
                  <xsl:choose>
                    <xsl:when test="$pLayerName='road'">
                      objOptions['type'] = VEMapStyle.Road;
                    </xsl:when>
                    <xsl:when test="$pLayerName='hybrid'">
                      objOptions['type'] = VEMapStyle.Hybrid;
                    </xsl:when>
                    <xsl:when test="$pLayerName='aerial'">
                      objOptions['type'] = VEMapStyle.Aerial;
                    </xsl:when>
                  </xsl:choose>
                  objOptions['animationEnabled'] = false;
                  objOLLayer = new OpenLayers.Layer.VirtualEarth(strLayerName, objOptions );

                  objArrayLayers.push(objOLLayer);

                  objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
                </xsl:when>

                <xsl:when test="$strGYMOProvider='yahoo'">
                  <xsl:choose>
                    <xsl:when test="$pLayerName='street'">
                    </xsl:when>
                    <xsl:when test="$pLayerName='hybrid'">
                      objOptions['type'] = YAHOO_MAP_HYB;
                    </xsl:when>
                    <xsl:when test="$pLayerName='satellite'">
                      objOptions['type'] = YAHOO_MAP_SAT;
                    </xsl:when>
                  </xsl:choose>
                  objOLLayer = new OpenLayers.Layer.Yahoo(strLayerName, objOptions );
                  objArrayLayers.push(objOLLayer);

                  objOLLayer.resources = ["<xsl:value-of select='$pResourceName'/>"];
                </xsl:when>


              </xsl:choose>
            </xsl:when>
            <xsl:when test="$pServiceType='wfs'">
              <!-- 'layer' params -->
              <xsl:variable name="layerParams">
                <xsl:call-template name="getDataStoreInfoFromWidget">
                  <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                  <xsl:with-param name="pServiceType" select="$pServiceType" />
                  <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                  <xsl:with-param name="pResourceName" select="$pResourceName"/>
                </xsl:call-template>
              </xsl:variable>

              <xsl:if test='not(./options/layername)'>
                strLayerName = "<xsl:value-of select='$pResourceName'/>";
              </xsl:if>
              <xsl:if test='./options/layername'>
                strLayerName = "<xsl:value-of select='./options/layername'/>";
              </xsl:if>

              <![CDATA[
              var aszURLArray = strURL.split("?");
              strURL = aszURLArray[0]+"/]]><xsl:value-of select='$layerParams'/><![CDATA[?"+aszURLArray[1];
              ]]>

              <!-- options -->
              var objOptions =
              <xsl:call-template name="getObjectFromNode">
                <xsl:with-param name="pNode" select="." />
              </xsl:call-template>;
              objOptions.strategies = [];

              objOptions.resources = ['<xsl:value-of select="$pResourceName" />'];
              objOptions.strategies.push (new OpenLayers.Strategy.BBOX());
              objOptions.protocol = new OpenLayers.Protocol.WFS({
                  url: strURL,
                  version: objOptions["version"] ? objOptions["version"] : undefined,
                  srsName: objOptions["projectionString"] ? objOptions["projectionString"] : undefined,
                  featureType: "<xsl:value-of select="$layerParams" />",
                  featureNS: objOptions.options["featureNS"] ? objOptions.options["featureNS"] : undefined,
                  featurePrefix: objOptions["featurePrefix"] ? objOptions["featurePrefix"] : undefined
              });

              objLayer = new OpenLayers.Layer.Vector(strLayerName, objOptions);
              objArrayLayers.push(objLayer);
            </xsl:when>
          </xsl:choose>

        <!--xsl:if-->
        <!-- End if ResouceAccess == true -->

      </xsl:for-each>

        <xsl:variable name='jsGlobalVariable'>
          <xsl:text>var objGPWidget</xsl:text>
          <xsl:value-of select="$pWidgetName" />
          <xsl:text>= oMap;</xsl:text>
        </xsl:variable>

        <xsl:value-of select="$jsGlobalVariable"/>

        <!-- add layers unless 'addLayers' option is set to false -->
        <xsl:choose>
          <xsl:when test="./options/addLayers = 'false'">
            <!-- do nothing -->
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="map:addLayers" >
              <xsl:with-param name="pMapName">
                <xsl:value-of select="$pWidgetName" />
              </xsl:with-param>
            </xsl:call-template>
          </xsl:otherwise>
        </xsl:choose>

      </script>

    </xsl:for-each>
  </xsl:template>

  <!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <xsl:template name="map:drawWidget">
  </xsl:template>


  <!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <xsl:template name="map:getWidgetDepedencies">

  </xsl:template>

  <!--
     Function: addLayers
     
     Add the layers in the 'LayersToAdd' array to the map.  Is automatically
     called if the 'addLayers' map option is set to 'true'.
    -->
  <xsl:template name="map:addLayers">
    <xsl:param name="pMapName" />
        objGPWidget<xsl:value-of select="$pMapName" />.addLayers(
            objGPWidget<xsl:value-of select="$pMapName" />LayersToAdd
        );
  </xsl:template>

  <!--
    Function: getStyleObject
    Given a <style> node, build a javascript object with "property: 'value',"
    properties.  It can then be used to create OpenLayers.Style() objects or
    defining rules.

    Parameters:
    {node}   - pStyle    A <style> node

    Returns:
    A javascript object ready to create OpenLayers objects, like Style, Rules...
    Example of return :  { strokeOpacity: 1, strokeColor: #ffee22, last: '' }
    -->
  <xsl:template name="map:getStyleObject">
    <xsl:param name="pStyle" />
    
        {
    <xsl:for-each select='$pStyle/*'>  <!-- Switch to style context - only one -->
           <xsl:if test="position()>1">,</xsl:if>
           <xsl:choose>
                <xsl:when test="name() = 'fillcolor'">fillColor: "#<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'fillopacity'">fillOpacity: "<xsl:value-of select='.'/>"</xsl:when>
                <!-- pointRadius == manual pointRadius -->
                <xsl:when test="name() = 'pointradius'">pointRadius: "<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'strokedashstyle'">strokeDashstyle: "<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'strokelinecap'">strokeLinecap: "<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'strokeopacity'">strokeOpacity: "<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'strokewidth'">strokeWidth: "<xsl:value-of select='.'/>"</xsl:when>
                <xsl:when test="name() = 'strokecolor'">strokeColor: "#<xsl:value-of select='.'/>"</xsl:when>
                <!-- radius == automatic pointRadius -->    
                <xsl:when test="name() = 'radius'"> pointRadius: "${radius}"</xsl:when>
                <xsl:otherwise>
                    'bidon' : null  <!-- fix for ie extra ',' -->    
                </xsl:otherwise>
	   </xsl:choose>
     </xsl:for-each>
         }
  </xsl:template>
  
  <xsl:template name="map:getStyleContext">
    <xsl:param name="pStyle" />
        {   
            context: {
                first: function(feature){return ''}

                <xsl:for-each select='$pStyle'>  <!-- Switch to style context -->
                
                    <xsl:for-each select='context/functions/*'>
                      <xsl:choose>

                        <!-- either attributegualfunction or 
                             fidegualfunction -->
                        <xsl:when test="count(./egual)!=0">
                          <xsl:choose>

                            <!-- attributegualfunction -->
                            <xsl:when test="count(./attribute)!=0">
                                <xsl:variable name="egual">
                                    <xsl:choose>            
                                        <xsl:when test="egual/getValueFrom">
                                            <xsl:for-each select='egual/getValueFrom'><xsl:call-template name="getValueFrom" /></xsl:for-each>    
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select='egual'/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                , <xsl:value-of select='name'/> : function(feature) {
                                    if (feature.attributes['<xsl:value-of select='attribute'/>'] == '<xsl:value-of select='$egual'/>') return '<xsl:value-of select='result'/>'; else return '<xsl:value-of select='defaultresult'/>';  
                                }
                            </xsl:when>
 
                            <!-- fidegualfunction -->
                            <xsl:otherwise>
                                <xsl:variable name="egual">
                                    <xsl:choose>            
                                        <xsl:when test="egual/getValueFrom">
                                            <xsl:for-each select='egual/getValueFrom'><xsl:call-template name="getValueFrom" /></xsl:for-each>    
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select='egual'/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                , <xsl:value-of select='name'/> : function(feature) {
                                    if (feature.fid == '<xsl:value-of select='$egual'/>') return '<xsl:value-of select='result'/>'; else return '<xsl:value-of select='defaultresult'/>';  
                                }
                            </xsl:otherwise>
                          </xsl:choose>
                        </xsl:when>
                        <!-- standard function -->
                        <xsl:otherwise>
                        /* xxxxxxx standard function */

                            , <xsl:value-of select='name'/> : function(feature) {
                                <xsl:value-of select='code'/>     
                            }
                        </xsl:otherwise>
                      </xsl:choose>                       
                        
                    </xsl:for-each>  
                    
                    <xsl:if test="radius">
                        ,radius: function(feature) {
                            <xsl:if test="maxradius">
                                if ( typeof(feature.attributes) != 'undefined' &amp;&amp; typeof(feature.attributes.count) != 'undefined')
                                {
                                    return Math.min(feature.attributes.count, <xsl:value-of select='maxradius'/>) + <xsl:value-of select='radius'/>;
                                }
                                else
                                {
                                    return <xsl:value-of select='radius'/>;
                                }                            
                            </xsl:if>
                            <xsl:if test="not(maxradius)">
                                if ( typeof(feature.attributes) != 'undefined' &amp;&amp; typeof(feature.attributes.count) != 'undefined')
                                {
                                    return feature.attributes.count + <xsl:value-of select='radius'/>;
                                }
                                else
                                {
                                    return <xsl:value-of select='radius'/>;
                                }
                            </xsl:if>
                        }
                    </xsl:if>  
                </xsl:for-each>    
            }  
        }
  </xsl:template>

</xsl:stylesheet>
