<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:querybyrect="http://geoprisma.org/querybyrect"
    extension-element-prefixes="dyn">
  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="querybyrect:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>querybyrect</xsl:text>
    </xsl:variable>

    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/querybyrect/QueryByRect.js</xsl:text>
        </xsl:attribute>    
    </script>
    
    <!-- Traduction (utilisation de la méthode extjs)
                http://extjs.com/learn/Tutorial:Localizing_Ext#Application_Localization
    --> 
    <script>            
        Ext.apply(OpenLayers.Control.QueryByRect.prototype, {
            i18n_wait_options_msg :           <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_wait_options_msg','JS')" />,
            i18n_wait_options_progress_text : <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_wait_options_progress_text','JS')" />,
            i18n_wait_options_completed_text : <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_wait_options_completed_text','JS')" />,
            i18n_wait_options_of_text : <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_wait_options_of_text','JS')" />,
            i18n_no_layer_queryable :         <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_no_layer_queryable','JS')" />,
            i18n_warning :                    <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_warning','JS')" />,
            i18n_message_no_result_msg :      <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_message_no_result_msg','JS')" />,
            i18n_message_no_result_title :    <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_message_no_result_title','JS')" />
        });   
    </script>
    
    <script type="text/javascript">
      var objGPWidget<xsl:value-of select="$WidgetType" />GeoExtToolbarOptions;
    </script>
  </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="querybyrect:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <!-- get draw mode  -->
    <xsl:variable name="pDrawMode">
      <xsl:value-of select="$drawmode" />
    </xsl:variable>

    <!-- get proxy url -->
    <xsl:variable name="pProxyURL">
      <xsl:value-of select="$proxyurl" />
    </xsl:variable>

    <!-- javascript START -->
    <script type="text/javascript">
        // QueryByRect

          var aResources = [];
          <!-- for each resource this widget is assign to, add its corresponding layer to the control  -->
          <xsl:for-each select="./resources/resource">
            <!-- Resoruce Name -->
            <xsl:variable name="resourceName">
              <xsl:value-of select="." />
            </xsl:variable>

            <!-- ServiceTypes supported by this widget -->
            <xsl:variable name="pFSServiceType">
              <xsl:text>featureserver</xsl:text>
            </xsl:variable>
            <xsl:variable name="pWFSServiceType">
              <xsl:text>wfs</xsl:text>
            </xsl:variable>

            <!-- Use the DataStoreName to validate that the resource has a valid
                 service type, either 'featureserver' or 'wfs'
              -->
            <xsl:variable name="pFSDataStoreName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pFSServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>name</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="pWFSDataStoreName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pWFSServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>name</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:variable name="pDataStoreName">
              <xsl:choose>
                <xsl:when test="$pFSDataStoreName != ''">
                  <xsl:value-of select="$pFSDataStoreName" />
	        </xsl:when>
	        <xsl:otherwise>
                  <xsl:value-of select="$pWFSDataStoreName" />
	        </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>

            <!-- If a DataStoreName was returned, that means the resource has
                 a valid service type.
              -->
            <xsl:if test="$pDataStoreName!=''">

              <!-- ServiceType -->
              <xsl:variable name="pServiceType">
                <xsl:choose>
                  <xsl:when test="$pFSDataStoreName != ''">
                    <xsl:value-of select="$pFSServiceType" />
	          </xsl:when>
	          <xsl:otherwise>
                    <xsl:value-of select="$pWFSServiceType" />
	          </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <!-- Resource Title -->
              <xsl:variable name="resourceTitle">
                <xsl:for-each select="/geoprisma/resources/resource[./name = $resourceName]/title">
                  <xsl:call-template name="getText" />
                </xsl:for-each>
              </xsl:variable>

              <!-- get service name -->
              <xsl:variable name="pServiceName">
                <xsl:call-template name="getDataStoreInfoFromWidget">
                  <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                  <xsl:with-param name="pServiceType" select="$pServiceType" />
                  <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
                  <xsl:with-param name="pResourceName" select="$resourceName" />
                </xsl:call-template>
              </xsl:variable>

              <!-- get layer name -->
              <xsl:variable name="pLayerName">
                <xsl:call-template name="getDataStoreInfoFromWidget">
                  <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                  <xsl:with-param name="pServiceType" select="$pServiceType" />
                  <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                  <xsl:with-param name="pResourceName" select="$resourceName" />
                </xsl:call-template>
              </xsl:variable>

            <!-- DataStoreText -->
            <xsl:variable name="pDataStoreText">
              <xsl:variable name="temp">
                  <xsl:call-template name="getDataStoreInfoFromWidget">
                    <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                    <xsl:with-param name="pServiceType" select="$pServiceType" />
                    <xsl:with-param name="pNodeName"><xsl:text>params/text</xsl:text></xsl:with-param>
                    <xsl:with-param name="pResourceName" select="$resourceName" />
                  </xsl:call-template>
              </xsl:variable>   
              <xsl:choose>
                <xsl:when test="$temp!=''">
                  <xsl:value-of select="$temp"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>''</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>

            szParams = OpenLayers.Util.getParameterString({
                'osmresource': "<xsl:value-of select="$resourceName" />",
                'osmservice': "<xsl:value-of select="$pServiceName" />"
            });

            var oProtocol;
            <xsl:choose>
              <xsl:when test="$pServiceType = 'featureserver'">
                oProtocol = new OpenLayers.Protocol.HTTP({
                    url: "<xsl:value-of select="$pProxyURL" />"+"/"+"<xsl:value-of select="$pLayerName" />"+"?"+szParams,
                    params: {
                        format: "GeoJSON",
                        service: "WFS",
                        request: "GetFeatures",
                        srs: oMap.getProjection()
                    },
                    format: new OpenLayers.Format.GeoJSON()
                });
	      </xsl:when>
              <xsl:when test="$pServiceType = 'wfs'">
                oProtocol = new OpenLayers.Protocol.WFS({
                    url: OpenLayers.Util.urlAppend("<xsl:value-of select="$pProxyURL" />", szParams),
                    featureType: "<xsl:value-of select="$pLayerName" />"
                });
	      </xsl:when>
	      <xsl:otherwise />
            </xsl:choose>

            aResources.push({
                'resource': "<xsl:value-of select="$resourceName" />",
                'resourceTitle': <xsl:value-of select="$resourceTitle" />,
                'protocol': oProtocol,
                'queryable': true
            });
            </xsl:if>
          </xsl:for-each> <!-- end for-each resource -->
	<!--Quick reference to the map object-->
        oMap = objGPWidget<xsl:value-of select="$pMapName" />;

        strTitle = <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_button_tooltip','JS')" />;

        <!-- get resetOnDeactivation  -->
        <xsl:variable name="pResetOnDeactivation">
          <xsl:call-template name="getWidgetOptionByName">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pOptionName">
              <xsl:text>resetOnDeactivation</xsl:text>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>
        var strReset = '<xsl:value-of select="$pResetOnDeactivation" />';
        strReset = (strReset == '') ? 'nothing' : strReset;

        <!-- get dropDownList  -->
        <xsl:variable name="pDropDownList">
          <xsl:call-template name="getWidgetOptionByName">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pOptionName">
              <xsl:text>dropDownList</xsl:text>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

          var objMarkerStyle;
          <xsl:for-each select="./options/markerStyle">
          objMarkerStyle =
            <xsl:call-template name="getObjectFromNode">
              <xsl:with-param name="pNode" select="." />
            </xsl:call-template>
          </xsl:for-each>;

          <!-- creation of the control -->
          objGPWidget<xsl:value-of select="$pWidgetName" /> = new OpenLayers.Control.QueryByRect({
              drawMode: '<xsl:value-of select="$pDrawMode" />',
              resetOnDeactivation: strReset,
              markerStyle: objMarkerStyle,
              renderform: false,
              title: strTitle,
              resources: aResources
          });

          <!-- GeoExt.Action object for the geoexttoolbar START -->
	  aoMenuItems = [];
	  <!--Text for the arrow menu-->
	  for (var i=0, len=aResources.length; i&lt;len; i++){
              oResource = aResources[i];

	      if (oResource.resourceTitle != ''){
                  aoMenuItems.push({'text': oResource.resourceTitle});
              } else {
                  aoMenuItems.push({'text': oResource.resource});
              }
	  }

          objAction = new GeoExt.Action({
            iconCls: 'querybyrect',
            control: objGPWidget<xsl:value-of select="$pWidgetName" />,
            map: oMap,
            // button options
            toggleGroup: oMap.id,
            allowDepress: false,
            tooltip: strTitle,
            // check item options
            <xsl:if test="$pDropDownList != 'false'">
	    menu: {
              defaultType: 'tbtext',
	      items: aoMenuItems
            },
            </xsl:if>
            group: oMap.id
          });

          <!-- Widget Object creation  -->
          <xsl:choose>
            <xsl:when test="$pDropDownList = 'false'">
              objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions = objAction;
            </xsl:when>
            <xsl:otherwise>
              objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions = new Ext.Toolbar.SplitButton(objAction);
	      objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions.arrowTooltip = <xsl:value-of select="php:functionString('QueryByRect::getText','i18n_button_arrow_tooltip','JS')" />;
            </xsl:otherwise>
          </xsl:choose>
          <!-- GeoExt.Action object for the geoexttoolbar END -->

    </script>
    <!-- javascript END -->
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="querybyrect:drawWidget">
    <xsl:param name="pWidgetName" />

  </xsl:template>


<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="querybyrect:getWidgetDepedencies">
<!-- TODO : this widget has a toolbar and map dependency -->
  </xsl:template>

</xsl:stylesheet>
