<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:queryonclick="http://geoprisma.org/queryonclick"
    extension-element-prefixes="dyn">
  <xsl:output method="html" indent="yes" encoding="utf-8" omit-xml-declaration="yes"/>
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="queryonclick:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>queryonclick</xsl:text>
    </xsl:variable>

    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/queryonclick/QueryOnClick.js</xsl:text>
        </xsl:attribute>    
    </script>
    
    <!-- Traduction (utilisation de la méthode extjs)
                http://extjs.com/learn/Tutorial:Localizing_Ext#Application_Localization
    --> 
    <script>            
        Ext.apply(OpenLayers.Control.QueryOnClick.prototype, {
            i18n_wait_options_msg :           <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_wait_options_msg','JS')" />,
            i18n_wait_options_progress_text : <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_wait_options_progress_text','JS')" />,
            i18n_wait_options_completed_text : <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_wait_options_completed_text','JS')" />,
            i18n_wait_options_of_text : <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_wait_options_of_text','JS')" />,
            i18n_no_layer_queryable :         <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_no_layer_queryable','JS')" />,
            i18n_warning :                    <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_warning','JS')" />,
            i18n_message_no_result_msg :      <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_message_no_result_msg','JS')" />,
            i18n_message_no_result_title :    <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_message_no_result_title','JS')" />
        });   
    </script>
    
    <script type="text/javascript">
      var objGPWidget<xsl:value-of select="$WidgetType" />GeoExtToolbarOptions;
    </script>
  </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="queryonclick:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <!-- get draw mode  -->
    <xsl:variable name="pDrawMode">
      <xsl:value-of select="$drawmode" />
    </xsl:variable>

    <!-- get proxy url -->
    <xsl:variable name="pProxyURL">
      <xsl:value-of select="$proxyurl" />
    </xsl:variable>

    <!-- javascript START -->
    <script type="text/javascript">
        // QueryOnClick

          var aResources = [];
          <!-- for each resource this widget is assign to, add its corresponding layer to the control  -->
          <xsl:for-each select="./resources/resource">
            <!-- Resoruce Name -->
            <xsl:variable name="resourceName">
              <xsl:value-of select="." />
            </xsl:variable>

            <!-- Resource Title -->
            <xsl:variable name="resourceTitle">
              <xsl:for-each select="/geoprisma/resources/resource[./name = $resourceName]/title">
                <xsl:call-template name="getText" />
              </xsl:for-each>
            </xsl:variable>

            <!-- Resource 'primaryField' option -->
            <xsl:variable name="resourcePrimaryField">
              <xsl:value-of select="/geoprisma/resources/resource[./name = $resourceName]/options/primaryField" />
            </xsl:variable>

            <!-- WMS support -->
            <xsl:variable name="pServiceType">
              <xsl:text>wms</xsl:text>
            </xsl:variable>

            <!-- get service name -->
            <xsl:variable name="pServiceName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName" />
              </xsl:call-template>
            </xsl:variable>

            <!-- get layer name -->
            <xsl:variable name="pLayerName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName" />
              </xsl:call-template>
            </xsl:variable>

            <!-- DataStoreText -->
            <xsl:variable name="pWMSDataStoreText">
              <xsl:variable name="temp">
                  <xsl:call-template name="getDataStoreInfoFromWidget">
                    <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                    <xsl:with-param name="pServiceType" select="$pServiceType" />
                    <xsl:with-param name="pNodeName"><xsl:text>params/text</xsl:text></xsl:with-param>
                    <xsl:with-param name="pResourceName" select="$resourceName" />
                  </xsl:call-template>
              </xsl:variable> 
              <xsl:choose>
                <xsl:when test="$temp!=''">
                  <xsl:value-of select="$temp"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>''</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>

            <!-- Service version (option) -->
            <xsl:variable name="serviceVersion">
              <xsl:value-of select="/geoprisma/services/service[./name = $pServiceName]/options/version" />
            </xsl:variable>

            <!-- Service info_format (option) -->
            <xsl:variable name="serviceInfoFormat">
              <xsl:value-of select="/geoprisma/services/service[./name = $pServiceName]/options/info_format" />
            </xsl:variable>

            <xsl:variable name="pWMSConfig">
              wms: {'queryLayers': "<xsl:value-of select="$pLayerName" />",
                    'service': "<xsl:value-of select="$pServiceName" />",
                    'url': "<xsl:value-of select="$pProxyURL" />",
              <!-- version & exceptions -->
              <xsl:if test="$serviceVersion != ''">
                    'version': '<xsl:value-of select="$serviceVersion" />',
              </xsl:if>
              <!-- info_format -->
              <xsl:if test="$serviceInfoFormat != ''">
                    'info_format': '<xsl:value-of select="$serviceInfoFormat" />',
              </xsl:if>
                    'dataStoreText': <xsl:value-of select="$pWMSDataStoreText" />},
            </xsl:variable>

        <!-- WFS support, TODO: check more than featureserver -->
            <xsl:variable name="pWFSServiceType">
              <xsl:text>featureserver</xsl:text>
            </xsl:variable>

            <!-- get service name -->
            <xsl:variable name="pWFSServiceName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pWFSServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName" />
              </xsl:call-template>
            </xsl:variable>

            <!-- get layer name -->
            <xsl:variable name="pWFSLayerName">
              <xsl:call-template name="getDataStoreInfoFromWidget">
                <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                <xsl:with-param name="pServiceType" select="$pWFSServiceType" />
                <xsl:with-param name="pNodeName"><xsl:text>params/layers</xsl:text></xsl:with-param>
                <xsl:with-param name="pResourceName" select="$resourceName" />
              </xsl:call-template>
            </xsl:variable>

            <!-- DataStoreText -->
            <xsl:variable name="pWFSDataStoreText">
              <xsl:variable name="temp">
                  <xsl:call-template name="getDataStoreInfoFromWidget">
                    <xsl:with-param name="pWidgetName" select="$pWidgetName" />
                    <xsl:with-param name="pServiceType" select="$pWFSServiceType" />
                    <xsl:with-param name="pNodeName"><xsl:text>params/text</xsl:text></xsl:with-param>
                    <xsl:with-param name="pResourceName" select="$resourceName" />
                  </xsl:call-template>
              </xsl:variable>   
              <xsl:choose>
                <xsl:when test="$temp!=''">
                  <xsl:value-of select="$temp"/>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>''</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:variable>


            <xsl:variable name="pWFSConfig">
              wfs: {'typename': "<xsl:value-of select="$pWFSLayerName" />",
                    'service': "<xsl:value-of select="$pWFSServiceName" />",
                    'url': "<xsl:value-of select="$pProxyURL" />",
                    'dataStoreText': <xsl:value-of select="$pWFSDataStoreText" />},
            </xsl:variable>

              oResource = {
                  <xsl:value-of select="$pWMSConfig" />
                  <xsl:value-of select="$pWFSConfig" />
                  'resource': "<xsl:value-of select="$resourceName" />",
                  'resourceTitle': <xsl:value-of select="$resourceTitle" />,
                  <xsl:if test="$resourcePrimaryField != ''">'resourcePrimaryField': '<xsl:value-of select="$resourcePrimaryField" />',</xsl:if>
                  'queryable': true
              };

              aResources.push(oResource);

          </xsl:for-each> <!-- end for-each resource -->
	<!--Quick reference to the map object-->
        oMap = objGPWidget<xsl:value-of select="$pMapName" />;

        strTitle = <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_button_tooltip','JS')" />;

        <!-- get resetOnDeactivation  -->
        <xsl:variable name="pResetOnDeactivation">
          <xsl:call-template name="getWidgetOptionByName">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pOptionName">
              <xsl:text>resetOnDeactivation</xsl:text>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>
        var strReset = '<xsl:value-of select="$pResetOnDeactivation" />';
        strReset = (strReset == '') ? 'marker' : strReset;

        <!-- get noMarker  -->
        <xsl:variable name="pNoMarker">
          <xsl:call-template name="getWidgetOptionByName">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pOptionName">
              <xsl:text>noMarker</xsl:text>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>
        var bNoMarker = '<xsl:value-of select="$pNoMarker" />';
        bNoMarker = (bNoMarker == 'true') ? true : false;

        <!-- get dropDownList  -->
        <xsl:variable name="pDropDownList">
          <xsl:call-template name="getWidgetOptionByName">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pOptionName">
              <xsl:text>dropDownList</xsl:text>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

          var objMarkerStyle;
          <xsl:for-each select="./options/markerStyle">
          objMarkerStyle =
            <xsl:call-template name="getObjectFromNode">
              <xsl:with-param name="pNode" select="." />
            </xsl:call-template>
          </xsl:for-each>;

          <!-- creation of the control -->
          objGPWidget<xsl:value-of select="$pWidgetName" /> = new OpenLayers.Control.QueryOnClick({
              drawMode: '<xsl:value-of select="$pDrawMode" />',
              resetOnDeactivation: strReset,
              markerStyle: objMarkerStyle,
              renderform: false,
              title: strTitle,
              noMarker: bNoMarker,
              <xsl:if test="./options/multipleKey">multipleKey: '<xsl:value-of select="./options/multipleKey" />',</xsl:if>
              <xsl:if test="./options/altKey">altKey: '<xsl:value-of select="./options/altKey" />',</xsl:if>
              <xsl:if test="./options/multiple">multiple: <xsl:value-of select="./options/multiple" />,</xsl:if>
              <xsl:if test="./options/toggle">toggle: <xsl:value-of select="./options/toggle" />,</xsl:if>
              <xsl:if test="./options/displayClass">displayClass: '<xsl:value-of select="./options/displayClass" />',</xsl:if>
              resources: aResources
          });

          <!-- GeoExt.Action object for the geoexttoolbar START -->
	  aoMenuItems = [];
	  <!--Text for the arrow menu-->
	  for (var i=0, len=aResources.length; i&lt;len; i++){
              oResource = aResources[i];

	      if(oResource['wms'].dataStoreText != ''){
                  aoMenuItems.push({'text': oResource['wms'].dataStoreText});
              } else if (oResource.resourceTitle != ''){
                  aoMenuItems.push({'text': oResource.resourceTitle});
              } else {
                  aoMenuItems.push({'text': oResource.resource});
              }
	  }

          objAction = new GeoExt.Action({
          <xsl:choose>
            <xsl:when test="./options/iconCls">
            iconCls: '<xsl:value-of select="./options/iconCls" />',
            </xsl:when>
            <xsl:otherwise>
	    iconCls: 'queryonclick',
            </xsl:otherwise>
          </xsl:choose>
            control: objGPWidget<xsl:value-of select="$pWidgetName" />,
            map: oMap,
            // button options
            toggleGroup: oMap.id,
            allowDepress: false,
            tooltip: strTitle,
            // check item options
            <xsl:if test="$pDropDownList != 'false'">
	    menu: {
              defaultType: 'tbtext',
	      items: aoMenuItems
            },
            </xsl:if>
            group: oMap.id
          });

          <!-- Widget Object creation  -->
          <xsl:choose>
            <xsl:when test="$pDropDownList = 'false'">
              objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions = objAction;
            </xsl:when>
            <xsl:otherwise>
              objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions = new Ext.Toolbar.SplitButton(objAction);
	      objGPWidget<xsl:value-of select="$pWidgetName" />GeoExtToolbarOptions.arrowTooltip = <xsl:value-of select="php:functionString('QueryOnClick::getText','i18n_button_arrow_tooltip','JS')" />;
            </xsl:otherwise>
          </xsl:choose>
          <!-- GeoExt.Action object for the geoexttoolbar END -->

    </script>
    <!-- javascript END -->
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="queryonclick:drawWidget">
    <xsl:param name="pWidgetName" />

  </xsl:template>


<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="queryonclick:getWidgetDepedencies">
<!-- TODO : this widget has a toolbar and map dependency -->
  </xsl:template>

</xsl:stylesheet>
