GeoExt.ux.WFSFilterBuilderButton = Ext.extend(Ext.Button, {

    /*
     * const: window_title_text {String} Title of the default window
     */
    window_title_text : 'Research',

    /*
     * const: close_button_text {String} Text in the close button of the default
     * window
     */
    close_button_text : 'Close',

    /*
     * const: csv_export_text {String}
     */
    csv_export_text : 'Export to CSV',

    /*
     * const: clear_filter_text {String}
     */
    clear_filter_text: 'Reset all filters',

    /*
     * const: csv_confirm_export_prefix_text {String}
     */
    csv_confirm_export_prefix_text: "The file would have more than ",

    /*
     * const: csv_confirm_suffix_prefix_text {String}
     */
    csv_confirm_export_suffix_text: " records. Proceed with download ?",

    /*
     * const: loading_please_wait_text {String}
     */
    loading_please_wait_text: "Loading in progress. Please wait...",

    /**
     * Property: window {Ext.Window} The window that contains the Tabpanel
     */
    window : null,

    /**
     * Property: map (Object) The default map of the application
     */
    map : null,

    /**
     * Property: layer (Object) The default wfs layer of the application
     */
    layer : null,


    /**
     * Property: comboBoxLayerType (Ext.form.ComboBox) The comboBox to select
     * the active layer
     */
    comboBoxLayerType : null,

    /**
     * Property: resetFiltersButton (Ext.Button) The button to reset
     * all filters
     */
    resetFiltersButton : null,

    /**
     * Property: filterBuilderSelector (GeoExt.ux.FilterBuilderSelector) The
     * panel that enable to choose some FilterBuilder
     */
    filterBuilderSelector : null,

    /**
     * Property: queryOnClick (OpenLayers.Control.QueryOnClick) The
     * queryOnClick widget that communicates with the widget
     */
    queryOnClick : null,

    /**
     * Property: wfsSearch (GeoExt.ux.WFSSearch) The plugin for quick queries
     * on top of the FeatureGridResult panel
     */
    wfsSearch : null,

    /**
     * Property: pagingToolbar (Ext.PagingToolbar) The toolbar for paging
     * management
     */
    pagingToolbar : null,

    /**
     * Property: featureGridResult (GeoExt.ux.FeatureGridResult) The panel that
     * contains the result of the research
     */
    featureGridResult : null,

    /**
     * Property: tabPanel (Ext.TabPanel) The TabPanel that contains the
     * featureGridResult and the filterBuilderSelector
     */
    tabPanel : null,


    /**
     * Property: featureConfigs (Array) An array of all the features added
     */
    featureConfigs: [],
    
    /**
     * 
     */
    actionButtons: [],

    /**
     * Property: currentIndex
     * (Integer) The currently selected layer in the combobox.  Used for
     *           validation purpose only.
     */
    currentIndex: null,

    /**
     * APIProperty: resetLayer
     * (Boolean) Allow comboBoxLayerType to be
     *           unselected by the resetFilterButton
     */
    resetLayer : false,
    
    /**
     * APIProperty: autoFilter
     * (Boolean) Filter automatically when the result
     *           tab is activated
     */
    autoFilter : false,

    /**
     * APIProperty: allowWFSCSVExport
     * (Boolean) Allow exporting a csv file using WFS
     *           using text/cvs outputFormat.
     */
    allowWFSCSVExport: true,

    /**
     * APIProperty: maxSafeCSVRecords
     * (Integer) Beyond this number, a warning message will be prompt before
     *           triggering an 'export to CVS request'.
     */
    maxSafeCSVRecords: 500,

    /**
     * APIProperty: skipStoreLoad
     * (Boolean) Used to skip loading the store at specific moments.
     */
    skipStoreLoad: false,

    /**
     * APIProperty: hideWindowOnRecenter
     * (Boolean) Hide the main window on map recenter on a feature.
     */
    hideWindowOnRecenter: true,

    /**
     * APIProperty: hideSearchFieldMenuOnCheck
     * (Boolean) Hide the search field menu when a menu item is checked.
     */
    hideSearchFieldMenuOnCheck: true,

    /**
     * Constructor: GeoExt.ux.WFSFilterBuilderButton Create a new
     * WFSFilterBuilderButton
     * 
     * Parameters: config - {Object} An optional object whose properties will be
     * set on this instance.
     */
    constructor : function(config) {
        config = config || {};
        Ext.apply(this, config);
        arguments.callee.superclass.constructor.call(this, config);
        this.iconCls = 'gx-wfsfilterbuilder-button';
    },


    /**
     * Method: initComponent
     * This method initialise every elements of the button
     */
    initComponent : function() {

        this.allowWFSCSVExport && this.addWFSCSVExportButton();
        
        this.on('click', function(button, event) {
            
            //It must have only one window
            if (!this.window && this.featureConfigs 
                && this.featureConfigs.length > 0) {
                
                //Create the states for the combox
                var states = [];
                for (var i = 0; i < this.featureConfigs.length; i++) {
                    config = this.featureConfigs[i];
                    state = [];
                    state.push(i); 
                    state.push(config.title); 
                    state.push(config.resource); 
                    states.push(state);
                }

                //Create the store for the combox
                var store = new Ext.data.SimpleStore({
                    fields : [ 'abbr', 'state' ],
                    data : states
                });

                this.addEvents('filter');

                this.cls = 'gx-wfsfilterbuilder-selector'

                this.setAutoScroll(true);

                var format = new OpenLayers.Format.Filter();
                this.comboBoxLayerType = new Ext.form.ComboBox({
                    store : store,
                    displayField : 'state',
                    typeAhead : true,
                    mode : 'local',
                    triggerAction : 'all',
                    selectOnFocus : true,
                    hideLabel: true,
                    width: this.layerTypeComboBoxWidth,
                    scope : this,
                    listeners : {
                        'select' : this.onComboBoxLayerTypeSelect,
                        scope : this
                    }
                });

                //Initialize the filter builder selector
                this.filterBuilderSelector = 
                    new GeoExt.ux.FilterBuilderSelector({
                        wfsFilterBuilderButton : this,
                        featureConfigs : this.featureConfigs,
                        width: this.width
                    });
                   
                this.wfsSearch =  new GeoExt.ux.WFSSearch({
                    wfsFilterBuilderButton : this,
                    minChars: 1,
                    autoFocus: true,
                    position: 'top'
                });

                this.pagingToolbar = new Ext.PagingToolbar({
                    xtype: 'paging',
                    store: this.featureConfigs[0].store,
                    displayInfo: true,
                    pageSize: 16,
                    listeners: {
                        beforechange: function(This, params) {
                            this.store.proxy.protocol.options = Ext.apply(
                                this.store.proxy.protocol.options, {
                                    startIndex: params.start+1,
                                    maxFeatures : params.limit
                                }
                            );
                        }
                    }
                });

                //Initialize the panel that contains all the results
                this.featureGridResult = new GeoExt.ux.FeatureGridResult({
                    wfsFilterBuilderButton: this,
                    store : this.featureConfigs[0].store,
                    featureConfigs : this.featureConfigs,
                    defaults : { sortable: false },
                    columns : [],
                    autoscroll : true,
                    map : this.map,
                    plugins: [this.wfsSearch],
                    tbar:[],
                    bbar: [this.pagingToolbar, "->", this.actionButtons],
                    disabled: true,

                    listeners: {
                        activate: {
                            fn: function() {
                                if(this.wfsFilterBuilderButton.autoFilter)
                                    Ext.defer(this.doFilter, 50, this);
                            },
                            scope: this.filterBuilderSelector
                        }
                    }
                });

                this.resetFiltersButton = new Ext.Button({
                    xtype: 'button',
                    iconCls: 'x-btn-text delete',
                    text: this.clear_filter_text,
                    style: { marginLeft: 10 },
                    handler: function() {
                        if(this.resetLayer)
                        {
                            this.comboBoxLayerType.clearValue();
                            this.comboBoxLayerType.selectedIndex = -1;
                            this.onComboBoxLayerTypeSelect();
                        }
                        else
                        {
                            this.resetFilters();
                            this.triggerRequest();
                        }
                    },
                    scope: this,
                    disabled: true
                });
                
                //Initialize a tabPanel with filterBuilderSelector 
                //and featureGridResult
                this.tabPanel = new Ext.TabPanel({
                    autoTabs : true,
                    activeTab : 0,
                    items : [
                        this.filterBuilderSelector,
                        this.featureGridResult
                    ],
                    tbar : new Ext.Toolbar({
                        items : [
                            this.comboBoxLayerType,
                            this.resetFiltersButton
                        ]
                    }),
                    deferredRender : false,
                    layoutOnTabChange: true,
                    border : false,
                    listeners : {
                        'beforetabchange' :  function(This, panelTab) {
                            // give focus to wfs filter button to release
                            // focus to do blur event on filter fields
                            this.filterBuilderSelector.wfsFilterBuilderButton.focus();
                        },
                        scope : this
                    }
                });


                //Listen the "afterqueries" event to update the selected store
                this.queryOnClick && this.queryOnClick.events.on({
                    'afterqueries' : this.onQueryOnClickAfterQuery,
                    scope : this
                });

                //Initialize and show the window
                this.window = new Ext.Window({
                    title : this.window_title_text,
                    constrain: true,
                    layout : 'fit',
                    width: 1000,
                    height: 550,
                    closeAction : 'hide',
                    collapsible: true,
                    plain : true,
                    items : this.tabPanel,
                    buttons : [
                        {
                            text : this.close_button_text,
                            handler : function() {
                                this.window.hide();
                            },
                            scope : this
                        }
                    ]
                });
                
                
            }

            if (this.window && !this.window.isVisible()) {
                //Show the window if it create
                this.window.show();
            }
        }, this);
        
        //Apply the configuration
        GeoExt.ux.WFSFilterBuilderButton.superclass.initComponent.apply(this, arguments);
    },


    /**
     * : addFeatureType
     * Add a store to the list of avaible stores
     *
     * Parameters:
     * addFeatureType - {Object} The configuration of the resource to add
     */
    addFeatureType : function(featureConfig) {
        this.featureConfigs.push(featureConfig)
        featureConfig.store.addListener("load", this.onStoreLoaded, this);
        featureConfig.store.addListener("beforeload", this.triggerRequest, this);
        featureConfig.store.layer = this.layer;
        
        // sort filter list
        this.featureConfigs = this.featureConfigs.sort(function(a, b) {
            if (a.title < b.title) {
                return -1;
            }
            if (a.title > b.title) {
                return 1;
            }
            return 0;
        });
    },

    /**
     * Method: setLayer
     * Set the layer
     *
     * Parameters:
     * layer - {OpenLayers.Layer.Vector} The layer
     */
    setLayer : function(layer) {
        if (this.layer == null) {
            this.layer = layer;
        }
    },


    /**
     * Method: setMap
     * Set the map
     *
     * Parameters:
     * map - {OpenLayers.Map} The map
     */
    setMap : function(map) {
        if (this.map == null) {
            this.map = map;
        }
    },
    

    /**
     * Method: setQueryOnClick
     * Set the queryOnClickWidget
     *
     * Parameters:
     * queryOnClick - {OpenLayers.Control.QueryOnClick} 
     *      The queryOnClick widget
     */
    setQueryOnClick : function(queryOnClick) {
        if (this.queryOnClick == null) {
            this.queryOnClick = queryOnClick;
        }
    },

    /**
     * Method: onStoreLoaded
     * 
     * The method is called after the load of each store.
     *
     * Parameters:
     * store - {gxp.data.WFSFeatureStore}  The store to loaded
     */
    onStoreLoaded : function(store) {

        //Remove the mask
        this.featureGridResult.getEl().unmask();

        if(!this.layer)
            return;
        
        var selectedIndex = 
            this.comboBoxLayerType.selectedIndex;
        
        //Add all the features of the store in the queryOnClick widget 
        // and in the layer
        if (this.queryOnClick) {
            this.queryOnClick.features = [];
        }
        this.layer.removeAllFeatures();

        // recreate the features array
        var features = [];
        if (store && store.data.length > 0) {
            for (var i = 0; i < store.data.length; i++) {
                var feature = store.data.items[i].data.feature;                
                features.push(feature);
            }
            this.layer.addFeatures(features);
        }
        
        // add the 'resourceName' property to each feature
        store.each(function(record) {
            if (record.getFeature().resource == null) {
                record.getFeature().resource = store.resourceName;
            }
        });

        // set the result 'singleResource' property and the queryOnClick
        // features
        if (this.queryOnClick && store && store.resourceName) {
            this.queryOnClick.results[0].singleResource = 
                store.resourceName;
            this.queryOnClick.features = features;
        }
    },

    /**
     * Method: addAction
     * 
     * Add a new action to the bottom right of the FeatureGridResult
     *
     * Parameters:
     * title - {string} Texte showed on the button
     * url - (string) Url called on click
     */
    addAction : function(title, url) {
        
        var button = new Ext.Button({
            text: title, 
            handler : function() {
                this.callAction(url);
            },
            scope: this
        });
        
        this.actionButtons.push(button);
    },

    /**
     * Method: callAction
     * 
     * Method called on click on a added action
     *
     * Parameters:
     * url - (string) Url called on click
     */
    callAction : function (url) {
        var selectedIndex = 
            this.comboBoxLayerType.selectedIndex;
        
        if (selectedIndex != -1) {
            var name = this.featureConfigs[selectedIndex].title;
            var recordsToSend = [];  
            var columns = [];
            
            for (var i = 0; i < this.featureGridResult.colModel.config.length; i++) {
                var config = this.featureGridResult.colModel.config[i];
                columns.push(config.header);
            }
            
            store = this.featureGridResult.store;
            store.each(function(record) {
                record.data.feature = null;
                recordsToSend.push(record.data);
            });

            recordsToSend = Ext.util.JSON.encode(recordsToSend);
            columns = Ext.util.JSON.encode(columns);
            this.openWindowWithPost(url, recordsToSend, columns, name);
        }
    },

    /**
     * private Method: openWindowWithPost
     * 
     * Call page with params in post
     *
     * Parameters:
     * url - (string) Url called on click
     * records - (Json) Datas to send to the page
     * columns - (Array) List of columns
     * name - (String) Name of the resource
     */
    openWindowWithPost: function (url, records, columns, name) {
        var form = document.createElement("form");
        form.setAttribute("method", "post");
        form.setAttribute("action", url);
        form.setAttribute("target", "_blank");


        var hiddenField = document.createElement("input");              
        hiddenField.setAttribute("name", "records");
        hiddenField.setAttribute("value", records);   
        form.appendChild(hiddenField);

        var hiddenField = document.createElement("input");  
        hiddenField.setAttribute("name", "columns");
        hiddenField.setAttribute("value", columns);
        form.appendChild(hiddenField);

        var hiddenField = document.createElement("input");  
        hiddenField.setAttribute("name", "name");
        hiddenField.setAttribute("value", name);
        form.appendChild(hiddenField);
        
        document.body.appendChild(form);

        form.submit();
    },

    /**
     * public Method: triggerRequest
     * 
     * Get current filters from the FilterBuilder and WFSSearch components,
     *     merge them then reload the current selected store;
     */
    triggerRequest: function() {
        // get current store
        var index = this.comboBoxLayerType.selectedIndex;
        if(index == -1) return;

        var store = this.featureConfigs[index].store;

        // set selected store and mask
        this.featureGridResult.selectedIndex = index;
        this.featureGridResult.getEl().mask(
            this.loading_please_wait_text, 'x-mask-loading');

        // get merged filter
        var filters = this.getMergedFilters()
        store.setOgcFilter(filters);

        // get total number of features matching the filters and load the store with the appended total
        store.proxy.protocol.read({
            filter: filters,
            readOptions: {output: "object"},
            resultType: "hits",
            maxFeatures: null,
            startIndex: 1,

            callback: function(response) {
                var index = this.comboBoxLayerType.selectedIndex;
                var store = this.featureConfigs[index].store;
                
                store.un("beforeload", this.triggerRequest, this);
                store.load({
                    callback: function (records, options, success)
                    {
                        if(success)
                        {
                            options.store.totalLength = options.numberOfFeatures;
                            this.pagingToolbar.onLoad(options.store, records, {
                                params : {
                                    start: options.store.proxy.protocol.options.startIndex-1,
                                    limit: options.store.proxy.protocol.options.maxFeatures
                                }
                            });
                        }
                        store.on("beforeload", this.triggerRequest, this);
                    },
                    scope: this,
                    numberOfFeatures: response.numberOfFeatures,
                    store: store
                });
            },
            scope: this
        });
        
        return false;
    },

    /** private: method[getMergedFilters]
     *  :returns: ``OpenLayers.Filter``
     *
     *  Get both FilterBuilder and WFSSearch filters
     */
    getMergedFilters: function() {
        var mergedFilters, fbsFilter, wfsSFilter;

        fbsFilter = this.filterBuilderSelector.getFilter();
        wfsSFilter = this.wfsSearch.getFilter();

        if (fbsFilter == null) {
            mergedFilters = wfsSFilter;
        } else if (wfsSFilter == null) {
            mergedFilters = fbsFilter;
        } else {
            mergedFilters = new OpenLayers.Filter.Logical({
                type: OpenLayers.Filter.Logical.AND,
                filters: [wfsSFilter, fbsFilter]
            });
        }
        
        return mergedFilters;
    },

    /** private: method[resetFilters]
     *  Reset and clear all filters used by this widget.
     */
    resetFilters: function() {
        this.filterBuilderSelector.setFilter();
        this.filterBuilderSelector.clearFilterBuilder();
        
        this.wfsSearch.setFilter();
        this.wfsSearch.clearSearch();
    },

    /**
     * Method: getActiveStore
     */
    getActiveStore: function() {
        var index = this.comboBoxLayerType.selectedIndex;
        return (index == -1) ? false : this.featureConfigs[index].store;
    },

    /**
     * Method: addWFSCSVExportButton
     */
    addWFSCSVExportButton: function() {
        this.actionButtons.push(new Ext.Button({
            text: this.csv_export_text,
            handler : this.wfsCVSExportButtonHandler,
            scope: this
        }));
    },

    /**
     * Method: currentWFSServiceAllowsCSVExport
     * Check if the current selected resource has a WFS service which supports
     * CSV export.
     */
    currentWFSServiceAllowsCSVExport: function() {
        var index = this.comboBoxLayerType.selectedIndex;
        return (index == -1) ? false : this.featureConfigs[index].allowCSV;
    },

    /**
     * public Method: wfsCVSExportButtonHandler
     * Callback method used by the csv export button.
     */
    wfsCVSExportButtonHandler: function(button, event) {
        var store = this.getActiveStore();

        if (!store) {
            return false;
        }

        if (!this.currentWFSServiceAllowsCSVExport()) 
        {
            return false;
        }

        var protocol = store.proxy.protocol;

        // (1) warning if too many records
        if (store.totalLength > this.maxSafeCSVRecords) {
            var message = [];
            message.push(this.csv_confirm_export_prefix_text);
            message.push(this.maxSafeCSVRecords);
            message.push(this.csv_confirm_export_suffix_text);
            if (!confirm(message.join(""))) {
                return false;
            }
        }

        // (2) build GetFeature GML request
        var filters = this.getMergedFilters();
        var options = {
            filter: filters,
            readOptions: {output: "object"},
            outputFormat: "text/csv",
            maxFeatures: null,
            startIndex: 1
        };
        OpenLayers.Util.applyDefaults(options, protocol.options || {});
        var data = OpenLayers.Format.XML.prototype.write.apply(
            protocol.format,
            [protocol.format.writeNode("wfs:GetFeature", options)]
        );

        // (3) create a form to do the commit.
        var id = Ext.id();
        var form = document.createElement("form");
        form.method = "post";
        form.action = protocol.url;
        form.target = id; // will make it open in a new window
        document.body.appendChild(form);

        var hidden;
        hidden = document.createElement('input');
        hidden.type = 'hidden';
        hidden.name = 'data';
        hidden.value = data;
        form.appendChild(hidden);

        form.submit();
    },

    onComboBoxLayerTypeSelect: function() {
        var index = this.comboBoxLayerType.selectedIndex;
        if (this.currentIndex != index) {
            this.currentIndex = index;

            // reset all filters
            this.resetFilters();

            if(index >= 0)
            {
                // adjust the filterBuilderSelector
                this.filterBuilderSelector.changeFeatureType({
                    'selectedIndex': index,
                    'tabPanel': this.tabPanel
                });

                this.featureGridResult.enable();
                
                // adjust the featureGridResult
                this.featureGridResult.selectedIndex = index;
                this.featureGridResult.adaptColumnsToStore();

                // adjust the pagingToolbar
                this.pagingToolbar.bindStore(
                    this.featureConfigs[index].store, true
                );

                if (this.skipStoreLoad) {
                    this.skipStoreLoad = false;
                } else {
                    this.featureConfigs[index].store.load();
                }

                this.resetFiltersButton.enable();
            }
            else
            {
                this.filterBuilderSelector.removeAll(true);
                this.filterBuilderSelector.GXPFilterBuilder = null;

                this.featureGridResult.disable();
                this.featureConfigs[this.featureGridResult.selectedIndex].store.removeAll();
                this.pagingToolbar.unbind(this.featureConfigs[this.featureGridResult.selectedIndex].store);
                this.featureGridResult.selectedIndex = index;

                this.resetFiltersButton.disable();
            }
        }
        this.resetFiltersButton.enable();
    },

    getSelectIndexByResource: function(resource) {
        var selectIndex = -1;

        Ext.each(this.featureConfigs, function(featureConfig, index) {
            if (featureConfig.resource == resource) {
                selectIndex = index;
                return false;
            }
        }, this);

        return selectIndex;
    },

    onQueryOnClickAfterQuery: function(e) {
        var selectedIndex = this.comboBoxLayerType.selectedIndex;
        if (selectedIndex == -1) {
            return;
        }
        var selectedResource = this.featureConfigs[selectedIndex].resource;

        // when only one resource was querried and it is not currently selected
        // in the combobox, then select it first while skipping the store load
        if (e.resourceNames.length == 1 &&
            e.resourceNames[0] != selectedResource) {
            this.skipStoreLoad = true;
            var index = this.getSelectIndexByResource(e.resourceNames[0]);
            this.comboBoxLayerType.select(index);
            this.onComboBoxLayerTypeSelect();
            this.comboBoxLayerType.setValue(
                this.comboBoxLayerType.store.getAt(index).get(
                    this.comboBoxLayerType.displayField
                )
            );
            selectedIndex = this.comboBoxLayerType.selectedIndex;
            selectedResource = this.featureConfigs[selectedIndex].resource;
        }
                        
        var selectedStore = this.featureConfigs[selectedIndex].store;
                        
        // push features that have a resource equal to the
        // currently selected resource (layer)
        var features = [];
        for ( var i = 0; i < e.features.length; i++) {
            if (e.features[i].resource == selectedResource) {
                features.push(e.features[i]);
            }
        }

        if (e.resourceNames.length > 0) {
            selectedStore.loadData(features);
        } else {
            selectedStore.removeAll();
        }
    }
});
