/* 
   Copyright (c) 2010- Nippour Geomatik, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license. 
*/ 

/**
 * @requires OpenLayers/Control.js
 * @requires mapfish/Protocol/MapFish.js
 */

/**
 * Class: OpenLayers.Control.WMSLayerAdder
 *
 *
 * Inherits From:
 *  - <OpenLayers.Control>
 */
OpenLayers.Control.WMSLayerAdder = OpenLayers.Class(OpenLayers.Control, {
	
	/*
     * i18n text for language pack
     */
	i18n_alert_error_title:'Error',
	i18n_alert_empty_url:'The URL field cannot be empty.',
	i18n_alert_empty_name:'The server name cannot be empty.',
	i18n_alert_no_selected_server:'You must select a server to remove it',
	i18n_alert_invalid_wms_getcapabilities:'Invalid GetCapabilities response.',
	i18n_btn_external_addserver:'Add',
	i18n_btn_external_delete:'Remove',
	i18n_btn_external_connect:'Connect',
	i18n_field_external_url:'URL',
	i18n_field_external_user:'User',
	i18n_field_external_pass:'Password',
	i18n_form_btn_add:'Add',
	i18n_form_btn_cancel:'Cancel',
	i18n_form_itemselect_add:'Add',
	i18n_form_itemselect_available:'Available',
	i18n_form_server_list:'Saved servers',
	i18n_prompt_servername_title:'Server name',
	i18n_prompt_servername_msg:'Please enter a name for the server:',
	i18n_tab_local_server:'Local server',
	i18n_tab_external_server:'External server',
	i18n_wmslayeradder_tooltip:'Add a layer',
	
    /**
     * Property: dispolayers
     * {Array} Array of layers to be made accessible
     */
    dispolayers: null,
												
    /**
     * Property: externalservers
     * {Array} Array of external servers
     */
    externalservers: null,
												
    /**
     * Property: map
     * {OpenLayers.Map} The map object to be called
     */
    map: null,
	
    /**
     * Property: mapfishlayertree
     * {mapfish.widgets.LayerTree} The mapfishlayertree to update (if set)
     */
    mapfishlayertree: null,
	
    /**
     * Property: service
     * {String} The service attached to the widget
     */
    service: null,
	
    /**
     * Property: resource
     * {String} The resource attached to the widget
     */
    resource: null,
	
	/**
     * Property: formwindow
     * {Ext.Window} Saves the generated window so it ca be reused
     */
    formwindow: null,

	
    /**
     * Property: type
     * {String} The type of <OpenLayers.Control> -- When added to a 
     *     <Control.Panel>, 'type' is used by the panel to determine how to 
     *     handle our events.
     */
    type: OpenLayers.Control.TYPE_BUTTON,
    
    /*
     * Method: addLayer
     * Add a new layer to openlayers and mapfishlayertree.
	 *
     * Parameters:
     * layers - {Array} Array of new layers.
     */
	 addlayer: function(layers) {
		var aLayers = Array();
		
		try{
			this.localLayers.multiselects[1].removeAll()
			this.extLayers.items.items[6].multiselects[1].store.removeAll();
		}catch(e){}
				
		for (var i=0; i<layers.length; i++)
		{
			if (this.map.getLayersByName(layers[i].name) < 1)
			{
				switch (layers[i].type)
				{
					case "wfs":
					case "featureserver":
						aLayers.push(new OpenLayers.Layer.WFS(
								layers[i].text,
								layers[i].url,
								{layers: layers[i].layer, transparent: true},
								{singleTile: false, isBaseLayer: false, opacity: layers[i].opacity}
							));
						break;
					case "tilecache":
						aLayers.push(new OpenLayers.Layer.TileCache(
								layers[i].text,
								layers[i].url,
								layers[i].layer,
								{singleTile: false, isBaseLayer: false, opacity: layers[i].opacity}
							));
						break;
					case "wms":
					default:
						aLayers.push(new OpenLayers.Layer.WMS(
								layers[i].text,
								layers[i].url,
								{layers: layers[i].layer, transparent: true},
								{singleTile: false, isBaseLayer: false, opacity: layers[i].opacity}
							));
						break;
				}
	
				this.mapfishlayertree && this.mapfishlayertree.getRootNode().insertBefore({text: layers[i].name, checked: true, leaf: true, layerNames: [layers[i].name], last: ''}, null);
			}
		}
		 
		this.map.addLayers(aLayers);
	 },


    /*
     * Method: addServer
     * Adds the server to the user's list
	 *
     * Parameters:
     * btn - {String} Clicked button.
	 * text - {String} Name for the server
     */
	 addServer : function(btn, text) {
		if (btn == "ok")
		{
			if (text != "")
			{
				var formServer = this.formwindow.items.items[0].getForm().getFieldValues(true);
				if (formServer.url != "")
				{
					 OpenLayers.loadURL(
						 "proxy.php", 
						 OpenLayers.Util.applyDefaults(
							 OpenLayers.Util.getParameters(''), 
							 {
								 'osmservice': '__blank__', 
								 'osmprocess': 'addServer', 
								 'osmwidgettype': 'wmslayeradder', 
								 'url':formServer.url,
								 'user':formServer.user,
								 'pwd':formServer.pass,
								 'desc':text
							 }
						 ), 
						 this, 
						 function(){
							OpenLayers.loadURL(
								 "proxy.php", 
								 OpenLayers.Util.applyDefaults(
									 OpenLayers.Util.getParameters(''), 
									 {'osmservice': '__blank__', 'osmprocess': 'getServerList', 'osmwidgettype': 'wmslayeradder', 'junk': (new Date()).getTime()}
								 ), 
								 this, 
								 this.loadCapabilitiesServers
							 );
						 }
					 );
				}else{
					Ext.MessageBox.alert(this.i18n_alert_error_title, this.i18n_alert_empty_url);
				}
			}else{
				Ext.MessageBox.alert(this.i18n_alert_error_title, this.i18n_alert_empty_name);
			}
		}
	},
	 
	 
    /*
     * Method: connectServer
     * Connects to the supplied server
     */
	 connectServer: function() {
		var form = this.formwindow.items.items[0].getForm().getFieldValues(true);
		
		if (form.user != "")
		{
			form.url = (form.url.search(/http:\/\//i)>=0)?form.url.replace(/http:\/\//g,'http://'+form.user+':'+form.pass+'@'):'http://'+form.user+':'+form.pass+'@'+form.url;
		}
		
		var params = OpenLayers.Util.applyDefaults(
			 OpenLayers.Util.getParameters(''), 
			 {
				 'osmservice': '__blank__', 
				 'osmprocess': 'curlCall', 
				 'osmwidgettype': 'wmslayeradder', 
				 'url': form.url
			 }
		);
		
		OpenLayers.loadURL("proxy.php", params, this, this.parseCapabilities);
	 },
	 
    /*
     * Method: getLayers
     * Get the good layers and add the layers.
	 *
     * Parameters:
     * names - {String} List of layers names.
     */
	 getLayers: function(names) {
		 var aLayers = Array();
		 
		 for (var i=0; i<this.dispolayers.length; i++){
			 if (names.search(this.dispolayers[i].text) >= 0)
			 {
				 aLayers.push(this.dispolayers[i]);
			 }
		 }
		 
		 var extLayers = this.extLayers.items.items[6].multiselects[1].store.getRange(0);
		 for (var i=0; i<extLayers.length; i++){
			 aLayers.push(extLayers[i].data);
		 }

		 
		 this.addlayer(aLayers);
	 },
	 
    /*
     * Method: loadCapabilities
     * Load the personal servers and other layers of the current user
     */
	 loadCapabilities : function() {
         var url = "proxy.php";
         OpenLayers.loadURL(
			 url, 
			 OpenLayers.Util.applyDefaults(
			     OpenLayers.Util.getParameters(''), 
			     {'osmservice': '__blank__', 'osmprocess': 'getServerList', 'osmwidgettype': 'wmslayeradder'}
             ), 
		   	 this, 
			 this.loadCapabilitiesServers
		 );
         OpenLayers.loadURL(
			 url, 
			 OpenLayers.Util.applyDefaults(
			     OpenLayers.Util.getParameters(''), 
			     {'osmservice': '__blank__', 'osmprocess': 'getCapabilities', 'osmwidgettype': 'wmslayeradder'}
             ), 
		   	 this, 
			 this.loadCapabilitiesWorkspace
		 );
     },


    /*
     * Method: loadCapabilitiesServers
     * Insert the capabilities in the server list
	 *
     * Parameters:
     * request - {Object} The request object returned by loadCapabilities.
     */
	 loadCapabilitiesServers : function(request) {
		 try
		 {
        	 this.serverList = eval(request.responseText);
			 
			 // If the form is already loaded
			 if (this.extLayers && this.serverList)
			 {
				this.extLayers.items.items[5].items.items[0].store.removeAll();
				this.extLayers.items.items[5].items.items[0].store.loadData(this.serverList);
			 }
		 }catch(e){}
     },


    /*
     * Method: loadCapabilitiesWorkspace
     * Insert the layers in the list
	 *
     * Parameters:
     * request - {Object} The request object returned by loadCapabilities.
     */
	 loadCapabilitiesWorkspace : function(request) {
         try
		 {
			 var layerArray = eval(request.responseText);
			 
			 // Add each layers
			 for(var i=0; i<layerArray.length; i++)
			 {
				 // Validate that it's not already in oMap
				 if (oMap.getLayersByResource(layerArray[i].name).length == 0)
				 {
					 for (var j=0; j<layerArray[i].datastores.length; j++)
					 {
						 if (layerArray[i].datastores[j].type == "wms" || layerArray[i].datastores[j].type == "tilecache")
						 {
							 this.dispolayers.push({
											"name": layerArray[i].name+"_"+layerArray[i].datastores[j].type, 
											"text": layerArray[i].name+"_"+layerArray[i].datastores[j].type, 
											"type": layerArray[i].datastores[j].type, 
											"url": "proxy.php?osmservice="+layerArray[i].datastores[j].service+"&osmresource="+layerArray[i].name, 
											"layer": layerArray[i].datastores[j].layers, 
											"opacity": 1
										});
						 }
					 }
				 }
			 }
		 }catch(e){}
     },


    /*
     * Method: parseCapabilities
     * Parse the returned GetCapabilities XML
	 *
     * Parameters:
     * request - {Object} The request object returned by connectServer.
     */
	 parseCapabilities : function(request) {
		var layerArray = Array();
		try
		{
			var extStore = new GeoExt.data.WMSCapabilitiesStore({
								data: request.responseText
							});
			var form = this.formwindow.items.items[0].getForm().getFieldValues(true);
			
			for(var i=0; i<extStore.data.length; i++)
			{
				layerArray.push({
						"name": extStore.data.items[i].data.name,
						"text": extStore.data.items[i].data.title,
						"type": "wms",
						"url": (form.user=="")?form.url:(form.url.search(/http:\/\//i)>=0)?form.url.replace(/http:\/\//g,'http://'+form.user+':'+form.pass+'@'):'http://'+form.user+':'+form.pass+'@'+form.url,
						"layer": extStore.data.items[i].data.name,
						"opacity": 1
					});
			}
			
			this.extLayers.items.items[5].multiselects[0].store.removeAll();
			this.extLayers.items.items[5].multiselects[0].store.loadData(layerArray);
		}catch(e){
			Ext.MessageBox.alert(this.i18n_alert_error_title, this.i18n_alert_invalid_wms_getcapabilities);
		}
	},


    /*
     * Method: removeServer
     * Remove selected server from the list
	 *
     * Parameters:
     * id - {String} ID of the server to delete.
     */
	 removeServer : function(id) {
		if (id != "")
		{
			 var url = "proxy.php";
			 OpenLayers.loadURL(
				 url, 
				 OpenLayers.Util.applyDefaults(
					 OpenLayers.Util.getParameters(''), 
					 {'osmservice': '__blank__', 'osmprocess': 'deleteServer', 'osmwidgettype': 'wmslayeradder', 'id_wms': id}
				 ), 
				 this, 
				 function(){
					OpenLayers.loadURL(
						 "proxy.php", 
						 OpenLayers.Util.applyDefaults(
							 OpenLayers.Util.getParameters(''), 
							 {'osmservice': '__blank__', 'osmprocess': 'getServerList', 'osmwidgettype': 'wmslayeradder', 'junk': (new Date()).getTime()}
						 ), 
						 this, 
						 this.loadCapabilitiesServers
					 );
				}
			 );
		}else{
			Ext.MessageBox.alert(this.i18n_alert_error_title, this.i18n_alert_no_selected_server);
		}
	},


	/*
     * Method: trigger
     * Open the window.
     */
	 trigger: function() {
		this.activate();
		
		if (!this.formwindow)
		{
			var ds = new Ext.data.JsonStore({
				data: this.dispolayers,
				fields: ['text','name'],
				sortInfo: {
					field: 'text',
					direction: 'ASC'
				}
			});
			
			this.localLayers = {
					xtype: 'itemselector',
					name: 'itemselectorLocal',
					imagePath: '../../lib/ext-3.0.0/examples/ux/images/',
					drawUpIcon: false,
					drawDownIcon: false,
					drawTopIcon: false,
					drawBotIcon: false,
					defaults:{autoHeight: true},
					multiselects: [{
						width: 315,
						height: 180,
						legend: this.i18n_form_itemselect_available,
						store: ds,
						displayField: 'text',
						valueField: 'name'
					},{
						width: 315,
						height: 180,
						legend: this.i18n_form_itemselect_add,
						store: []
					}]
				};
				
			this.extLayers = new Ext.Panel({
				border: false,
				width:670,
				layout:'table',
                layoutConfig: {columns:3},
				defaults:{autoHeight: true},
				items:[
					{
						border: false,
						colspan: 3,
						layout: 'form',
						items: [{
							xtype:'textfield',
							fieldLabel: this.i18n_field_external_url,
							name: 'url',
							width: 520
						}]
					},
					{
						border: false,
						width: 210,
						layout: 'form',
						items: [{
							xtype:'textfield',
							fieldLabel: this.i18n_field_external_user,
							name: 'user',
							width: 100
						}]
					},
					{
						border: false,
						width: 210,
						layout: 'form',
						items: [{
							xtype:'textfield',
							fieldLabel: this.i18n_field_external_pass,
							name: 'pass',
							inputType: 'password',
							width: 100
						}]
					},
					new Ext.Button({
						text: this.i18n_btn_external_connect,
						width: 75,
						scope: this,
						style: {marginRight: '-25px'},
						handler: function(b, e){this.connectServer();}
					}),
					{
						border: false,
						width: 210,
						height: 190,
						items: [{
								xtype: 'multiselect',
								name: 'extList',
								id: 'extList',
								legend: this.i18n_form_server_list,
								width: 200,
								height: 175,
								allowBlank:false,
								store: new Ext.data.JsonStore({
									fields: ['description','id','url','user','pass'],
									data: this.serverList,
									sortInfo: {
										field: 'description',
										direction: 'ASC'
									}
								}),
								displayField: 'description',
								valueField: 'id',
								ddReorder: true,
								listeners:{
									 scope: this,
									 'change': function(e, t)
												{ 
													for(var i=0; i<this.serverList.length; i++)
													{
														if (this.serverList[i].id == t)
														{
															this.extLayers.items.items[0].items.items[0].setValue(this.serverList[i].url);
															this.extLayers.items.items[1].items.items[0].setValue(this.serverList[i].user);
															this.extLayers.items.items[2].items.items[0].setValue(this.serverList[i].pass);
														}
													} 
												}
								}
	
							},
							new Ext.Panel({
								border: false,
								width:200,
								layout:'table',
								layoutConfig: {columns:2},
								items:[
									new Ext.Button({
										text: this.i18n_btn_external_addserver,
										width: 75,
										height: 20,
										scope: this,
										style: {marginRight: '5px'},
										handler: function(b, e){Ext.MessageBox.prompt(this.i18n_prompt_servername_title, this.i18n_prompt_servername_msg, this.addServer, this);}
									}),
									new Ext.Button({
										text: this.i18n_btn_external_delete,
										width: 75,
										height: 20,
										scope: this,
										handler: function(b, e){this.removeServer(this.extLayers.items.items[5].items.items[0].getValue());}
									})
								]
							})
						]
					},
					{
						xtype: 'itemselector',
						colspan: 2,
						width: 390,
						name: 'itemselectorExtern',
						id: 'itemselectorExtern',
						imagePath: '../../lib/ext-3.0.0/examples/ux/images/',
						drawUpIcon: false,
						drawDownIcon: false,
						drawTopIcon: false,
						drawBotIcon: false,
						multiselects: [{
							width: 180,
							height: 200,
							legend: this.i18n_form_itemselect_available,
							store: new Ext.data.JsonStore({
								fields: ['text','name','type','url','opacity','layer'],
								sortInfo: {
									field: 'text',
									direction: 'ASC'
								}
							}),
							displayField: 'text',
							valueField: 'name'
						},{
							width: 180,
							height: 200,
							legend: this.i18n_form_itemselect_add,
							store: new Ext.data.JsonStore({
								fields: ['text','name','type','url','opacity','layer'],
								sortInfo: {
									field: 'text',
									direction: 'ASC'
								}
							}),
							displayField: 'text',
							valueField: 'name'
						}]
					}
				]
			});
			
			var tabPanelBox = new Ext.TabPanel({
				width:670,
				activeTab: 0,
				frame:true,
				bodyStyle: 'padding:10px;',
				defaults:{autoHeight: true},
				items:[
					{items:[this.localLayers], title: this.i18n_tab_local_server},
					{items:[this.extLayers], title: this.i18n_tab_external_server}
				]
			});
			
			var formPanelBox = new Ext.FormPanel({
				labelWidth: 75, // label settings here cascade unless overridden
				width: 670,
				defaults:{autoHeight: true},
				layout: 'fit',
				items:[tabPanelBox],
				buttons: [{
					text: this.i18n_form_btn_add,
					scope: this,
					handler: function(b, e){this.getLayers(this.formwindow.items.items[0].getForm().getValues().itemselectorLocal); this.formwindow.hide();}
				},{
					text: this.i18n_form_btn_cancel,
					scope: this,
					handler: function(b, e){this.formwindow.hide()}
				}]
			});
			
			this.formwindow = new Ext.Window({
					closeAction: 'hide',
					title:this.i18n_wmslayeradder_tooltip,
					items: [formPanelBox],
					layout: 'fit',
					width: 700,
					defaults:{autoHeight: true}
				});
		}
		
		this.formwindow.show();
	},
													
    CLASS_NAME: "OpenLayers.Control.WMSLayerAdder"												 
});
