<?php
/* 
   Copyright (c) 2010- Nippour Geomatik, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license. 
*/ 

class WMSLayerAdder extends org_geoprisma_widget_WidgetBase
{
    static public function getText($pstrMsg)
    {
        return parent::getText($pstrMsg, self::getType());
    }
     
    static public function getAction()
    {
        return 'read';
    }    
     
    static public function getType()
    {
        return 'wmslayeradder';
    }

    static public function getServiceType()
    {
        return 'wms';
    }
	
    static public function getProcessList()
    {
        return array(
            "getResourceCapabilities",
            "getServerList",
            "addServer",
            "updateServer",
            "deleteServer",
            "curlCall"
        );	
    }
	
    static public function linkResource($pobjResource, $pobjWidget) 
    {
        if ($pobjWidget == null) 
        {
            return;
        }

        // decode the 'layers' option that is saved in json
        try
        {
            $objOption = $pobjWidget->getOption('layers');
            $objOption->setValue(
                json_decode($objOption->getValue(), true)
            );
        }
        catch (Exception $objError)
        {}
    }
	
	 
    /**
     * This function return all the resources available
     * with the layer name and the service information
     */
    static public function curlCall()
    {
        $url = $_REQUEST["url"] . "&service=wms&request=GetCapabilities&version=1.1.1";

        ob_start();
        $objCurl = curl_init();
        curl_setopt($objCurl, CURLOPT_URL, $url);
        curl_setopt($objCurl,CURLOPT_RETURNTRANSFER,1);
        $result = curl_exec($objCurl);
        curl_close($objCurl);
        ob_end_flush();

        $result =  str_replace('\n',"",$result);
        $result =  str_replace('\r',"",$result);
        echo html_entity_decode(htmlentities($result),ENT_QUOTES,"utf-8");
    }
 
 
    static public function getServerList()
    {
        $pobjWidget = func_get_arg(0);
        try
        {
            $strmode = (string)$pobjWidget->getOption("mode")->getValue();
        }
        catch (Exception $objError)
        {
            $strmode = "cookie";
        }
		
        switch ($strmode)
        {
            //------------
            //MODE DATABASE
            //------------
            case "db":
                try
                {
                    $strdsn = (string)($pobjWidget->getOption("dsn")->getValue());
                    $struser = (string)($pobjWidget->getOption("user")->getValue());
                    $strpwd = (string)($pobjWidget->getOption("password")->getValue());
                }
                catch (Exception $e) 
                {
                    $missingBaseParam = true;
                }
	
                if(!$missingBaseParam){
                    $objPDO = new PDO($strdsn,$struser,$strpwd);
                    $querynode = "select_query";
                    try
                    {
                        $strbind_list = (string)($pobjWidget->getOption("select_list")->getValue());
                    }
                    catch (Exception $e)
                    {
                        $noBind = true;
                    }
					
                    try
                    {
                        $strsqlquery = (string)($pobjWidget->getOption($querynode)->getValue());
                    }
                    catch (Exception $e) 
                    {
                        $missingQuery=true;
                    }

                    $objStatement = $objPDO->prepare($strsqlquery);
                    if (!$noBind) 
                    {
                        $arrbind_list = explode(',',$strbind_list);
                        $i=0;
                        foreach ($arrbind_list as $bind_item) 
                        {
                            eval("\$ev[]=".$bind_item.";");
                            $objStatement->bindParam(':'.$i,$ev[$i]);
                            $i++;
                        }
                    }

                    if ($objStatement->execute()) 
                    {
                        $result = $objStatement->fetchAll(PDO::FETCH_ASSOC);
                        foreach($result as $value)
                            $arrList[]= $value;
                    }
                    else
                    {
                        $arrList[]= false;
                    }

                    echo json_encode($arrList);
                }

                break;

            //------------
            //MODE COOKIE
            //------------
            case "cookie":
                $arrServers = array();

                for ($i=0; $i<count($_COOKIE["server_name"]); $i++)
                {
                    if (isset($_COOKIE["server_name"]) && $_COOKIE["server_name"][$i] != " ") 
                    {
                        $arrServers[] = array("id"=>$i, "description"=>$_COOKIE["server_name"][$i], "url"=>$_COOKIE["server_url"][$i], "user"=>$_COOKIE["server_user"][$i], "pass"=>$_COOKIE["server_pwd"][$i]);
                    }
                }

                echo json_encode($arrServers);

                break;
        }

    }
    static public function addServer()
    {
        $pobjWidget = func_get_arg(0);
        try
        {
            $strmode = (string)$pobjWidget->getOption("mode")->getValue();
        }
        catch (Exception $objError) 
        {
            $strmode = "cookie";
        }
		
        switch ($strmode)
        {
            //------------
            //MODE DATABASE
            //------------
            case "db":
                try
                {
                    $strdsn = (string)($pobjWidget->getOption("dsn")->getValue());
                    $struser = (string)($pobjWidget->getOption("user")->getValue());
                    $strpwd = (string)($pobjWidget->getOption("password")->getValue());
                }
                catch (Exception $e)
                {
                    $missingBaseParam = true;
                }
	
                if(!$missingBaseParam){
                    $objPDO = new PDO($strdsn,$struser,$strpwd);
                    $querynode="insert_query";
                    try{
                        $strbind_list = (string)($pobjWidget->getOption("insert_list")->getValue());
                    }
                    catch (Exception $e)
                    {
                        $noBind = true;
                    }
	
                    try
                    {
                        $strsqlquery = (string)($pobjWidget->getOption($querynode)->getValue());
                    }
                    catch (Exception $e) 
                    {
                        $missingQuery=true;
                    }

                    $objStatement = $objPDO->prepare($strsqlquery);
                    if (!$noBind) 
                    {
                        $arrbind_list = explode(',',$strbind_list);
                        $i=0;
                        foreach ($arrbind_list as $bind_item)
                        {
                            eval("\$ev[]=".$bind_item.";");
                            $objStatement->bindParam(':'.$i,$ev[$i]);
                            $i++;
                        }
                    }

                    if ($objStatement->execute()) 
                    {
                        $result = $objStatement->fetchAll(PDO::FETCH_ASSOC);
                        foreach ($result as $value) 
                        {
                            $arrList[]= $value;
                        }
                    }
                    else
                    {
                        $arrList[]= false;
                    }

                    echo json_encode($arrList);
                }

                break;

            //------------
            //MODE COOKIE
            //------------
            case "cookie":

                $lastServer = count($_COOKIE["server_name"])+1-1;

                setcookie("server_name[".$lastServer."]", $_REQUEST['desc'], time()+60*60*24*30*12);
                setcookie("server_url[".$lastServer."]", $_REQUEST['url'], time()+60*60*24*30*12);
                setcookie("server_user[".$lastServer."]", $_REQUEST['user'], time()+60*60*24*30*12);
                setcookie("server_pwd[".$lastServer."]", $_REQUEST['pwd'], time()+60*60*24*30*12);
            break;
        }
    }
	
    static public function updateServer()
    {
        $pobjWidget = func_get_arg(0);
        try
        {
            $strmode = (string)$pobjWidget->getOption("mode")->getValue();
        }
        catch (Exception $objError)
        {
            $strmode = "cookie";
        }
		
        switch ($strmode)
        {
            //------------
            //MODE DATABASE
            //------------
            case "db":
                try
                {
                    $strdsn = (string)($pobjWidget->getOption("dsn")->getValue());
                    $struser = (string)($pobjWidget->getOption("user")->getValue());
                    $strpwd = (string)($pobjWidget->getOption("password")->getValue());
                }
                catch (Exception $e) 
                {
                    $missingBaseParam = true;
                }
	
                if(!$missingBaseParam) 
                {
                    $objPDO = new PDO($strdsn,$struser,$strpwd);
                    $querynode="update_query";
                    try
                    {
                        $strbind_list = (string)($pobjWidget->getOption("update_list")->getValue());
                    }
                    catch (Exception $e) 
                    {
                        $noBind = true;
                    }
	
                    try
                    {
                        $strsqlquery = (string)($pobjWidget->getOption($querynode)->getValue());
                    }
                    catch (Exception $e) 
                    {
                        $missingQuery=true;
                    }

                    $objStatement = $objPDO->prepare($strsqlquery);
                    if (!$noBind) 
                    {
                        $arrbind_list = explode(',',$strbind_list);
                        $i=0;
                        foreach ($arrbind_list as $bind_item) 
                        {
                            eval("\$ev[]=".$bind_item.";");
                            $objStatement->bindParam(':'.$i,$ev[$i]);
                            $i++;
                        }
                    }

                    if ($objStatement->execute()) 
                    {
                        $result = $objStatement->fetchAll(PDO::FETCH_ASSOC);
                        foreach ($result as $value)
                        {
                            $arrList[]= $value;
                        }
                    }
                    else
                    {
                        $arrList[]= false;
                    }

                    echo json_encode($arrList);
                }

                break;

            //------------
            //MODE COOKIE
            //------------
            case "cookie":
                break;
        }
    }
	
    static public function deleteServer()
    {
        $pobjWidget = func_get_arg(0);
        try
        {
            $strmode = (string)$pobjWidget->getOption("mode")->getValue();
        }
        catch (Exception $objError)
        {
            $strmode = "cookie";
        }
		
        switch ($strmode)
        {
            //------------
            //MODE DATABASE
            //------------
            case "db":
                try
                {
                    $strdsn = (string)($pobjWidget->getOption("dsn")->getValue());
                    $struser = (string)($pobjWidget->getOption("user")->getValue());
                    $strpwd = (string)($pobjWidget->getOption("password")->getValue());
            }
                catch (Exception $e) 
                {
                    $missingBaseParam = true;
                }
	
                if (!$missingBaseParam) 
                {
                    $objPDO = new PDO($strdsn,$struser,$strpwd);
                    $querynode="delete_query";
                    try
                    {
                        $strbind_list = (string)($pobjWidget->getOption("delete_list")->getValue());
                    }
                    catch (Exception $e) 
                    {
                        $noBind = true;
                    }
	
                    try
                    {
                        $strsqlquery = (string)($pobjWidget->getOption($querynode)->getValue());
                    }
                    catch (Exception $e)
                    {
                        $missingQuery=true;
                    }

                    $objStatement = $objPDO->prepare($strsqlquery);
                    if (!$noBind) 
                    {
                        $arrbind_list = explode(',',$strbind_list);
                        $i=0;
                        foreach ($arrbind_list as $bind_item) 
                        {
                            eval("\$ev[]=".$bind_item.";");
                            $objStatement->bindParam(':'.$i,$ev[$i]);
                            $i++;
                        }
                    }

                    if ($objStatement->execute()) 
                    {
                        $result = $objStatement->fetchAll(PDO::FETCH_ASSOC);
                        foreach ($result as $value) 
                        {
                            $arrList[]= $value;
                        }
                    }
                    else
                    {
                        $arrList[]= false;
                    }

                    echo json_encode($arrList);
                }
                break;

            //------------
            //MODE COOKIE
            //------------
            case "cookie":
                setcookie("server_name[".($_REQUEST["id_wms"]+1-1)."]", ' ', time()+60*60*24*30*12);
                setcookie("server_url[".($_REQUEST["id_wms"]+1-1)."]", ' ', time()+60*60*24*30*12);
                setcookie("server_user[".($_REQUEST["id_wms"]+1-1)."]", ' ', time()+60*60*24*30*12);
                setcookie("server_pwd[".($_REQUEST["id_wms"]+1-1)."]", ' ', time()+60*60*24*30*12);
                break;
        }
    }

    /**
    * This function return all the resources available
    * with the layer name and the service information
    */
    static public function getResourceCapabilities()
    {
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig();
 
        $objACL = org_geoprisma_acl_ACLFactory::getACL();

        $objResources = $objConfig->getResources();
  
        $objArrayResult = array();

        // Loop through all the resources available
        foreach ($objResources as $objResource) 
        {
            if ($objACL->isAuthorized($objResource, "read")) 
            {
                // We will return all the resources datastore
                // We must extract the layer name, the service and the source
                $objArrayDataStores = array();
                foreach ($objResource->getDataStores() as $objDataStore)
                {
                    $strType = '';
                    $strService = '';
                    $strSource = '';
                    $strLayers = '';

                    $objService = $objDataStore->getService();
                    switch(get_class($objService))
                    {
                        case 'org_geoprisma_service_WMSService':
                            $strType = 'wms';
                            break;
                        case 'org_geoprisma_service_TileCacheService':
                            $strType = 'tilecache';
                            break;
                        case 'org_geoprisma_service_FeatureServerService':
                            $strType = 'featureserver';
                            break;
                        default:
                            continue;
                    }
                    $strService = $objService->getName();

                    // Must extract the layers from the params
                    $strLayers = $objDataStore->getLayerString();

                    if ($strLayers) 
                    {
                        $objArrayDataStores[] = array(
                            'type'    => $strType,
                            'layers'  => $strLayers,
                            'service' => $strService
                        );
                    }
                }

                if (count($objArrayDataStores) > 0) 
                {
                    $objArrayResult[] = array(
                        'name' => $objResource->getName(),
                        'datastores' => $objArrayDataStores
                    );
                }
            }
        }

        // The return value is a JSON object
        echo json_encode($objArrayResult);
    }
}
  
?>
