<?php
/**
* Classe de base des accessfilter, Lier aux fichier de configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des accessfilter, Lier aux fichier de configuration 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage AccessFilter 
* @author     Pascal Martin 
*/ 
class org_geoprisma_accessfilter_AccessFilter extends org_geoprisma_ConfigElement
{
    private $m_strName = null;
    private $m_objConditions = null;

    /**
    * Construit un nouvelle instance de org_geoprisma_accessfilter_AccessFilter
    * 
    * @param string                        $pstrName         AccessFilter name
    * @param org_geoprisma_service_Service &$pobjService     Service object
    * @param string                        $pstrLayers       String of layers
    * @param array                         $pobjArrayOptions AccessFilter options
    * 
    * @return org_geoprisma_accessfilter_AccessFilter
    */
    public function __construct(
        $pstrName,
        $pobjArrayConditions
    ) {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);

        // name & type
        // Available types: BBox, SessionParam
        $this->m_strName = $pstrName;

        // condition
        $this->m_objArrayConditions = new ArrayObject();
        if (is_array($pobjArrayConditions)) 
        {
            foreach ($pobjArrayConditions as $strCondition) 
            {
                $this->m_objArrayConditions->append($strCondition);     
            }
        }
    }

    /**
    * Return the AccessFilter name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;
    }

    /**
    * Return the AccessFilter condition
    *
    * @return string
    */
    public function getCondition($pnCondition)
    {
        return $this->m_objArrayConditions[$pnCondition];
    }

    /**
    * Return the AccessFilter conditions
    *
    * @return string
    */
    public function getConditions()
    {
        return $this->m_objArrayConditions;
    }

    /**
    * Return the AccessFilter input.
    * input can be a string or something from the session with: 
    *   SESSION:session_variable  (Later we could have BD:string)
    *
    * @return string
    */
    public function readInput($pstrInput)
    {
        $objArrayInputParts = explode(':', $pstrInput);

        switch($objArrayInputParts[0])
        {
          case 'SESSION':
            if(isset($objArrayInputParts[1]) && 
               isset($_SESSION[$objArrayInputParts[1]]))
            {
                return $_SESSION[$objArrayInputParts[1]];
            }
          default:
            return $objArrayInputParts[0];
        }

        return '';
    }
    
    /**
    *  Ajout le accessfilter courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementAccessFilters Noeud xml ou le accessfilter doit Ãªtre ajouter
    * @param DOMDocument &$pobjDomDocument          Document XMl ou le noeud xml accessfilter est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementAccessFilters, DOMDocument &$pobjDomDocument)
    {
        $objDomElementAccessFilter = $pobjDomDocument->createElement('accessfilter'); 
        $pobjDomElementAccessFilters->appendChild($objDomElementAccessFilter);        
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName()); 
        $objDomElementAccessFilter->appendChild($objDomElementName);
    }
    
}

?>
