<?php
/**
* Interface pour les config, récupération de la config    
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Interface pour les config, récupération de la config
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Config 
* @author     Pascal Martin 
*/ 
abstract class org_geoprisma_config_Config implements com_borealis_foundation_util_Singleton
{
    protected $m_objArrayExclusiveUnboundWidgetName = null;
    protected $m_objArrayProcessList = array("getProcessList");

    /**
    * Return DOMDocument with the config inside ready to passe to xsltprocessor
    * 
    * @return DOMDocument
    */
    public function getDomDocument()
    {    
        $objDomDocument = new DOMDocument();
 
        $objDomElementRoot = $objDomDocument->createElement('geoprisma');
        $objDomDocument->appendChild($objDomElementRoot);       
        
        // ------------------------------------
        // Ajout des services 
        // ------------------------------------
        $objDomElementServices = $objDomDocument->createElement('services');
        $objDomElementRoot->appendChild($objDomElementServices);   
        foreach ($this->getServices() as $objService)
        {
            $objService->insertIntoXML($objDomElementServices, $objDomDocument);   
        }   
        
        // ------------------------------------
        // Ajout des datastores  
        // ------------------------------------
        $objDomElementDatastores = $objDomDocument->createElement('datastores');
        $objDomElementRoot->appendChild($objDomElementDatastores);   
        foreach ($this->getDatastores() as $objDatastore)
        {
            $objDatastore->insertIntoXML($objDomElementDatastores, $objDomDocument);   
        }

        // ------------------------------------
        // Ajout des widgets
        // ------------------------------------
        $objArrayWidgetTypeUsed = array();
        $objArrayWidget = $this->getAuthorizedWidget();

        $objDomElementWidgets = $objDomDocument->createElement('widgets');
        $objDomElementRoot->appendChild($objDomElementWidgets);   
        foreach ($objArrayWidget as $objArrayWigetInfo)
        {   
            $objDomElementWidget = $objArrayWigetInfo['widget']->insertIntoXML($objDomElementWidgets, $objDomDocument); 
            if ($objDomElementWidget !== null) 
            { 
                if (!in_array($objArrayWigetInfo['widget']->getType(), $objArrayWidgetTypeUsed)) 
                {
                    $objArrayWidgetTypeUsed[] = $objArrayWigetInfo['widget']->getType();     
                }
                
                $objDomElementDatastores = $objDomDocument->createElement('datastores');
                $objDomElementWidget->appendChild($objDomElementDatastores);
                foreach ($objArrayWigetInfo['datastores'] as $objArrayDatastoreInfo)
                {    
                    $objDomElementDatastore = $objDomDocument->createElement('datastore');
                    $objDomElementDatastores->appendChild($objDomElementDatastore);
                    $objDomElementResourceName = $objDomDocument->createElement('resourcename', $objArrayDatastoreInfo['resourcename']);
                    $objDomElementDatastoreName = $objDomDocument->createElement('datastorename', $objArrayDatastoreInfo['datastorename']);
                    $objDomElementServiceName = $objDomDocument->createElement('servicename', $objArrayDatastoreInfo['servicename']);
                    $objDomElementDatastore->appendChild($objDomElementResourceName);                
                    $objDomElementDatastore->appendChild($objDomElementDatastoreName);
                    $objDomElementDatastore->appendChild($objDomElementServiceName);
                }
                
                $objDomElementResources = $objDomDocument->createElement('resources');
                $objDomElementWidget->appendChild($objDomElementResources);
                foreach ($objArrayWigetInfo['resources'] as $strRessource)
                {    
                    $objDomElementResource = $objDomDocument->createElement('resource', $strRessource); 
                    $objDomElementResources->appendChild($objDomElementResource);
                }
            }
        }
        
        // ------------------------------------
        // Ajout des widgets type
        // ------------------------------------
        $objDomElementWidgetTypeUsed = $objDomDocument->createElement('widgettypesused');
        $objDomElementRoot->appendChild($objDomElementWidgetTypeUsed);   
        foreach ($objArrayWidgetTypeUsed as $strWidgetType)
        {
            $objDomElementWidget = $objDomDocument->createElement('widget', $strWidgetType);
            $objDomElementWidgetTypeUsed->appendChild($objDomElementWidget);   
        }
       
        // ------------------------------------
        // Ajout des resources
        // ------------------------------------ 
        $objDomElementResources = $objDomDocument->createElement('resources');
        $objDomElementRoot->appendChild($objDomElementResources);   
        foreach ($this->getResources() as $objResource)
        {
            $objResource->insertIntoXML($objDomElementResources, $objDomDocument);   
        }
        
        
        return $objDomDocument;
    }
    
    /**
    * Retourne la liste des widget et des datastore acessible    
    * 
    * @return array
    */
    private function getAuthorizedWidget()
    {
        $objArrayWidget = array();
        $objArrayMapWidget = array();
        $objArrayMapFishLayerTreeWidget = array();
        $objArrayOtherWidget = array();

        // -----------------------------------------------------
        // Widgets that don't need to be linked to resources 
        // are automatically added.
        // ----------------------------------------------------- 
        foreach ($this->getWidgets() as $objWidget)
        {
            if ($objWidget instanceof org_geoprisma_widget_MapWidget) 
            {
                $objAuthorizedWidget = array(
                    'widget' => $objWidget,
                    'datastores' => (array)$objWidget->getDatastores(),
                    'resources' => (array)$objWidget->getResources()
                );

                $objArrayMapWidget[$objWidget->getName()]
                    = $objAuthorizedWidget;
            }
            else if ($objWidget instanceof org_geoprisma_widget_MapFishLayerTreeWidget) 
            {
                $objAuthorizedWidget = array(
                    'widget' => $objWidget,
                    'datastores' => (array)$objWidget->getDatastores(),
                    'resources' => (array)$objWidget->getResources()
                );

                $objArrayMapFishLayerTreeWidget[$objWidget->getName()]
                    = $objAuthorizedWidget;
            }
            else
            {
                $objArrayResourceOptions 
                    = $objWidget->getMandatoryResourceOptionList();

                if ($objArrayResourceOptions === null
                    && $this->isExclusiveUnboundWidget($objWidget)
                ) 
                {
                    $objAuthorizedWidget = array(
                        'widget' => $objWidget,
                        'datastores' => (array)$objWidget->getDatastores(),
                        'resources' => (array)$objWidget->getResources()
                    );

                    $objArrayOtherWidget[$objWidget->getName()] = $objAuthorizedWidget;
                }
            }
        }

        foreach ($objArrayMapWidget as $objMapWidget) 
        {
            $objArrayWidget[$objMapWidget['widget']->getName()] = $objMapWidget;
        }
        foreach ($objArrayMapFishLayerTreeWidget as $objMapFishLayerTreeWidget)
        {
            $objArrayWidget[$objMapFishLayerTreeWidget['widget']->getName()] = $objMapFishLayerTreeWidget;
        }
        foreach ($objArrayOtherWidget as $objOtherWidget) 
        {
            $objArrayWidget[$objOtherWidget['widget']->getName()] = $objOtherWidget;
        }

        // -----------------------------------------------------
        // Adding widget going throught resources
        // ----------------------------------------------------- 
        $objArrayLayerWidget = array();
        $objArrayOtherWidget = array();

        foreach ($this->getResources() as $objResource)
        {
            foreach ($objResource->getWidgets() as $objWidget)
            {
                // map and mapfishlayertree widgets have already been processed
                // earlier (see above), so in case they have been linked to
                // resources, simply skip them.
                if ($objWidget instanceof org_geoprisma_widget_MapWidget
                    || $objWidget instanceof org_geoprisma_widget_MapFishLayerTreeWidget 
                ) 
                {
                    continue;
                }

                $strRessource = $objResource->getName();
                $strAction = $objWidget->getAction();

                if (org_geoprisma_acl_ACLFactory::getACL()->isAuthorized($objResource, $strAction)) 
                {
                    if ($objWidget->getType() == "layer") 
                    {
                        if (!array_key_exists($objWidget->getName(), $objArrayLayerWidget)) 
                        {
                            $objArrayLayerWidget[$objWidget->getName()] = array(
                                'widget' => $objWidget,
                                'datastores' => array(),
                                'resources' => array()
                            );
                        }
                        // Add widget ressource
                        if (!in_array($objResource->getName(), $objArrayLayerWidget[$objWidget->getName()]['resources'])) 
                        {
                            $objArrayLayerWidget[$objWidget->getName()]['resources'][] = $objResource->getName();   
                        } 
                        // Add widget datastore
                        foreach ($objResource->getDatastores() as $objDatastore) 
                        {
                            $objArrayLayerWidget[$objWidget->getName()]['datastores'][] = array(
                                'resourcename' => $objResource->getName(), 
                                'datastorename' => $objDatastore->getName(),
                                'servicename' => $objDatastore->getService()->getName()
                            );
                        }
                    } // end if layer widget
                    else
                    {
                        if (!array_key_exists($objWidget->getName(), $objArrayOtherWidget)) 
                        {
                            $objArrayOtherWidget[$objWidget->getName()] = array(
                                'widget' => $objWidget,
                                'datastores' => array(),
                                'resources' => array()
                            );
                        }
                        // Add widget ressource
                        if (!in_array($objResource->getName(), $objArrayOtherWidget[$objWidget->getName()]['resources'])) 
                        {
                            $objArrayOtherWidget[$objWidget->getName()]['resources'][] = $objResource->getName();   
                        } 
                        // Add widget datastore
                        foreach ($objResource->getDatastores() as $objDatastore) 
                        {
                            $objArrayOtherWidget[$objWidget->getName()]['datastores'][] = array(
                                'resourcename' => $objResource->getName(), 
                                'datastorename' => $objDatastore->getName(),
                                'servicename' => $objDatastore->getService()->getName()
                            );
                        }
                    } // end if not layer widget
                }
            }
        }

        // add layer widgets, then the others
        foreach ($objArrayLayerWidget as $objLayerWidget)
        {
            $objArrayWidget[$objLayerWidget['widget']->getName()] = $objLayerWidget;
        }
        foreach ($objArrayOtherWidget as $objOtherWidget) 
        {
            $objArrayWidget[$objOtherWidget['widget']->getName()] = $objOtherWidget;
        }

        return $objArrayWidget; 
    }
    
    /**
    * Retourne un object service
    * 
    * @param string $pstrServiceName Service Name
    * 
    * @return org_geoprisma_service_Service 
    */
    abstract public function getService($pstrServiceName);
    
    /**
    * Retourne la liste de service disponible service
    * 
    * @return ArrayObject of org_geoprisma_service_Service 
    */
    abstract public function getServices(); 
    
    /**
    * Retorrne un object layout
    * 
    * @param string $pstrLayoutName Layout name
    * 
    * @return org_geoprisma_layout_Layout
    */
    abstract public function getLayout($pstrLayoutName);
    
    /**
    * Retorrne la liste des layouts disponible
    * 
    * @return ArrayObject of org_geoprisma_layout_Layout
    */
    abstract public function getLayouts();
    
    /**
    * Retorrne un object datastore
    * 
    * @param string $pstrDatastoreName Datastore name
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    abstract public function getDatastore($pstrDatastoreName);
    
    /**
    * Retorrne la liste des datastores disponible
    * 
    * @return ArrayObject of org_geoprisma_datastore_Datastore
    */
    abstract public function getDatastores();
    
    /**
    * Retorrne un object widget
    * 
    * @param string $pstrWidgetName Widget name
    * 
    * @return org_geoprisma_widget_Widget
    */
    abstract public function getWidget($pstrWidgetName);
    
    /**
    * Retorrne la liste des widget disponible
    * 
    * @return ArrayObject of org_geoprisma_widget_Widget
    */
    abstract public function getWidgets();
    
    /**
    * Retorrne un object accessfilter
    * 
    * @param string $pstrAccessFilterName AccessFilter name
    * 
    * @return org_geoprisma_accessfilter_AccessFilter
    */
    abstract public function getAccessFilter($pstrAccessFilterName);
    
    /**
    * Retorrne la liste des accessfilter disponible
    * 
    * @return ArrayObject of org_geoprisma_accessfilter_AccessFilter
    */
    abstract public function getAccessFilters();
    
    /**
    * Retorrne un object resource
    * 
    * @param string $pstrResourceName Resource name
    * 
    * @return org_geoprisma_resource_Resource
    */
    abstract public function getResource($pstrResourceName);
    
    /**
    * Retorrne la liste des resources disponible
    * 
    * @return ArrayObject of org_geoprisma_resource_Resource
    */
    abstract public function getResources();

    /**
    * Get the process names the config supports.
    * 
    * @return array
    */
    public function getProcessList()
    {
        return $this->m_objArrayProcessList;
    }

    /**
    * Checks if whether the given url is a access controled service or not.
    * 
    * @param string $pstrURL url to check
    *
    * @return boolean
    */
    public function isServiceHost($pstrURL)
    {
        //TODO

        return false;
    }

    /**
    * Checks if whether the given url is a proxy url or not.
    * 
    * @param string $pstrURL url to check
    *
    * @return boolean
    */
    public function isProxyURL($pstrURL)
    {
        $bIsProxyURL = false;
        $objProxyURL = null;

        $objURL = parse_url($pstrURL);
        $objURL['path'] = self::canonicalize($objURL['path']);

        $objArrayProxyURLs = $this->getCanonicalizeProxyURLs();

        // first, check the host
        for ($iIter=0, $iLen=count($objArrayProxyURLs); $iIter<$iLen; $iIter++) 
        {
            if (!isset($objArrayProxyURLs[$iIter]['host'])) 
            {
                if (isset($_SERVER['HTTP_X_FORWARDED_HOST'])) 
                {
                    $objArrayForwarded = explode(
                        ',',
                        $_SERVER['HTTP_X_FORWARDED_HOST']
                    );
                    $objArrayProxyURLs[$iIter]['host'] = $objArrayForwarded[0];
                }
                else
                {
                    $objArrayProxyURLs[$iIter]['host'] = $_SERVER['HTTP_HOST'];
                }
            }
            if ($objArrayProxyURLs[$iIter]['host'] === $objURL['host']) 
            {
                $objProxyURL = $objArrayProxyURLs[$iIter];
                break;
            }
        }

        // then, check the path
        if ($objProxyURL && $objArrayProxyURLs[$iIter]['path'] === $objURL['path']) 
        {
            $bIsProxyURL = true;
        }

        return $bIsProxyURL;
    }

    /**
    * Returns an array of parsed proxy urls.  Throws an error if no proxy would
    * be returned.
    *
    * @return array
    */
    public function getProxyURLs() 
    {
        $objArrayProxyURLs
            = org_geoprisma_SettingImpl::getProxyURLAliasesArray();
        array_push(
            $objArrayProxyURLs,
            org_geoprisma_SettingImpl::getProxyURL()
        );

        for ($iIter=count($objArrayProxyURLs)-1; $iIter>=0; $iIter--) 
        {
            if (!$objArrayProxyURLs[$iIter]) 
            {
                array_splice($objArrayProxyURLs, $iIter, 1);
            }
            else
            {
                $objArrayProxyURLs[$iIter]
                    = parse_url($objArrayProxyURLs[$iIter]);
            }
        }

        if (count($objArrayProxyURLs) == 0) 
        {
            throw new Exception("No proxy URL is set in config.");
        }

        return $objArrayProxyURLs;
    }

    /**
    * Returns an array of canonicalized proxy urls.
    *
    * @return array
    */
    public function getCanonicalizeProxyURLs()
    {
        $objArrayProxyURLs = $this->getProxyURLs();

        for ($iIter=0, $iLen=count($objArrayProxyURLs); $iIter<$iLen; $iIter++) 
        {
            if (strpos($objArrayProxyURLs[$iIter]['path'], "/") !== 0) 
            {
                $objArrayProxyURLs[$iIter]['path'] 
                    = dirname($_SERVER['SCRIPT_NAME'])."/".
                    $objArrayProxyURLs[$iIter]['path'];
            }
            $objArrayProxyURLs[$iIter]['path'] = self::canonicalize(
                $objArrayProxyURLs[$iIter]['path']
            );
        }

        return $objArrayProxyURLs;
    }

    /**
    * Returns a canonicalized version of a path string.
    *
    * @param string $pstrPath path string to canonicalize.
    *
    * @return string
    */
    static public function canonicalize($pstrPath)
    {
        $objArrayPaths = explode('/', $pstrPath);
        $objArrayKeys = array_keys($objArrayPaths, '..');

        foreach ($objArrayKeys AS $iKeyPos => $strKeyElement) 
        {
            array_splice(
                $objArrayPaths,
                $strKeyElement - ($iKeyPos * 2 + 1), 2
            );
        }

        $strPath = implode('/', $objArrayPaths);
        $strPath = str_replace('./', '', $strPath);

        return $strPath;
    }

    /**
    * Return whether the unbound widget (not linked to a resource) is exclusive
    * or not, i.e. its name is inside the exclusive list (if set) and is not
    * a MapPanel widget.
    *
    * @param org_geoprisma_widget_Widget $pobjWidget Widget
    * 
    * @return Boolean
    */
    public function isExclusiveUnboundWidget($pobjWidget)
    {
        $bIsExclusive = false;
        $objArrayNames = $this->m_objArrayExclusiveUnboundWidgetName;
        if (!isset($objArrayNames) || !is_array($objArrayNames) || count($objArrayNames) === 0 ) 
        {
            $bIsExclusive = true;
        }
        else if ($pobjWidget->getType() == "mappanel" ) 
        {
            $bIsExclusive = true;
        }
        else if (array_search($pobjWidget->getName(), $objArrayNames) !== false) 
        {
            $bIsExclusive = true;
        }

        return $bIsExclusive;
    }
}
  
?>
