<?php
/**
* Class Proxy file when request is a get
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Julien-Samuel Lacroix
* @copyright 2010, Mapgears
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Class Proxy file when request is a get
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage FileProxy
* @author     Julien-Samuel Lacroix
*/ 
class org_geoprisma_proxy_file_FileGetProxy extends org_geoprisma_proxy_file_FileProxy
{

    private $m_objArrayAvailableLayers = array();
    private $m_objArrayAvailableResources = array();

    /**
    * Foward the call end return the result
    * 
    * @return void
    */
    public function process()
    {
        $strFilePath = $this->getLayer();

        $objFileTree = new org_geoprisma_proxy_file_FileTree($this->getService()->getSource());

        echo $objFileTree->get($strFilePath, $this->m_objArrayAvailableLayers, $this->m_objArrayAvailableResources);
    }
    
    /**
    * Retourne le nom de l'action réaliser par le proxy (Create - Read - Update - Delete)
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }

    /**
    * Retourne la liste de layer accèder par la query
    * 
    * @return ArrayObject
    */
    private function getInlineLayers()
    {
        $objArrayLayers = array();

        $strFilePath = $this->getLayer();

        $objFileTree = new org_geoprisma_proxy_file_FileTree(
            $this->getService()->getSource()
        );
        $objArrayLayers = $objFileTree->getFileList($strFilePath);

        return new ArrayObject($objArrayLayers);  
    }

    /**
    * Returns an array of org_geoprisma_resource_Resource objects being accessed
    *     by the query.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig The config object
    * @param array                       $pobjRequest The request array that
    *                                                 contains the 'osmservice'
    *                                                 and 'osmresource'
    *                                                 parameters.  Defaults to
    *                                                 $_REQUEST.
    *
    * @return array    
    */
    public function getResourcesFromRequest(&$pobjConfig, $pobjRequest=null)
    {
        $objRequest = ($pobjRequest) ? $pobjRequest : $_REQUEST;

        if (!isset($objRequest['osmservice'])) 
        {
            throw new Exception("osmservice param is missing");
        }

        // In the case of a get, we need to work with all the resources
        // available for this service
        $objArrayResource = Array();
        
        $objService = $this->getService();
        $objConfigResources = $pobjConfig->getResources();
        foreach ($objConfigResources as $objResource)
        {
            if ($objResource->getDatastore($objService) && org_geoprisma_acl_ACLFactory::getACL()->isAuthorized($objResource, $this->getAction())) 
            {
                $objArrayResource[] = $objResource;
            }
        }

        return $objArrayResource;
    }

    /**
    * Validate the 'layers' being accessed by the query.  Each layer must be
    *     present in one of the specified resource datastore using a specified
    *     service.
    * 
    * @param org_geoprisma_service_Service $pobjService        The service to
    *                                                          use for 
    *                                                          validation.
    *                                                          Defaults to
    *                                                          the proxy
    *                                                          service.
    * @param ArrayObject                   $pobjArrayResources ArrayObject of
    *                                                          resource objects.
    *                                                          Defaults to the
    *                                                          proxy resources.
    * @param ArrayObject                   $pobjArrayLayers    ArrayObject of
    *                                                          layers to
    *                                                          validate.
    *                                                          Defaults to the
    *                                                          layers returned
    *                                                          by the proxy
    *                                                          'getLayers'
    *                                                          method. 
    *
    * @return void    
    */
    public function validateLayersFromRequest(
        $pobjService=null,
        $pobjArrayResources=null,
        $pobjArrayLayers=null
    ) {
        $objService = ($pobjService === null) ? $this->getService() : $pobjService;
        $objArrayResources = ($pobjArrayResources === null) ? $this->getResources() : $pobjArrayResources;
        $objArrayLayers = ($pobjArrayLayers === null) ? $this->getInlineLayers() : $pobjArrayLayers;
        $strFilePath = $this->getLayer();
        if (substr($strFilePath, -1) != '/' && strlen($strFilePath) > 0) 
        {
            $strFilePath .= '/';
        }

        $bAllAuthorized = true;
        foreach ($objArrayLayers as $strLayer) 
        {
            $bIsAuthorized = false;
            foreach ($objArrayResources as $objResource) 
            {
                $objDatastore = $objResource->getDatastore($objService);
                if (self::isAuthorizedLayer($objDatastore, $strFilePath.$strLayer)) 
                {
                    $this->m_objArrayAvailableLayers[] = $strLayer;
                    $this->m_objArrayAvailableResources[$strLayer] = $objResource->getName();
                    break;
                }
            }
        }

        if (count($this->m_objArrayAvailableLayers) <= 0) 
        {
            throw new Exception('Not Authorized by Datastore layer');    
        }
    }


    /**
    * Validate if the layer is accessible in this datastore.
    * 
    * @param org_geoprisma_datastore_Datastore $pobjDatastore Datastore
    * @param String                            $pstrLayer     Layer name
    *
    * @return Boolean
    */
    private static function isAuthorizedLayer($pobjDatastore, $pstrLayer)
    {
        $objArrayDSLayers = $pobjDatastore->getLayers();

        $bAuthorized = false;
        foreach ($objArrayDSLayers as $strDSLayer)
        {
            if (substr($strDSLayer, -1) == '/') 
            {
                $strDSLayer = substr($strDSLayer, 0, -1);
            }

            $objArrayDSPath = explode('/', $strDSLayer);
            $objArrayLayerPath = explode('/', $pstrLayer);
            $iNumDSPathNode = count($objArrayDSPath);
            $iNumLayerPathNode = count($objArrayLayerPath);
            $iNumPathNode = min($iNumDSPathNode, $iNumLayerPathNode);
            for ($iIter=0; $iIter<$iNumPathNode; $iIter++)
            {
                if ($objArrayDSPath[$iIter] != $objArrayLayerPath[$iIter]) 
                {
                    break;
                }

                if (($iIter+1) >= $iNumPathNode) 
                {
                    $bAuthorized = true;
                }
            }

            if ($bAuthorized) 
            {
                break;
            }
        }

        return $bAuthorized;
    }
}

?>
