<?php
/**
* Proxy of MapFishPrint type
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube
* @copyright 2010, Centre de Geomatique du Quebec
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Proxy of MapFishPrint type
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Alexadnre Dube
*/     
class org_geoprisma_proxy_mapfishprint_MapFishPrintProxy extends org_geoprisma_proxy_Proxy
{
    /**
    * @var Array
    */
    private $m_objArraySpec = null;

    /**
    * @var Array
    */
    private $m_objArraySpecLayers = null;

    /**
    * Get the action this proxy does.
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }

    /**
    * Forward the call end return the result
    * 
    * @return void
    */
    public function process()
    {
        // get the print capabilities to have the 'createURL'
        $objCapabilities
            = json_decode($this->getService()->getCapabilities(), true);

        // get the params of the 'spec' with original 'source' values of the
        // services
        $strParams = stripslashes(urldecode(json_encode($this->getSourceSpec())));

        ob_start();
        $objCurl = curl_init();
        curl_setopt($objCurl, CURLOPT_URL, $objCapabilities['createURL']);
        curl_setopt($objCurl, CURLOPT_POST, 1);
        curl_setopt($objCurl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($objCurl, CURLOPT_POSTFIELDS, $strParams);
        curl_setopt($objCurl, CURLOPT_HTTPHEADER, array('Content-Type: text/plain')); 
        $strResult = curl_exec($objCurl);
        $objResult = json_decode($strResult);

        $objArrayInfo = curl_getinfo($objCurl);

        if ($objResult == null || !isset($objResult->getURL)) 
        {
            $objResult = $strResult;
        }
        else
        {
            $objResult = $objResult->getURL;
        }

        curl_close($objCurl);

        if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) 
        {
            if (!session_id()) 
            {
                session_start();
            }

            $strID = uniqid();

            $_SESSION[$strID] = array(
                "resources" => array(),
                "url" => $objResult
            );

            foreach ($this->getResources() as $iIndex => $objResource) 
            {
                array_push(
                    $_SESSION[$strID]['resources'],
                    $objResource->getName()
                );
            }

            $strPrintURL = $this->getService()->urlAppend(
                org_geoprisma_SettingImpl::getProxyURL(),
                array(
                    "osmservice" => $this->getService()->getName(),
                    "osmfile" => $strID
                )
            );

            echo json_encode(array('getURL' => $strPrintURL));
        }
        else
        {
            header("Content-type: application/force-download");
            header("Content-Transfer-Encoding: Binary");
            header('Content-Disposition: attachment; filename="map.pdf"');

            readfile($objResult);
        }

        ob_end_flush();
    }

    // --------------------------
    // Getter
    // --------------------------
    
    /**
    * Not used by this proxy.
    * 
    * @return void
    */
    public function getLayers()
    {
        return;
    }

    /**
    * Returns the spec layers array
    * 
    * @return array
    */
    public function getSpecLayers() 
    {
        return $this->m_objArraySpecLayers;
    }

    // --------------------------
    // Setter
    // --------------------------

    /**
    * Set the spec layers array property
    *
    * @param array $pobjArraySpecLayers The 'spec' layers array
    *
    * @return void
    */
    private function setSpecLayers($pobjArraySpecLayers) 
    {
        $this->m_objArraySpecLayers = $pobjArraySpecLayers;
    }

    // --------------------------
    // Other methods
    // --------------------------

    /**
    * Get the spec array converted from the $_REQUEST['spec'] parameter if
    * request is GET or from 'php://input' if POST.
    *
    * @return array
    */
    public function getSpec()
    {
        if (!$this->m_objArraySpec) 
        {
            if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) 
            {
                $strPostRequest = file_get_contents('php://input');
                $this->m_objArraySpec = json_decode($strPostRequest, true);
            }
            else
            {
                if (!isset($_REQUEST['spec'])) 
                {
                    throw new Exception("spec param is missing");
                }

                $this->m_objArraySpec = json_decode($_REQUEST['spec'], true);
            }
        }

        return $this->m_objArraySpec;
    }

    /**
    * Get the resources from the 'spec' object.  At the same time, get all 
    *     the requested 'layers' as well and the resources according 'services'
    *     that are going to be used to render the images to print.
    *
    * @param org_geoprisma_config_Config &$pobjConfig The config object.
    *
    * @return array of org_geoprisma_resource_Resource objects
    */
    public function getResourcesFromRequest(&$pobjConfig)
    {
        $objSpec = $this->getSpec();
        $objArrayResources = Array();
        $objArraySpecLayers = Array();

        foreach ($objSpec['layers'] as $objLayer) 
        {
            if (!isset($objLayer['baseURL']) && $objLayer['type'] == "Vector") 
            {
                continue;
            }

            if (!$pobjConfig->isProxyURL($objLayer['baseURL'])) 
            {
                continue;
            }

            $objURL = parse_url($objLayer['baseURL']);

            parse_str($objURL['query']);

            if (!isset($osmresource)) 
            {
                throw new Exception("osmresource param is missing");
            }

            if (!isset($osmservice)) 
            {
                throw new Exception("osmservice param is missing");
            }

            $objArraySubResources = parent::getResourcesFromRequest(
                $pobjConfig,
                array(
                    'osmresource' => $osmresource,
                    'osmservice' => $osmservice
                )
            );

            // keep track of the renderer service to use for each resource
            foreach ($objArraySubResources as $objSubResource) 
            {
                $objSubResource->m_objRendererService
                    = $pobjConfig->getService($osmservice);
            }

            // for each 'layers' of this layer, keep track of each mentioned
            // resource for later validation
            if (isset($objLayer['layers'])) 
            {
                foreach ($objLayer['layers'] as $strLayer) 
                {
                    array_push(
                        $objArraySpecLayers,
                        array(
                            'layer' => $strLayer,
                            'resources' => $objArraySubResources
                        )
                    );
                }
            }
            else if (isset($objLayer['layer'])) 
            {
                array_push(
                    $objArraySpecLayers,
                    array(
                        'layer' => $objLayer['layer'],
                        'resources' => $objArraySubResources
                    )
                );
            }
            else 
            {
                throw new Exception("The layer had no 'layer' or 'layers' properties defining which layer to print.");
            }

            $objArrayResources = array_merge(
                $objArrayResources, $objArraySubResources
            );

            unset($osmresource);
            unset($osmservice);
        }

        $this->setSpecLayers($objArraySpecLayers);

        return $objArrayResources;
    }

    /**
    * Validate the resources from the request using the parent method of the
    *     same name.
    *
    * @return void
    */
    public function validateResourcesFromRequest() 
    {
        $objArrayResources = $this->getResources();

        foreach ($objArrayResources as $objResource) 
        {
            parent::validateResourcesFromRequest(
                $objResource->m_objRendererService,
                Array($objResource)
            );
        }
    }

    /**
    * Validate the layers from the request using the parent method of the
    *     same name.
    *
    * @return void
    */
    public function validateLayersFromRequest()
    {
        $objArraySpecLayers = $this->getSpecLayers();

        foreach ($objArraySpecLayers as $objSpecLayer) 
        {
            // resources that are grouped together always share the same
            // service so it is safe to get the first one since they are all
            // the same.
            $objRendererService 
                = $objSpecLayer['resources'][0]->m_objRendererService;

            parent::validateLayersFromRequest(
                $objRendererService,
                $objSpecLayer['resources'],
                Array($objSpecLayer['layer'])
            );
        }
    }

    /**
    * Get the 'spec' object with all 'baseURL' changed to point to the according
    *     service source url instead of the geoprisma proxy url.
    *
    * @return array
    */
    public function getSourceSpec() 
    {
        $objSpec = $this->getSpec();
        $objConfig = org_geoprisma_config_ConfigFactory::getConfig(); 

        for ($iIter=0, $iLen=count($objSpec['layers']); $iIter<$iLen; $iIter++) 
        {
            $objLayer = $objSpec['layers'][$iIter];

            if (!isset($objLayer['baseURL']) && $objLayer['type'] == "Vector") 
            {
                continue;
            }

            if (!$objConfig->isProxyURL($objLayer['baseURL'])) 
            {
                continue;
            }

            $objURL = parse_url($objLayer['baseURL']);

            parse_str($objURL['query']);
            $objSpec['layers'][$iIter]['baseURL']
                = $objConfig->getService($osmservice)->getSource();
            unset($osmresource);
            unset($osmservice);
        }

        return $objSpec;
    }
} 

?>
