<?php
/**
* Class Proxy de type tilecache
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Class Proxy de type tilecache
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Pascal Martin 
*/     
class org_geoprisma_proxy_tilecache_TileCacheProxy extends org_geoprisma_proxy_Proxy
{
    /**
    * Retourne le nom de l'action réaliser par le proxy (Create - Read - Update - Delete)
    *                                   
    * @return string
    */
    public function getAction()
    {
        return self::CRUD_READ;    
    }

    /**
    * Foward the call end return the result
    * 
    * @return void
    */
    public function process()
    {
        ob_start();

        if (isset($_GET['staticCache'])) 
        {
            $objCurl = curl_init(urldecode(str_replace("?staticCache=", "", $this->addParam($this->getService()->getSource()))));
        }
        else
        {
            $objCurl = curl_init($this->addParam($this->getService()->getSource()));
        }

        curl_exec($objCurl);
        $objArrayInfo = curl_getinfo($objCurl);
        curl_close($objCurl);

        header("Content-type: $objArrayInfo[content_type]");
        if ($this->getCaching() === true) 
        {
            header("Pragma: public");
            header("Cache-Control: maxage=".$this->getExpires());
            header('Expires: ' . gmdate('D, d M Y H:i:s', time()+$this->getExpires()) . ' GMT');
        }

        ob_end_flush();
    }
    
    /**
    * Retourne la liste de layer accèder par la query
    * 
    * @return ArrayObject
    */
    public function getLayers()
    {
        $objArrayLayer = new ArrayObject();
        $bLayersFound = false;
        foreach ($_GET as $strKey => $strValue) 
        {
            if (strtoupper($strKey) == "LAYERS") 
            {
                $bLayersFound = true;
                break;
            }
        }   
        if ($bLayersFound && isset($_GET[$strKey])) 
        {
            $objArrayLayer = new ArrayObject(explode(',', $_GET[$strKey]));   
        }
        // This is to support TileCache StaticCacheOption
        else if (isset($_GET['staticCache'])) 
        {
            $objArrayPathInfo = explode('/', $_GET['staticCache']);
            $objArrayLayer->append($objArrayPathInfo[1]);
        }

        return $objArrayLayer;         
    }

    /**
    * Return true if caching is enabled
    *
    * @return boolean
    */
    public function getCaching()
    {
        return org_geoprisma_SettingImpl::getTileCacheProxyCaching();
    }

    /**
    * Return expires
    *
    * @return integer
    */
    public function getExpires()
    {
        return org_geoprisma_SettingImpl::getTileCacheProxyCachingExpires();
    }
}
  
?>
