<?php
/**
* Service de type mapserver     
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Service de type mapserver 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/   
class org_geoprisma_service_MapServerService extends org_geoprisma_service_Service
{
    const MAPSERVER_SERVICE = 'mapserver';
    const TYPE = 'mapserver';
    const TYPE_ID = 6;

    private $m_strMapFile = null;
    
    /**
    * Construit un instance de org_geoprisma_service_MapServerService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_MapServerService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];
        
        $this->m_strMapFile = (string)$pobjArray['mapfile'];
        
        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }
    
    /**
    * Return the mapfile
    * 
    * @return string 
    */
    public function getMapFile()
    {
        return $this->m_strMapFile;
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        return array();
    }
    
    /**
    *  Ajout le service courant (this) dans le xml     
    * 
    * @param DOMElement  &$pobjDomElementServices Noeud xml ou le service doit être ajouter
    * @param DOMDocument &$pobjDomDocument        Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::MAPSERVER_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementMapFile = $pobjDomDocument->createElement('mapfile', $this->getMapFile());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 
        $objDomElementService->appendChild($objDomElementMapFile);

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }
}
  
?>
